/////////////////////////////////////////////////////////////////
//
// Arxiu      : BRASView.cpp
//
// Descripci : Arxiu d'implementaci de la classe de gesti de
//				la vista (prpia de l'estructura generada per
//				l'AppWizard del MSVC++, en l'esquema
//				Document/Vista).
//				S'hi han afegit les segents funcionalitats:
//				* Omplir l'rea de la finestra amb el color
//				  d'overlay assignat a la cmara. (veure
//				  descripci de la cmara per a ms informaci)
//				* Superposar a la imatge obtinguda a travs de
//				  la cmara els eixos o els punts de referncia
//				  per tal de permetre a l'usuari visualitzar
//				  les dades sobre la imatge captada.
//				* Mostrar un rectangle al voltant de l'rea
//				  de treball, aix com l'rea que s'ha d'examinar
//				  pels objectes de la classe CPriAnalisiMoviment
//				* Permetre la captura dels punts necessaris per
//				  a la calibraci de l'entorn de treball i per
//				  al clcul de la velocitat de la cinta.
//				* Permetre la captura dels punts necessaris per
//				  a la definici de la posici i tamany de 
//				  l'rea de treball.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"

#include "BRASDoc.h"
#include "BRASView.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

IMPLEMENT_DYNCREATE(CBRASView, CView)

BEGIN_MESSAGE_MAP(CBRASView, CView)
	//{{AFX_MSG_MAP(CBRASView)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_KILLFOCUS()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//
// Constructor.
// m_colorFons ha de contenir el color del qual s'ha d'omplir
// l'interior de la finestra. Aquest color est definit en 
// el mdul de la cmara, i s aquell que empra la cmara
// per a realitzar l'overlay.
//
CBRASView::CBRASView()
{
	m_colorFons=RGB(CAM_R_OVERLAY, CAM_G_OVERLAY, CAM_B_OVERLAY);
	m_dibuixarPunts=false;
	m_dibuixarOrientacio=false;
	m_puntCapturat=3;
	m_puntCapturat2=0;
}
//
// Destructor
//
CBRASView::~CBRASView()
{
}

//
// Funcions prpies de Windows i de les MFC, creades
// per l'Application Wizard del MSVC++5.0
//
BOOL CBRASView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CView::PreCreateWindow(cs);
}

#ifdef _DEBUG
void CBRASView::AssertValid() const
{
	CView::AssertValid();
}

void CBRASView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CBRASDoc* CBRASView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CBRASDoc)));
	return (CBRASDoc*)m_pDocument;
}
#endif //_DEBUG

//
// La segent funci es cridada per l'entorn quan es genera
// un missatge de windows WM_DRAW, s a dir, quan la finestra
// necessita ser repintada.
// Per tant, s aqu on s'ha de pintar el fons de la finestra
// del color desitjat (m_colorFons). I tamb s aqu on
// s'han de dibuixar els elements que volguem superposar a la
// imatge capturada per la cmara: es superposaran tots
// aquells que tenguin un color distint a m_colorFons.
//
void CBRASView::OnDraw(CDC* pDC) 
{
	CRect rectangleFinestra;
	CPen elVerd(PS_SOLID, 2, RGB(0,255,0));
	CPen elVermell(PS_SOLID, 3, RGB(255, 0, 0));
	CPen elBlau(PS_SOLID, 3, RGB(0, 0, 255));

	CBRASDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);

	// A continuaci s'omple la finestra d'un sol color 
	// per tal de realitzar l'overlay descrit al
	// manual de la cmara.
	// S'obt la posici i dimensions de la finestra,
	// es crea un objecte Brush del color m_colorFons
	// i s pinta tot el fons de la finestra d'aquest color.
	GetClientRect(rectangleFinestra);
	CBrush brColorFons(m_colorFons);
    pDC->FillRect(&rectangleFinestra, &brColorFons);

	// A continuaci es dibuixen els elements que volem superposar
	// Si es vol destacar un punt de la finestra, es posa
	// m_dibuixarPunts a true i les coordenades del punt en
	// m_x i m_y. El punt es ressaltar en color vermell.
	if (m_dibuixarPunts) {
		pDC->FillRect(CRect(m_x-5, m_y-5, m_x+5, m_y+5), &CBrush(RGB(255, 0, 0)));
	}

	// Si es vol mostrar l'eix de coordenades propi d'un objecte,
	// aix com el vector d'agafada, es posa m_dibuixarOrientacio
	// a true, en m_xc i m_yc les coordenades (en el sistema
	// de la pantalla) de l'origen de coordenades del nou sistema,
	// i en m_xmn i m_ymn les coordenades (en el sistema de 
	// pantalla) del punt del permetre que est a una distncia 
	// mnima del centre de gravetat de l'objecte. A partir
	// d'aquests dos punts es determinem els eixos X i Y del
	// nou sistema de coordenades. 
	// En m_xo, m_yo, m_xa, m_ya es posen les coordenades del
	// vector d'agafada de l'objecte, que es dibuixar en vermell.
	if (m_dibuixarOrientacio) {
		pDC->SelectObject(&elVerd);
		pDC->MoveTo(m_xc, m_yc);
		pDC->SelectObject(&elBlau);
		pDC->LineTo(m_xmn, m_ymn);
		pDC->SelectObject(&elVerd);
		pDC->MoveTo(m_xc, m_yc);
		pDC->LineTo(-m_ymn + m_yc + m_xc, m_xmn - m_xc + m_yc);
		pDC->SelectObject(&elVermell);
		pDC->MoveTo(m_xo, m_yo);
		pDC->LineTo(m_xa, m_ya);
	}
	// A continuaci, si est configurat perqu es faci, es
	// mostrar un rectangle al voltant de l'rea de treball.
	if (m_sistema.m_genconfig.m_mostrarFinestra) {
		pDC->FrameRect(CRect(m_sistema.m_imgconfig.m_xFinestra
						,m_sistema.m_imgconfig.m_yFinestra
						,(m_sistema.m_imgconfig.m_xFinestra)+(m_sistema.m_imgconfig.m_ampleFinestra)
						,(m_sistema.m_imgconfig.m_yFinestra)+(m_sistema.m_imgconfig.m_altFinestra)), 
						new CBrush(RGB(255, 255, 255)));
		pDC->SelectObject(&elVerd);
		pDC->MoveTo(((m_sistema.m_imgconfig.m_xFinestra)+(m_sistema.m_imgconfig.m_ampleFinestra)-(m_sistema.m_cellconfig.m_amplada)), m_sistema.m_imgconfig.m_yFinestra);
		pDC->LineTo(((m_sistema.m_imgconfig.m_xFinestra)+(m_sistema.m_imgconfig.m_ampleFinestra)-(m_sistema.m_cellconfig.m_amplada)), ((m_sistema.m_imgconfig.m_yFinestra)+(m_sistema.m_imgconfig.m_altFinestra)-1));

	}
}
	
// La segent funci respon a l'event de click al bot esquerre
// del ratol. En que es fa en aquest cas depen del valor
// de m_puntCapturat.
void CBRASView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	int xtmp=point.x;
	int ytmp=point.y;
	double stuff;
	switch (m_puntCapturat) {
	// Els casos '0' i '1' sn per a la captura de punts
	// en la calibraci del sistema. 
	case 0:
		m_puntCapturat=1;
		m_sistema.m_cellconfig.m_xp1=point.x;
		m_sistema.m_cellconfig.m_yp1=point.y;
		break;
	case 1:
		m_puntCapturat=3;
		m_sistema.m_cellconfig.m_xp2=point.x;
		m_sistema.m_cellconfig.m_yp2=point.y;
		AfxMessageBox("Procs de calibraci finalitzat amb xit");
		break;
	// El cas '3' s quan no s'ha de fer res en especial.
	case 3:
		break;
	// Els casos '4' i '5' sn per al clcul de la velocitat
	// de la cinta..
	case 4:
		m_sistema.pantallaARobot(&xtmp, &ytmp, &stuff);
		m_sistema.m_camara.aturarGravacio();
		m_sistema.m_camara.llegirDeMemoria(m_img2);
		m_puntCapturat=5;
		m_sistema.m_cellconfig.m_xInicial=(float)xtmp;
		m_sistema.m_cellconfig.m_yInicial=(float)ytmp;
		break;
	case 5:
		m_sistema.pantallaARobot(&xtmp, &ytmp, &stuff);
		m_puntCapturat=3;
		m_sistema.m_cellconfig.m_xFinal=(float)xtmp;
		m_sistema.m_cellconfig.m_yFinal=(float)ytmp;
		m_sistema.m_cellconfig.m_vx=(float)((m_sistema.m_cellconfig.m_xFinal-m_sistema.m_cellconfig.m_xInicial)/m_sistema.m_cellconfig.m_temps);
		m_sistema.m_cellconfig.m_vy=(float)((m_sistema.m_cellconfig.m_yFinal-m_sistema.m_cellconfig.m_yInicial)/m_sistema.m_cellconfig.m_temps);
		AfxMessageBox("Procs de clcul de la velocitat de la cinta finalitzat satisfactriament");
		delete m_img1;
		delete m_img2;

		break;
	}
	CView::OnLButtonDown(nFlags, point);
}

//
// Aquesta funci forma part del procs de clcul de la velocitat
// de la cinta. Una vegada s'han capturat les dues imatges, 
// es crida aquesta funci, la qual indica a l'usuari que faci
// click sobre els centre de l'objecte en els dos casos.
//
void CBRASView::finalitzarObtencioVelocitat(CPriImatge *imatge, CPriImatge *imatge2, float temps)
{
	m_puntCapturat=4;
	m_img1=imatge;
	m_img2=imatge2;
	m_sistema.m_cellconfig.m_temps=temps;
	AfxMessageBox("Ara heu de fer click sobre el centre de l'objecte a la primera imatge.\nDesprs feis el mateix per a la segona imatge");
	m_sistema.m_camara.aturarGravacio();
	m_sistema.m_camara.llegirDeMemoria(m_img1);
}

//
// Aquesta funci respon a l'event de click amb el bot dret
// del ratol.
// S'empra aquesta funcionalitat per a capturar els punts
// que configuren l'rea de treball.
//
void CBRASView::OnRButtonDown(UINT nFlags, CPoint point) 
{
	int tmp;
	switch (m_puntCapturat2) {
	// Es captura el primer punt
	case 0:
		m_puntCapturat2=1;
		m_top=point.y;
		m_left=point.x;
		break;
	// Es captura el segon punt.
	case 1:
		m_right=point.x;
		m_bottom=point.y;
		m_puntCapturat2=0;
		// En el cas que l'usuari hagi fet click en l'ordre
		// invers (bottom-right, top-left) en comptes de
		// (top-left, bottom-right), s'intercanvien els valors
		// dels punts.
		if (m_right<m_left) {
			tmp=m_right;
			m_right=m_left;
			m_left=tmp;
		}
		if (m_bottom<m_top) {
			tmp=m_bottom;
			m_bottom=m_top;
			m_top=tmp;
		}
		// Es comprova si les mides de la finestra estan
		// dins dels lmits, i ho ajusta si no s aix.
		if (m_left<5) m_left=5;
		if (m_right>IMG_AMPLEI_DEFECTE-5) m_right=IMG_AMPLEI_DEFECTE-5;
		if (m_top<5) m_top=5;
		if (m_bottom>IMG_ALTI_DEFECTE-5) m_bottom=IMG_ALTI_DEFECTE-5;
		// S'assigna a l'objecte de configuraci del processament
		// de la imatge les coordenades de la finestra, i
		// a continuaci s'envia a la finestra un missatge per
		// a que es repinti.
		m_sistema.m_imgconfig.m_xFinestra=m_left;
		m_sistema.m_imgconfig.m_yFinestra=m_top;
		m_sistema.m_imgconfig.m_ampleFinestra=(m_right-m_left);
		m_sistema.m_imgconfig.m_altFinestra=(m_bottom-m_top);
		m_sistema.m_imgconfig.UpdateData(false);
		RedrawWindow();
		break;
	}
	CView::OnRButtonDown(nFlags, point);
}

// Aquesta funci es cridada quan la finestra perd el focus.
// S'ha decidit que si es perd el focus (p.ex., es passa a una
// altra aplicaci o s'activa algun quadre de dileg o finestra
// de configuraci), i s'havia introduit el primer punt
// de definici de l'rea de treball, aquest ja no s vlid
// i s'ha de tornar a introduir.
void CBRASView::OnKillFocus(CWnd* pNewWnd) 
{
	CView::OnKillFocus(pNewWnd);
	
	m_puntCapturat2=0;
	
}
