/////////////////////////////////////////////////////////////////
//
// Arxiu      : BRASView.h
//
// Descripci : Arxiu de capalera de la classe de gesti de
//				la vista (prpia de l'estructura generada per
//				l'AppWizard del MSVC++, en l'esquema
//				Document/Vista).
//				S'hi han afegit les segents funcionalitats:
//				* Omplir l'rea de la finestra amb el color
//				  d'overlay assignat a la cmara. (veure
//				  descripci de la cmara per a ms informaci)
//				* Superposar a la imatge obtinguda a travs de
//				  la cmara els eixos o els punts de referncia
//				  per tal de permetre a l'usuari visualitzar
//				  les dades sobre la imatge captada.
//				* Mostrar un rectangle al voltant de l'rea
//				  de treball, aix com l'rea que s'ha d'examinar
//				  pels objectes de la classe CPriAnalisiMoviment
//				* Permetre la captura dels punts necessaris per
//				  a la calibraci de l'entorn de treball i per
//				  al clcul de la velocitat de la cinta.
//				* Permetre la captura dels punts necessaris per
//				  a la definici de la posici i tamany de 
//				  l'rea de treball.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#if !defined(AFX_BRASVIEW_H__D36FE610_EA07_11D3_B5CA_00C026C084F2__INCLUDED_)
#define AFX_BRASVIEW_H__D36FE610_EA07_11D3_B5CA_00C026C084F2__INCLUDED_

//
// Arxius de capalera
//
#include "Sistema.h"
#include "PriImatge.h"
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class CBRASView : public CView
{
protected:
	// Constructor
	CBRASView();
	DECLARE_DYNCREATE(CBRASView)

public:
	CBRASDoc* GetDocument();

public:

// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CBRASView)
	public:
	virtual BOOL PreCreateWindow(CREATESTRUCT& cs);
	protected:
	virtual void OnDraw(CDC* pDC);
	//}}AFX_VIRTUAL

public:
	// El Sistema. Aquesta variable s l'nica de la classe
	// CSistema declarada en tota l'aplicaci. El sistema
	// cont tots els elements responsables del funcionament
	// de l'aplicaci, aix com les funcions d'interacci entre
	// ells.
	CSistema m_sistema;
	// Degut a la forma en que els events sn capturats i
	// passats entre els distints objectes d'una estructura
	// document/vista, la part del procs de clcul de velocitat
	// de la cinta que implica clicar sobre els objectes
	// visualitzats en pantalla s'ha de fer des d'aquesta
	// classe. Per a realitzar aquesta tasca tenim la funci
	// finalitzarObtencioVelocitat i la variable membre
	// m_puntCapturat..
	// * funci finalitzarObtencioVelocitat : Una vegada s'han
	//   capturat les imatges necessries per a l'obtenci de
	//   la velocitat, es crida aquesta funci per a que
	//   mostri el missatge que indica qu s'ha de fer, i
	//   s'inicia el procs de captura dels punts.
	// * variable m_puntCapturat : Aquesta classe permet la
	//	 captura de punts per a calibraci de l'entorn de treball
	//	 i clcul de la velocitat de la cinta. La variable
	//	 m_puntCapturat permet mantenir l'estat de si s'est
	//	 esperant que s'introdueixi algun punt, i, en cas
	//	 afirmatiu, quin dels punts s'est esperant.
	void finalitzarObtencioVelocitat(CPriImatge *imatge, CPriImatge *imatge2, float temps);
	int m_puntCapturat;
	// Dins de la vista tamb ens encarregam de superposar 
	// distintes dades a la imatge capturada per la cmara.
	// Aix inclou: el punt del centre de l'objecte, l'eix de
	// coordenades de cada objecte i el vector d'agafada del
	// mateix.
	// Els sencers declarats a continuaci contenen les coordenades
	// dels elements que s'han de dibuixar, mentres que les
	// variables booleanes determinen qu s el	que s'ha de
	// dibuixar.
	int m_xo, m_yo, m_xc, m_yc, m_xmn, m_ymn, m_xa, m_ya, m_x, m_y;
	bool m_dibuixarPunts, m_dibuixarOrientacio;
	// Per tal de simular que la imatge capturada per la cmara
	// est en una finestra es fa servir una tcnica d'overlay.
	// La cmara permet que la imatge capturada es mostri sols
	// en aquelles zones de la pantalla que sn d'un determinat
	// color. S'ha elegit una tonalitat de magenta, color poc
	// freqent. 
	// El que es fa s pintar el fons de la finestra d'aquest color
	// i aix la imatge capturada per la cmara s'hi dibuixar
	// a dins.
	COLORREF m_colorFons;
	virtual ~CBRASView();
#ifdef _DEBUG
	virtual void AssertValid() const;
	virtual void Dump(CDumpContext& dc) const;
#endif

protected:

// Generated message map functions
protected:
	//{{AFX_MSG(CBRASView)
	afx_msg void OnLButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnRButtonDown(UINT nFlags, CPoint point);
	afx_msg void OnKillFocus(CWnd* pNewWnd);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	// Es possible definir la posici i tamany de l'area
	// de treball fent click amb el bot dret del ratol
	// sobre la finestra. Les variables segents sn
	// les que s'encarreguen de determinar quin dels
	// punts de la finestra estam esperant capturar, i 
	// els valors dels punts que ja s'han capturat.
	int m_puntCapturat2;
	int m_top;
	int m_left;
	int m_right;
	int m_bottom;
	// Punters a les dues imatges capturades en el procs
	// de clcul de la velocitat de la cinta.
	CPriImatge *m_img1;
	CPriImatge *m_img2;
};

#ifndef _DEBUG  // debug version in BRASView.cpp
inline CBRASDoc* CBRASView::GetDocument()
   { return (CBRASDoc*)m_pDocument; }
#endif

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_BRASVIEW_H__D36FE610_EA07_11D3_B5CA_00C026C084F2__INCLUDED_)
