/////////////////////////////////////////////////////////////////
//
// Arxiu      : DlgCfgCriteris.cpp
//
// Descripci : Arxiu d'implementaci de la classe 
//				d'obtenci de criteris.
//				Aquesta classe permet, mitjanant un quadre
//				de dileg, observar objectes i extreure'n
//				la regla general que els defineix.
//				El seu funcionament permet:
//				* Observar objectes i extreure'n les
//				  propietats.
//				* Crear classes d'objectes, i assignar
//				  manualment cada objecte especfic
//				  a una de les classes.
//				* Definir els lmits que permeten incloure
//				  un objecte en una o altra classe.
//				* Classificaci automtica dels objectes en
//				  classes a partir d'uns criter
//				* Obtenci dels criteris que defineixen un
//				  objecte a partir d'observacions concretes
//				  de l'objecte.
//				* Emmagatzemar en el format .dob les dades
//				  obtingudes.
//				Es distingeixen:
//				* Objectes especfics, captats per la cmara.
//				  La seva informaci s'emmagatzema en objectes
//				  de la classe CAuxPropietatObjecte.
//				* Classes. Una classe t un nom (p.ex. TRIANG) i
//				  una llista d'objectes especfics associats.
//				* Objectes generats. Un objecte generat t
//				  un nom i les caracterstiques que el
//				  defineixen. Un objecte generat difereix de
//				  l'objecte emprat per l'identificaci en que
//				  aquest darrer t una llista de criteris, 
//				  cada un dels quals s de la forma
//				  atribut comparador expressi. Per contra,
//				  un criteri d'un objecte generat s ms simple,
//				  i sols pot esser atribut < valor1 i
//				  atribut > valor2.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "DlgCfgCriteris.h"
#include "BRASDoc.h"
#include "BRASView.h"
#include "Sistema.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CDlgCfgCriteris::CDlgCfgCriteris(CWnd* pParent /*=NULL*/)
	: CDialog(CDlgCfgCriteris::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgCfgCriteris)
	m_usarArea = FALSE;
	m_usarCompacitat = FALSE;
	m_usarDMax = FALSE;
	m_usarDMin = FALSE;
	m_usarPerimetre = FALSE;
	m_margeArea = 0.0f;
	m_margeCompacitat = 0.0f;
	m_margeDMax = 0.0f;
	m_margeDMin = 0.0f;
	m_margePerimetre = 0.0f;
	m_nomClasse = _T("");
	m_usarDMaxMin = FALSE;
	m_margeDMaxMin = 0.0f;
	m_distanciaArea = 0.0f;
	m_distanciaCompacitat = 0.0f;
	m_distanciaDMax = 0.0f;
	m_distanciaDMaxMin = 0.0f;
	m_distanciaDMin = 0.0f;
	m_distanciaPerimetre = 0.0f;
	//}}AFX_DATA_INIT
}

void CDlgCfgCriteris::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgCfgCriteris)
	DDX_Control(pDX, IDC_LLISTA_BITACORA, m_llistaBitacora);
	DDX_Control(pDX, IDC_LLISTA_NOCLASSIFICATS, m_llistaNoClassificat);
	DDX_Control(pDX, IDC_LLISTA_GENERAT, m_llistaGenerat);
	DDX_Control(pDX, IDC_ARBRE_CLASSIFICATS, m_arbreClassificats);
	DDX_Check(pDX, IDC_CHECK_USAR_AREA, m_usarArea);
	DDX_Check(pDX, IDC_CHECK_USAR_COMPACITAT, m_usarCompacitat);
	DDX_Check(pDX, IDC_CHECK_USAR_DMAX, m_usarDMax);
	DDX_Check(pDX, IDC_CHECK_USAR_DMIN, m_usarDMin);
	DDX_Check(pDX, IDC_CHECK_USAR_PERIMETRE, m_usarPerimetre);
	DDX_Text(pDX, IDC_EDIT_MARGE_AREA, m_margeArea);
	DDX_Text(pDX, IDC_EDIT_MARGE_COMPACITAT, m_margeCompacitat);
	DDX_Text(pDX, IDC_EDIT_MARGE_DMAX, m_margeDMax);
	DDX_Text(pDX, IDC_EDIT_MARGE_DMIN, m_margeDMin);
	DDX_Text(pDX, IDC_EDIT_MARGE_PERIMETRE, m_margePerimetre);
	DDX_Text(pDX, IDC_EDIT_NOM_CLASSE, m_nomClasse);
	DDX_Check(pDX, IDC_CHECK_USAR_DMAXMIN, m_usarDMaxMin);
	DDX_Text(pDX, IDC_EDIT_MARGE_DMAXMIN, m_margeDMaxMin);
	DDX_Text(pDX, IDC_EDIT_DISTANCIA_AREA, m_distanciaArea);
	DDX_Text(pDX, IDC_EDIT_DISTANCIA_COMPACITAT, m_distanciaCompacitat);
	DDX_Text(pDX, IDC_EDIT_DISTANCIA_DMAX, m_distanciaDMax);
	DDX_Text(pDX, IDC_EDIT_DISTANCIA_DMAXMIN, m_distanciaDMaxMin);
	DDX_Text(pDX, IDC_EDIT_DISTANCIA_DMIN, m_distanciaDMin);
	DDX_Text(pDX, IDC_EDIT_DISTANCIA_PERIMETRE, m_distanciaPerimetre);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDlgCfgCriteris, CDialog)
	//{{AFX_MSG_MAP(CDlgCfgCriteris)
	ON_BN_CLICKED(IDC_BOTO_OBSERVAR, OnBotoObservar)
	ON_BN_CLICKED(IDC_BOTO_BUIDAR, OnBotoBuidar)
	ON_BN_CLICKED(IDC_BOTO_BORRAR, OnBotoBorrar)
	ON_BN_CLICKED(IDC_BOTO_AFEGIR_CLASSE, OnBotoAfegirClasse)
	ON_BN_CLICKED(IDC_BOTO_BORRAR_CLASSE, OnBotoBorrarClasse)
	ON_BN_CLICKED(IDC_BOTO_CLASSIFICAR, OnBotoClassificar)
	ON_BN_CLICKED(IDC_BOTO_DESCLASSIFICAR, OnBotoDesclassificar)
	ON_BN_CLICKED(IDC_BOTO_GENERAR, OnBotoGenerar)
	ON_BN_CLICKED(IDC_BOTO_GUARDAR, OnBotoGuardar)
	ON_NOTIFY(NM_CLICK, IDC_LLISTA_NOCLASSIFICATS, OnClickLlistaNoclassificats)
	ON_BN_CLICKED(IDC_BOTO_AUTOMATIC, OnBotoAutomatic)
	ON_BN_CLICKED(IDC_BOTO_BORRAR_TOT, OnBotoBorrarTot)
	ON_BN_CLICKED(IDC_BOTO_RENOMENAR, OnBotoRenomenar)
	ON_BN_CLICKED(IDC_BOTO_BORRAR_MISSATGES, OnBotoBorrarMissatges)
	ON_BN_CLICKED(IDC_TANCAR, OnTancar)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

BOOL CDlgCfgCriteris::OnInitDialog() 
{
	m_llistaNoClassificat.SubclassDlgItem(IDC_LLISTA_NOCLASSIFICATS, this);
	m_llistaNoClassificat.DeleteAllItems();
	m_arbreClassificats.SubclassDlgItem(IDC_ARBRE_CLASSIFICATS, this);
	m_arbreClassificats.DeleteAllItems();
	m_llistaGenerat.SubclassDlgItem(IDC_LLISTA_GENERAT, this);
	m_llistaNoClassificat.InsertColumn(1, "Area", LVCFMT_LEFT, 50);
	m_llistaNoClassificat.InsertColumn(2, "Permetre", LVCFMT_LEFT, 50);
	m_llistaNoClassificat.InsertColumn(3, "Compacitat", LVCFMT_LEFT, 50);
	m_llistaNoClassificat.InsertColumn(4, "DMin", LVCFMT_LEFT, 50);
	m_llistaNoClassificat.InsertColumn(5, "DMax", LVCFMT_LEFT, 50);
	m_llistaNoClassificat.InsertColumn(6, "DM/Dm", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(0, "Nom", LVCFMT_LEFT, 60);
	m_llistaGenerat.InsertColumn(1, "aMin", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(2, "aMax", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(3, "pMin", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(4, "pMax", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(5, "cMin", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(6, "cMax", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(7, "dmMin", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(8, "dmMax", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(9, "dMMin", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(10, "dMMax", LVCFMT_LEFT, 50);
	m_llistaGenerat.InsertColumn(11, "(dM/dm)m", LVCFMT_LEFT, 60);
	m_llistaGenerat.InsertColumn(12, "(dM/dm)M", LVCFMT_LEFT, 65);
	m_llistaBitacora.SubclassDlgItem(IDC_LLISTA_BITACORA, this);

	omplirLlistaNoClassificats();
	return TRUE;  
}

//
// Omple l'objecte d'interfcie m_llistaNoClassificats
// a partir de les dades contingudes en m_llistaObjectes.
// De fet, m_llistaNoClassificats no s ms que l'objecte
// emprat per a visualitzar m_llistaObjectes.
//
void CDlgCfgCriteris::omplirLlistaNoClassificats()
{
	int i;
	LV_ITEM li;

	li.mask=LVIF_TEXT;
	li.pszText=new char[50];

	m_llistaNoClassificat.DeleteAllItems();
	for (i=0;i<m_llistaObjectes.GetSize();i++) {
		li.iItem=i;
		li.iSubItem=0;
		sprintf(li.pszText, "%d", m_llistaObjectes[i].m_area);
		m_llistaNoClassificat.InsertItem(&li);

		li.iItem=i;
		li.iSubItem=1;
		sprintf(li.pszText, "%d", m_llistaObjectes[i].m_perimetre);
		m_llistaNoClassificat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=2;
		sprintf(li.pszText, "%f", m_llistaObjectes[i].m_compacitat);
		m_llistaNoClassificat.SetItem(&li);
		
		li.iItem=i;
		li.iSubItem=3;
		sprintf(li.pszText, "%d", m_llistaObjectes[i].m_dMin);
		m_llistaNoClassificat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=4;
		sprintf(li.pszText, "%d", m_llistaObjectes[i].m_dMax);
		m_llistaNoClassificat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=5;
		sprintf(li.pszText, "%f", (float)(((float)(m_llistaObjectes[i].m_dMax))/((float)(m_llistaObjectes[i].m_dMin))));
		m_llistaNoClassificat.SetItem(&li);
	}
}

//
// Inicialitzaci del quadre de dileg.
// En primer lloc es posen en les variables membre uns
// valors per defecte. Desprs s'intenta carregar
// el contingut de l'objecte des de disc. Si existeix
// l'arxiu, el carrega. Si no, empram les dades introduides
// abans.
//
void CDlgCfgCriteris::inicialitzar(void *sistema)
{
	Create(IDD_CREACIO_CRITERIS);
	m_numClasse=0;
	m_vista=NULL;
	m_camara=&(((CSistema *)sistema)->m_camara);
	m_cimg=&(((CSistema *)sistema)->m_imgconfig);
	m_llistaObjectes.RemoveAll();
	m_classes.RemoveAll();
	m_usarArea = 1;
	m_usarCompacitat = 1;
	m_usarDMax = 1;
	m_usarDMin = 1;
	m_usarPerimetre = 1;
	m_usarDMaxMin=1;
	m_margeArea = 1000;
	m_margeCompacitat = (float)0.2;
	m_margeDMax = 10;
	m_margeDMin = 10;
	m_margePerimetre = 500;
	m_margeDMaxMin=(float)0.2;
	m_identificacio.inicialitzar(sistema);
	m_sistema=sistema;
	m_nomArxiu=(((CSistema *)sistema)->m_arxiuConfiguracioCriteris);
	carregar();
	UpdateData(false);
	
}

//
// La segent funci es crida quan l'usuari pitja el bot
// Observar en el quadre de dileg.
// El que fa s capturar un fotograma, realitzar-hi el procs
// descrit en la configuraci del procs d'imatge i obtenir-ne
// les propietats dels objectes visibles. Finalment afegeix la
// llista de propietats obtingudes al quadre de dileg.
//
void CDlgCfgCriteris::OnBotoObservar() 
{
	int i, xc, yc, per, xmn, ymn, dmn, xmx, ymx, dmx;
	long area;
	double comp, orien;
	// Es crea una imatge amb les mides adequades.
	CPriImatge laImatge, laImatgeTemporal;
	CPriImgMarques marques;
	CPriRegio regio;
	laImatge.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_cimg->m_xFinestra, m_cimg->m_yFinestra, m_cimg->m_ampleFinestra, m_cimg->m_altFinestra);
	laImatgeTemporal.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_cimg->m_xFinestra, m_cimg->m_yFinestra, m_cimg->m_ampleFinestra, m_cimg->m_altFinestra);
	marques.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_cimg->m_xFinestra, m_cimg->m_yFinestra, m_cimg->m_ampleFinestra, m_cimg->m_altFinestra);
	regio.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_cimg->m_xFinestra, m_cimg->m_yFinestra, m_cimg->m_ampleFinestra, m_cimg->m_altFinestra);
	// Es captura el fotograma
	m_camara->capturarFotograma();
	m_camara->posarEnMemoria(&laImatge);
	// Posa la imatge adequade en l'espai temporal segons el que
	// s'hagi determinat en les preferncies.
	switch (m_cimg->m_estilSiluetes) {
	case 0 : break;
	case 1 : break;
	case 2 : laImatgeTemporal=laImatge; break;
	}
	// Realitza el procs de la imatge
	((CSistema *)m_sistema)->realitzarProcesImatge(&laImatge);
	// Crea la imatge de marques
	laImatge.crearMarques(&marques);

	// Itera i per cada regi n'obt les propietats.
	for (i=1;i<marques.obtenirNumMarques();i++) {
		marques.crearRegio(i, &regio);
		area = regio.obtenirArea();
		regio.obtenirCentreGravetat(&xc, &yc);
		// Dibuixa el contorn sols si el volem veure.
		if (m_cimg->m_estilSiluetes==0)
			per = regio.obtenirPerimetre(NULL);
		else 
			per = regio.obtenirPerimetre(&laImatgeTemporal);

		comp = regio.obtenirCompacitat();
		regio.obtenirDMin(&xmn, &ymn, &dmn);
		regio.obtenirDMax(&xmx, &ymx, &dmx);

		if (m_cimg->m_estilSiluetes!=0) {
			laImatgeTemporal.dibuixarPunt(xc, yc, 2);
			laImatgeTemporal.dibuixarPunt(xmn, ymn, 2);
			laImatgeTemporal.dibuixarPunt(xmx, ymx, 2);
		}

		orien = regio.obtenirOrientacio();
		m_llistaObjectes.Add(CAuxPropietatObjecte(per, xc, yc, area, xmn, xmx, ymn, ymx, dmn, dmx, comp, orien));
	}
	if (m_cimg->m_estilSiluetes!=0) m_camara->llegirDeMemoria(&laImatgeTemporal);
	omplirLlistaNoClassificats();
}

BOOL CDlgCfgCriteris::estaActiu()
{
	return IsWindowVisible();
}

void CDlgCfgCriteris::alternar()
{
	if (estaActiu()) {
		((CBRASView *)m_vista)->m_dibuixarPunts=false;
		((CBRASView *)m_vista)->RedrawWindow();
		ShowWindow(SW_HIDE);
	}
	else 
		ShowWindow(SW_SHOW);
}

void CDlgCfgCriteris::OnOK()
{
	return;
}

void CDlgCfgCriteris::OnCancel()
{
	ShowWindow(SW_HIDE);
}

void CDlgCfgCriteris::OnBotoBuidar() 
{
	m_llistaObjectes.RemoveAll();
	omplirLlistaNoClassificats();
}

void CDlgCfgCriteris::OnBotoBorrar() 
{
	int elItem=m_llistaNoClassificat.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	if (elItem==-1) return;
	m_llistaObjectes.RemoveAt(elItem);
	omplirLlistaNoClassificats();
}

//
// Omple l'objecte d'interfcie m_arbreClassificats a partir
// de les dades contingudes en m_classes.
// De fet, l'nica funci de m_arbreClassificats s visualitzar
// les dades de m_classes.
//
void CDlgCfgCriteris::omplirArbreClasses()
{
	int i, j;
	HTREEITEM elPare;
	char missatge[200];
	CAuxPropietatObjecte *p;
	m_arbreClassificats.DeleteAllItems();
	for (i=0;i<m_classes.GetSize();i++) {
		elPare=m_arbreClassificats.InsertItem(TVIF_PARAM|TVIF_SELECTEDIMAGE|TVIF_STATE|TVIF_TEXT, m_classes[i]->obtenirNom(), 0, 0, TVIS_BOLD, TVIS_BOLD, (LPARAM)i, TVI_ROOT, TVI_LAST );
		for (j=0;j<m_classes[i]->m_propietats.GetSize();j++) {
			p=&(m_classes[i]->m_propietats[j]);
			sprintf(missatge, "A:%d / P:%d / C:%f / Dm:%d / DM:%d / DMm:%f", p->m_area, p->m_perimetre, p->m_compacitat, p->m_dMin, p->m_dMax, (double)(((double)(p->m_dMax))/((double)(p->m_dMin))));
			m_arbreClassificats.InsertItem(TVIF_PARAM|TVIF_SELECTEDIMAGE|TVIF_STATE|TVIF_TEXT, missatge, 0, 0, NULL, TVIS_BOLD, (LPARAM)j, elPare, TVI_LAST );
		}
	}
}

//
// Aquesta funci s'activa quan es pitja el bot de Afegir classe.
// Es comprova que el nom per la nova classe no sigui CLASxx, ja
// que aquest format de noms s el que genera la classe quan
// ho ha de fer automticament. Tamb comprova que realment 
// s'hagi escrit alguna cosa, i si el nom ja est en s.
// Si totes les comprovacions es realitzen satisfactriament, 
// es procedeix a inserir la classe.
//
void CDlgCfgCriteris::OnBotoAfegirClasse() 
{
	UpdateData(true);
	int i;
	if (m_nomClasse.IsEmpty()) return;
	m_nomClasse.MakeUpper();
	if (m_nomClasse.Left(4)=="CLAS") {
		AfxMessageBox("Els noms comenats en CLAS estan reservats pel sistema");
		return;
	}
	m_nomClasse=m_nomClasse.Left(6);
	for (i=0;i<m_classes.GetSize();i++) {
		if (m_classes[i]->obtenirNom()==m_nomClasse) {
			AfxMessageBox("Ja hi ha una classe amb aquest nom");
			return;
		}
	}
	m_classes.Add(new CAuxClasse(m_nomClasse));
	omplirArbreClasses();
	m_nomClasse=_T("");
	UpdateData(false);
}

//
// Funci que s'activa en resposta a l'event de polsaci del bot
// de borrat de classe.
// Borra la classe sempre i quan no tengui cap objecte especfic
// associat.
//
void CDlgCfgCriteris::OnBotoBorrarClasse() 
{
	HTREEITEM tmp;
	if ((tmp=m_arbreClassificats.GetSelectedItem())==NULL) return;
	if (m_classes[m_arbreClassificats.GetItemData(tmp)]->m_propietats.GetSize()!=0) {
		AfxMessageBox("La classe que voleu eliminar no est buida.\nEliminau abans tots els seus elements.");
		return;
	}
	m_classes.RemoveAt(m_arbreClassificats.GetItemData(tmp));
	omplirArbreClasses();
}

//
// Resposta a la polsaci del bot de classificar ('>').
// Trasllada l'objecte especfic seleccionat des de la
// llista d'objectes observats fins a la llista d'objectes
// especfics de la classe seleccionada.
//
void CDlgCfgCriteris::OnBotoClassificar() 
{
	int itemLlista, itemArbre;
	HTREEITEM tmp, tmp2;
	((CBRASView *)m_vista)->m_dibuixarPunts=false;
	((CBRASView *)m_vista)->RedrawWindow();
	itemLlista=m_llistaNoClassificat.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	if (itemLlista==-1) return;
	if ((tmp=m_arbreClassificats.GetSelectedItem())==NULL) return;
	tmp2=m_arbreClassificats.GetParentItem(tmp);
	if (tmp2==NULL)
		itemArbre=m_arbreClassificats.GetItemData(tmp);
	else 
		itemArbre=m_arbreClassificats.GetItemData(tmp2);
	m_classes[itemArbre]->m_propietats.Add(m_llistaObjectes[itemLlista]);
	m_llistaObjectes.RemoveAt(itemLlista);
	omplirArbreClasses();
	omplirLlistaNoClassificats();
}

//
// Aquesta funci es crida en resposta a l'event de polsaci
// del bot de desclassificaci ('<').
// Trasllada l'objecte especfic seleccionat en l'arbre
// de classes cap a la llista d'objectes especfics observats-
//
void CDlgCfgCriteris::OnBotoDesclassificar() 
{
	int itemArbre, pareItemArbre;
	HTREEITEM tmp, tmp2;

	if ((tmp=m_arbreClassificats.GetSelectedItem())==NULL) return;
	tmp2=m_arbreClassificats.GetParentItem(tmp);
	if (tmp2==NULL) return;
	else {
		itemArbre=m_arbreClassificats.GetItemData(tmp);
		pareItemArbre=m_arbreClassificats.GetItemData(tmp2);
	}
	m_llistaObjectes.Add(m_classes[pareItemArbre]->m_propietats[itemArbre]);
	m_classes[pareItemArbre]->m_propietats.RemoveAt(itemArbre);
	omplirArbreClasses();
	omplirLlistaNoClassificats();
	return;
}

//
// Resposta a la polsaci del bot 'Generar'.
// El funcionament s el segent:
// * Per a cada classe fer
//   * Calcular els valors mnims i mxims de cada atribut dels
//     objectes que pertanyen a la dita classe.
//   * Modificar aquests valors mnims i mxims segons els
//     marges especificats.
//   * Passar aquesta informaci a la forma d'objectes generats.
//   * Si hi ha colisi, avisar.
//
//
void CDlgCfgCriteris::OnBotoGenerar() 
{
	int i, j, pmin=30000, pmax=0, dminmin=30000, dminmax=0, dmaxmin=30000, dmaxmax=0;
	long amin=60000, amax=0;
	double cmin=30000, cmax=0, dmaxminmin=30000, dmaxminmax=0;
	CAuxPropietatObjecte *p;
	afegirBitacora("---> Inici Generaci <---");
	UpdateData(true);
	m_objectes.RemoveAll();
	for (i=0;i<m_classes.GetSize();i++) {
		pmin=30000;
		pmax=0;
		dminmin=30000;
		dminmax=0; 
		dmaxmin=30000; 
		dmaxmax=0;
		amin=60000;
		amax=0;
		cmin=30000;
		cmax=0;
		dmaxminmin=30000;
		dmaxminmax=0;
		for (j=0;j<m_classes[i]->m_propietats.GetSize();j++) {
			p=&(m_classes[i]->m_propietats[j]);
			if (m_usarArea) actualitzaMaxMin(p->m_area, &amin, &amax);
			if (m_usarPerimetre) actualitzaMaxMin(p->m_perimetre, &pmin, &pmax);
			if (m_usarCompacitat) actualitzaMaxMin(p->m_compacitat, &cmin, &cmax);
			if (m_usarDMin) actualitzaMaxMin(p->m_dMin, &dminmin, &dminmax);
			if (m_usarDMax) actualitzaMaxMin(p->m_dMax, &dmaxmin, &dmaxmax);
			if (m_usarDMaxMin) actualitzaMaxMin((((double)(p->m_dMax))/((double)(p->m_dMin))), &dmaxminmin, &dmaxminmax);
		}
		if (m_usarArea) {
			amin-=(long)m_margeArea;
			amax+=(long)m_margeArea;
		}
		if (m_usarPerimetre) {
			pmin-=(int)m_margePerimetre;
			pmax+=(int)m_margePerimetre;
		}
		if (m_usarCompacitat) {
			cmin-=(double)m_margeCompacitat;
			cmax+=(double)m_margeCompacitat;
		}
		if (m_usarDMin) {
			dminmin-=(int)m_margeDMin;
			dminmax+=(int)m_margeDMin;
		}
		if (m_usarDMax) {
			dmaxmin-=(int)m_margeDMax;
			dmaxmax+=(int)m_margeDMax;
		}
		if (m_usarDMaxMin) {
			dmaxminmin-=(double)m_margeDMaxMin;
			dmaxminmax+=(double)m_margeDMaxMin;
		}
		m_objectes.Add(CAuxObjecteGenerat(m_classes[i]->obtenirNom(), amin, amax, pmin, pmax, cmin, cmax, dminmin, dminmax, dmaxmin, dmaxmax, dmaxminmin, dmaxminmax));
	}
	//
	// Ara s'ha de mirar si hi ha intersecci entre les
	// definicions que s'han generat dels objectes.
	//
	for (i=0;i<m_objectes.GetSize()-1;i++) {
		for (j=i+1;j<m_objectes.GetSize();j++) {
			mirarInterseccio(&(m_objectes[i]), &(m_objectes[j]));
		}
	}
	omplirLlistaGenerat();
	afegirBitacora("---> Final Generaci <---");
}
//
// Les segents tres funcions actualitzen els valors mxims
// i mnims. S'empren en la generaci d'objectes generats.
//
void CDlgCfgCriteris::actualitzaMaxMin(int v, int * min, int * max)
{
	if (v<(*min)) *min=v;
	if (v>(*max)) *max=v;
}

void CDlgCfgCriteris::actualitzaMaxMin(long v, long * min, long * max)
{
	if (v<(*min)) *min=v;
	if (v>(*max)) *max=v;
}

void CDlgCfgCriteris::actualitzaMaxMin(double v, double * min, double * max)
{
	if (v<(*min)) *min=v;
	if (v>(*max)) *max=v;
}

//
// Omple l'objecte d'interfcie m_llistaGenerat a partir
// de les dades contingudes en m_objectes.
// De fet l'nica funci de m_llistaGenerat s la de visualitzar
// els continguts de m_objectes.
//
void CDlgCfgCriteris::omplirLlistaGenerat()
{
	int i;
	LV_ITEM li;

	li.mask=LVIF_TEXT;
	li.pszText=new char[50];

	m_llistaGenerat.DeleteAllItems();
	for (i=0;i<m_objectes.GetSize();i++) {
		li.iItem=i;
		li.iSubItem=0;
		sprintf(li.pszText, "%s", m_objectes[i].obtenirNom());
		m_llistaGenerat.InsertItem(&li);

		li.iItem=i;
		li.iSubItem=1;
		if (m_usarArea) sprintf(li.pszText, "%d", m_objectes[i].obtenirAMin());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=2;
		if (m_usarArea) sprintf(li.pszText, "%d", m_objectes[i].obtenirAMax());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);
		
		li.iItem=i;
		li.iSubItem=3;
		if (m_usarPerimetre) sprintf(li.pszText, "%d", m_objectes[i].obtenirPMin());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=4;
		if (m_usarPerimetre) sprintf(li.pszText, "%d", m_objectes[i].obtenirPMax());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=5;
		if (m_usarCompacitat) sprintf(li.pszText, "%f", m_objectes[i].obtenirCMin());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=6;
		if (m_usarCompacitat) sprintf(li.pszText, "%f", m_objectes[i].obtenirCMax());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=7;
		if (m_usarDMin) sprintf(li.pszText, "%d", m_objectes[i].obtenirDmMin());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=8;
		if (m_usarDMin) sprintf(li.pszText, "%d", m_objectes[i].obtenirDmMax());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=9;
		if (m_usarDMax) sprintf(li.pszText, "%d", m_objectes[i].obtenirDMMin());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=10;
		if (m_usarDMax) sprintf(li.pszText, "%d", m_objectes[i].obtenirDMMax());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=11;
		if (m_usarDMaxMin) sprintf(li.pszText, "%f", m_objectes[i].obtenirDMaxMinMin());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);

		li.iItem=i;
		li.iSubItem=12;
		if (m_usarDMaxMin) sprintf(li.pszText, "%f", m_objectes[i].obtenirDMaxMinMax());
		else sprintf(li.pszText, " ");
		m_llistaGenerat.SetItem(&li);
	}
}

//
// Transforma el llistat d'objectes generats en un llistat
// d'objectes en el format que empren els objectes de la
// classe CIdentificaci.
// El propsit d'aquesta conversi s poder fer servir les 
// rutines per a guardar en format .dob de la classe
// CIdentificacio.
// 
void CDlgCfgCriteris::transformarObjectesGeneratsEnObjectes(CLlistaObjectesGenerats * og, CLlistaObjectes * o)
{
	int i, j, k, l, pmin, pmax, dminmin, dminmax, dmaxmin, dmaxmax;
	long amin, amax;
	double cmin, cmax, dmaxminmin, dmaxminmax;
	CString nom;
	o->RemoveAll();
	for (i=0;i<og->GetSize();i++) {
		nom=(*og)[i].obtenirNom();
		if (m_usarArea) {
			amin=(*og)[i].obtenirAMin();
			amax=(*og)[i].obtenirAMax();
		}
		if (m_usarPerimetre) {
			pmin=(*og)[i].obtenirPMin();
			pmax=(*og)[i].obtenirPMax();
		}
		if (m_usarCompacitat) {
			cmin=(*og)[i].obtenirCMin();
			cmax=(*og)[i].obtenirCMax();
		}
		if (m_usarDMin) {
			dminmin=(*og)[i].obtenirDmMin();
			dminmax=(*og)[i].obtenirDmMax();
		}
		if (m_usarDMax) {
			dmaxmin=(*og)[i].obtenirDMMin();
			dmaxmax=(*og)[i].obtenirDMMax();
		}
		if (m_usarDMaxMin) {
			dmaxminmin=(*og)[i].obtenirDMaxMinMin();
			dmaxminmax=(*og)[i].obtenirDMaxMinMax();
		}
		j=(o->Add(new CAuxObjecte(nom, (*og)[i].obtenirDX(), (*og)[i].obtenirDY(), (*og)[i].obtenirRot())));
		if (m_usarArea) {		
			k=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atArea, cmMajor));
			l=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atArea, cmMenor));
			(*((*o)[j]->obtenirCriteris()))[k]->obtenirExpressio()->Add(new CAuxElementExpressio(true, amin, atNul, opInici));
			(*((*o)[j]->obtenirCriteris()))[l]->obtenirExpressio()->Add(new CAuxElementExpressio(true, amax, atNul, opInici));
		}
		if (m_usarPerimetre) {		
			k=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atPerimetre, cmMajor));
			l=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atPerimetre, cmMenor));
			(*((*o)[j]->obtenirCriteris()))[k]->obtenirExpressio()->Add(new CAuxElementExpressio(true, pmin, atNul, opInici));
			(*((*o)[j]->obtenirCriteris()))[l]->obtenirExpressio()->Add(new CAuxElementExpressio(true, pmax, atNul, opInici));
		}
		if (m_usarCompacitat) {		
			k=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atCompacitat, cmMajor));
			l=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atCompacitat, cmMenor));
			(*((*o)[j]->obtenirCriteris()))[k]->obtenirExpressio()->Add(new CAuxElementExpressio(true, cmin, atNul, opInici));
			(*((*o)[j]->obtenirCriteris()))[l]->obtenirExpressio()->Add(new CAuxElementExpressio(true, cmax, atNul, opInici));
		}
		if (m_usarDMin) {		
			k=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atDMin, cmMajor));
			l=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atDMin, cmMenor));
			(*((*o)[j]->obtenirCriteris()))[k]->obtenirExpressio()->Add(new CAuxElementExpressio(true, dminmin, atNul, opInici));
			(*((*o)[j]->obtenirCriteris()))[l]->obtenirExpressio()->Add(new CAuxElementExpressio(true, dminmax, atNul, opInici));
		}
		if (m_usarDMax) {		
			k=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atDMax, cmMajor));
			l=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atDMax, cmMenor));
			(*((*o)[j]->obtenirCriteris()))[k]->obtenirExpressio()->Add(new CAuxElementExpressio(true, dmaxmin, atNul, opInici));
			(*((*o)[j]->obtenirCriteris()))[l]->obtenirExpressio()->Add(new CAuxElementExpressio(true, dmaxmax, atNul, opInici));
		}
		if (m_usarDMaxMin) {
			k=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atDMax, cmMajor));
			l=(*o)[j]->obtenirCriteris()->Add(new CAuxCriteri(atDMax, cmMenor));
			(*((*o)[j]->obtenirCriteris()))[k]->obtenirExpressio()->Add(new CAuxElementExpressio(true, dmaxminmin, atNul, opInici));
			(*((*o)[j]->obtenirCriteris()))[k]->obtenirExpressio()->Add(new CAuxElementExpressio(false, 0, atDMin, opProducte));
			(*((*o)[j]->obtenirCriteris()))[l]->obtenirExpressio()->Add(new CAuxElementExpressio(true, dmaxminmax, atNul, opInici));
			(*((*o)[j]->obtenirCriteris()))[l]->obtenirExpressio()->Add(new CAuxElementExpressio(false, 0, atDMin, opProducte));
		}
	}
}

//
// Guarda els objectes generats en format .dob, convertint-los
// prviament al format de llista d'objectes emprat per 
// la classe CIdentificacio.
//
void CDlgCfgCriteris::OnBotoGuardar() 
{
	char directoriActual[300];
	static char BASED_CODE szFilter[] = "Arxius de definici d'objectes (*.dob)|*.dob|Tots els Arxius (*.*)|*.*||";
	CFileDialog fdlg(false, ".dob", NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter);
	GetCurrentDirectory(200, directoriActual);
	if (fdlg.DoModal()!=IDOK) return;
	SetCurrentDirectory(directoriActual);
	transformarObjectesGeneratsEnObjectes(&m_objectes, &(m_identificacio.m_objectes));
	m_identificacio.guardarObjectes(fdlg.GetPathName());
}

//
// Guarda l'estat de l'objecte a disc
//
void CDlgCfgCriteris::guardar()
{
	int i, j;
	CFile fitxer;
	if (!fitxer.Open(m_nomArxiu, CFile::modeWrite | CFile::modeCreate)) {
		AfxMessageBox ("Problemes guardant l'arxiu");
		return;
	}

	CArchive arxiu(&fitxer, CArchive::store);

	UpdateData(true);

	arxiu << m_usarArea;
	arxiu << m_usarCompacitat;
	arxiu << m_usarPerimetre;
	arxiu << m_usarDMin;
	arxiu << m_usarDMax;
	arxiu << m_usarDMaxMin;
	arxiu << m_margeArea;
	arxiu << m_margeCompacitat;
	arxiu << m_margePerimetre;
	arxiu << m_margeDMin;
	arxiu << m_margeDMax;
	arxiu << m_margeDMaxMin;
	arxiu << m_numClasse;
	arxiu << m_distanciaArea;
	arxiu << m_distanciaPerimetre;
	arxiu << m_distanciaCompacitat;
	arxiu << m_distanciaDMin;
	arxiu << m_distanciaDMax;
	arxiu << m_distanciaDMaxMin;

	//
	// Guardar m_classes
	//
	arxiu << m_classes.GetSize();	// Guardam el nmero d'elements
									// en m_classes
	for (i=0;i<m_classes.GetSize();i++) {
		arxiu << m_classes[i]->obtenirNom();
		// Ara es guarda el nmero d'objectes concrets que hi ha
		// en la classe.
		arxiu << m_classes[i]->m_propietats.GetSize(); 
		for (j=0;j<m_classes[i]->m_propietats.GetSize();j++) {
			arxiu << (m_classes[i]->m_propietats[j].m_area);
			arxiu << (m_classes[i]->m_propietats[j].m_perimetre);
			arxiu << (m_classes[i]->m_propietats[j].m_compacitat);
			arxiu << (m_classes[i]->m_propietats[j].m_dMin);
			arxiu << (m_classes[i]->m_propietats[j].m_dMax);
		}
	}

	//
	// Guardam m_llistaObjectes
	//
	arxiu << m_llistaObjectes.GetSize();
	for (i=0;i<m_llistaObjectes.GetSize();i++) {
			arxiu << m_llistaObjectes[i].m_area;
			arxiu << m_llistaObjectes[i].m_perimetre;
			arxiu << m_llistaObjectes[i].m_compacitat;
			arxiu << m_llistaObjectes[i].m_dMin;
			arxiu << m_llistaObjectes[i].m_dMax;
	}
	
	arxiu.Close();
	fitxer.Close();
}

//
// Carrega l'estat de l'objecte des de disc.
//
void CDlgCfgCriteris::carregar()
{
	int nc, np, ca, pa, i, j;
	CString nom;
	CFile fitxer;
	if (!fitxer.Open(m_nomArxiu, CFile::modeRead)) {
		AfxMessageBox ("Problemes llegint l'arxiu");
		return;
	}
	CArchive arxiu(&fitxer, CArchive::load);

	arxiu >> m_usarArea;
	arxiu >> m_usarCompacitat;
	arxiu >> m_usarPerimetre;
	arxiu >> m_usarDMin;
	arxiu >> m_usarDMax;
	arxiu >> m_usarDMaxMin;
	arxiu >> m_margeArea;
	arxiu >> m_margeCompacitat;
	arxiu >> m_margePerimetre;
	arxiu >> m_margeDMin;
	arxiu >> m_margeDMax;
	arxiu >> m_margeDMaxMin;
	arxiu >> m_numClasse;
	arxiu >> m_distanciaArea;
	arxiu >> m_distanciaPerimetre;
	arxiu >> m_distanciaCompacitat;
	arxiu >> m_distanciaDMin;
	arxiu >> m_distanciaDMax;
	arxiu >> m_distanciaDMaxMin;

	UpdateData(false);

	m_classes.RemoveAll();
	m_llistaObjectes.RemoveAll();
	//
	// Carregam m_classes
	//
	arxiu >> nc;
	for (i=0;i<nc;i++) {
		arxiu >> nom;
		ca=m_classes.Add(new CAuxClasse(nom));
		arxiu >> np;
		for (j=0;j<np;j++) {
			pa=m_classes[ca]->m_propietats.Add(CAuxPropietatObjecte());
			arxiu >> (m_classes[ca]->m_propietats[pa].m_area);
			arxiu >> (m_classes[ca]->m_propietats[pa].m_perimetre);
			arxiu >> (m_classes[ca]->m_propietats[pa].m_compacitat);
			arxiu >> (m_classes[ca]->m_propietats[pa].m_dMin);
			arxiu >> (m_classes[ca]->m_propietats[pa].m_dMax);
		}
	}
	if (m_classes.GetSize()==0) m_numClasse=0;

	//
	// Carregam m_llistaObjectes
	//
	arxiu >> nc;
	for (i=0;i<nc;i++) {
		ca=m_llistaObjectes.Add(CAuxPropietatObjecte());
		arxiu >> (m_llistaObjectes[ca].m_area);
		arxiu >> (m_llistaObjectes[ca].m_perimetre);
		arxiu >> (m_llistaObjectes[ca].m_compacitat);
		arxiu >> (m_llistaObjectes[ca].m_dMin);
		arxiu >> (m_llistaObjectes[ca].m_dMax);
	}
	arxiu.Close();
	fitxer.Close();
	omplirArbreClasses();
	omplirLlistaNoClassificats();
}

//
// Finalitza l'objecte
//
void CDlgCfgCriteris::finalitzar()
{
	guardar();
}

void CDlgCfgCriteris::assignarVista(void * vista)
{
	m_vista=vista;
}

//
// Quan es fa click a la llista d'objectes no classificats, 
// si l'objecte ha estat capturat des de la imatge recentment,
// s'ha de marcar d'alguna forma per a qu l'usuari
// sapi a quin objecte dels que es veuen es correspon la
// llista de propietats que t en pantalla.
// Aix s el que fa aquesta funci
//
void CDlgCfgCriteris::OnClickLlistaNoclassificats(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int elItem;
	if (m_vista==NULL) {
		*pResult = 0;
		return;
	}
	elItem=m_llistaNoClassificat.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	if (elItem==-1) return;
	((CBRASView *)m_vista)->m_x=m_llistaObjectes[elItem].m_xCentre;
	((CBRASView *)m_vista)->m_y=m_llistaObjectes[elItem].m_yCentre;
	((CBRASView *)m_vista)->m_dibuixarPunts=true;
	((CBRASView *)m_vista)->RedrawWindow();
	*pResult = 0;
}

// 
// Funci de resposta a l'event de pitjar el bot
// de genraci automtica de classes ('>>').
// El que fa s el segent:
// * Per a cada objecte no classificat (o1)
//   * Per a cada classe (c)
//     * Per a cada objecte en la classe (o2)
//        * Si o1 s semblant a o2 llavors
//          afegir o1 a la llista d'objectes de c
//   * Si o1 no ha estat inserit en cap classe, crear una nova
//     classe i inserir-hi o1.
//
void CDlgCfgCriteris::OnBotoAutomatic() 
{
	int i, j, k, l;
	int l1, l2, l3;
	char nom[30];
	bool trobat=false;
	CAuxPropietatObjecte *p1, *p2;
	UpdateData(true);
	l1=m_llistaObjectes.GetSize();
	for (i=0;i<l1;i++) {
		trobat=false;
		p1=&(m_llistaObjectes[i]);
		l2=m_classes.GetSize();
		for (j=0;j<l2;j++) {
			l3=m_classes[j]->m_propietats.GetSize();
			for (k=0;k<l3;k++) {
				p2=&(m_classes[j]->m_propietats[k]);
				if (semblants(p1, p2) && !trobat) {
					m_classes[j]->m_propietats.Add(*p1);
					trobat=true;
				}
			}
		}
		if (!trobat) {
			sprintf(nom, "CLAS%d", m_numClasse++);
			l=m_classes.Add(new CAuxClasse(nom));
			m_classes[l]->m_propietats.Add(*p1);
		}
	}
	m_llistaObjectes.RemoveAll();
	omplirLlistaNoClassificats();
	omplirArbreClasses();
}

//
// Determina si dos objectes especfics sn semblants, segons
// les caracterstiques definides en el quadre de dileg.
//
bool CDlgCfgCriteris::semblants(CAuxPropietatObjecte * p1, CAuxPropietatObjecte * p2)
{
	if (m_usarArea) {
		if (!semblant(p1->m_area, p2->m_area, m_distanciaArea)) return false;
	}
	if (m_usarPerimetre) {
		if (!semblant(p1->m_perimetre, p2->m_perimetre, m_distanciaPerimetre)) return false;
	}
	if (m_usarCompacitat) {
		if (!semblant(p1->m_compacitat, p2->m_compacitat, m_distanciaCompacitat)) return false;
	}
	if (m_usarDMin) {
		if (!semblant(p1->m_dMin, p2->m_dMin, m_distanciaDMin)) return false;
	}
	if (m_usarDMax) {
		if (!semblant(p1->m_dMax, p2->m_dMax, m_distanciaDMax)) return false;
	}
	if (m_usarDMaxMin) {
		if (!semblant( (double)((p1->m_dMax)/(p1->m_dMin)), (double)((p2->m_dMax)/(p2->m_dMin)), m_distanciaDMaxMin)) return false;
	}
	return true;
}

//
// Les segents tres funcions determinen si a i b sn
// semblants, essent c el marge que s'accepta.
//
bool CDlgCfgCriteris::semblant(int a, int b, float c)
{
	return ((a<(b+(int)c))&&(a>(b-(int)c)));
}

bool CDlgCfgCriteris::semblant(long a, long b, float c)
{
	return ((a<(b+(long)c))&&(a>(b-(long)c)));
}

bool CDlgCfgCriteris::semblant(double a, double b, float c)
{
	return ((a<(b+(double)c))&&(a>(b-(double)c)));
}

//
// Resposta a la polsaci del bot de borrar el contingut de
// l'arbre de classes ('<<').
// Fa un recorregut per totes les classes, envia el seu
// contingut cap a la llista de no classificats i borra les
// classes.
//
void CDlgCfgCriteris::OnBotoBorrarTot() 
{
	int i, j;
	for(i=0;i<m_classes.GetSize();i++) {
		for (j=0;j<m_classes[i]->m_propietats.GetSize();j++) 
			m_llistaObjectes.Add(m_classes[i]->m_propietats[j]);
		m_classes[i]->m_propietats.RemoveAll();
	}
	m_classes.RemoveAll();
	omplirArbreClasses();
	omplirLlistaNoClassificats();
	return;
}

//
// Resposta a la polsaci del bot de renomenar una classe.
// Comprova si el nou nom s correcte (distint de ""i que
// no comenci per CLASS) i renomena la classe seleccionada.
//
void CDlgCfgCriteris::OnBotoRenomenar() 
{
	HTREEITEM tmp;
	UpdateData(true);
	if (m_nomClasse.IsEmpty()) return;
	m_nomClasse.MakeUpper();
	if (m_nomClasse.Left(4)=="CLAS") {
		AfxMessageBox("Els noms comenats en CLAS estan reservats pel sistema");
		return;
	}
	m_nomClasse=m_nomClasse.Left(6);
	if ((tmp=m_arbreClassificats.GetSelectedItem())==NULL) return;
	m_classes[m_arbreClassificats.GetItemData(tmp)]->posarNom(m_nomClasse);
	omplirArbreClasses();
	m_nomClasse=_T("");
	UpdateData(false);
}

//
// Comprova si dos objectes generats intersecten.
// Si s aix retorna true i mostra la informaci en el
// quadre de missatges.
//
bool CDlgCfgCriteris::mirarInterseccio(CAuxObjecteGenerat * o1, CAuxObjecteGenerat * o2)
{
	bool interseccio=true;
	char missatge[200], tmp[50];
	if (m_usarArea) {
		interseccio=interseccio && ((o1->obtenirAMax()>o2->obtenirAMin())&& (o1->obtenirAMax()<o2->obtenirAMax()));
		strcpy(tmp, "area");
	}
	if (m_usarPerimetre && interseccio) {
		interseccio=interseccio && ((o1->obtenirPMax()>o2->obtenirPMin())&& (o1->obtenirPMax()<o2->obtenirPMax()));
		strcpy(tmp, "perimetre");
	}
	if (m_usarCompacitat && interseccio) {
		interseccio=interseccio && ((o1->obtenirCMax()>o2->obtenirCMin())&& (o1->obtenirCMax()<o2->obtenirCMax()));
		strcpy(tmp, "compacitat");
	}
	if (m_usarDMin && interseccio) {
		interseccio=interseccio && ((o1->obtenirDmMax()>o2->obtenirDmMin())&& (o1->obtenirDmMax()<o2->obtenirDmMax()));
		strcpy(tmp, "DMin");
	}
	if (m_usarDMax && interseccio) {
		interseccio=interseccio && ((o1->obtenirDMMax()>o2->obtenirDMMin())&& (o1->obtenirDMMax()<o2->obtenirDMMax()));
		strcpy(tmp, "DMax");
	}
	if (m_usarDMaxMin && interseccio) {
		interseccio=interseccio && ((o1->obtenirDMaxMinMax()>o2->obtenirDMaxMinMin())&& (o1->obtenirDMaxMinMax()<o2->obtenirDMaxMinMax()));
		strcpy(tmp, "DMaxMin");
	}
	if (interseccio) {
		sprintf(missatge, "Colisio entre %s i %s per %s", o1->obtenirNom(), o2->obtenirNom(), tmp);
		afegirBitacora(missatge);
	}
	return interseccio;
}

//
// Afegeix una lnia al quadre de missatges
//
void CDlgCfgCriteris::afegirBitacora(CString cadena)
{
	m_llistaBitacora.AddString(cadena);
	m_llistaBitacora.SetTopIndex(m_llistaBitacora.GetTopIndex()+1);
}

//
// Borra tot el contingut del quadre de missatges.
//
void CDlgCfgCriteris::OnBotoBorrarMissatges() 
{
	m_llistaBitacora.ResetContent();
}


void CDlgCfgCriteris::OnTancar() 
{
	ShowWindow(SW_HIDE);	
}
