/////////////////////////////////////////////////////////////////
//
// Arxiu      : DlgCfgCriteris.h
//
// Descripci : Arxiu de capalera de la classe 
//				d'obtenci de criteris.
//				Aquesta classe permet, mitjanant un quadre
//				de dileg, observar objectes i extreure'n
//				la regla general que els defineix.
//				El seu funcionament permet:
//				* Observar objectes i extreure'n les
//				  propietats.
//				* Crear classes d'objectes, i assignar
//				  manualment cada objecte especfic
//				  a una de les classes.
//				* Definir els lmits que permeten incloure
//				  un objecte en una o altra classe.
//				* Classificaci automtica dels objectes en
//				  classes a partir d'uns criter
//				* Obtenci dels criteris que defineixen un
//				  objecte a partir d'observacions concretes
//				  de l'objecte.
//				* Emmagatzemar en el format .dob les dades
//				  obtingudes.
//				Es distingeixen:
//				* Objectes especfics, captats per la cmara.
//				  La seva informaci s'emmagatzema en objectes
//				  de la classe CAuxPropietatObjecte.
//				* Classes. Una classe t un nom (p.ex. TRIANG) i
//				  una llista d'objectes especfics associats.
//				* Objectes generats. Un objecte generat t
//				  un nom i les caracterstiques que el
//				  defineixen. Un objecte generat difereix de
//				  l'objecte emprat per l'identificaci en que
//				  aquest darrer t una llista de criteris, 
//				  cada un dels quals s de la forma
//				  atribut comparador expressi. Per contra,
//				  un criteri d'un objecte generat s ms simple,
//				  i sols pot esser atribut < valor1 i
//				  atribut > valor2.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#if !defined(AFX_DIALEGCREACIOCRITERIS_H__82ACBAA2_09F4_11D4_B60D_00C026C084F2__INCLUDED_)
#define AFX_DIALEGCREACIOCRITERIS_H__82ACBAA2_09F4_11D4_B60D_00C026C084F2__INCLUDED_

//
// Arxius de capalera
//
#include "AuxPropietatObjecte.h"
#include "DspCamara.h"
#include "DlgCfgImatge.h"
#include "AuxClasse.h"
#include "AuxObjecteGenerat.h"
#include "ScrIdentificacio.h"

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class CDlgCfgCriteris : public CDialog
{
public:
	// Constructor
	CDlgCfgCriteris(CWnd* pParent = NULL);
	// Es necessita una referncia a la vista per a poder
	// superposar imatges sobre la que ha capturat la cmara.
	void assignarVista(void *vista);
	// Realitza les tasques de finalitzaci de l'objecte
	void finalitzar();
	// Alterna l'estat del quadre de dileg entre visible 
	// i invisible
	void alternar();
	// Determina si el quadre de dileg est visible o no.
	BOOL estaActiu();
	// Inicialitza el quadre de dileg.
	void inicialitzar(void *sistema);
	// La llista dels objectes concrets observats.
	CLlistaPropietatsObjectes m_llistaObjectes;
	void OnCancel();
	void OnOK();

// Dialog Data
	//{{AFX_DATA(CDlgCfgCriteris)
	enum { IDD = IDD_CREACIO_CRITERIS };
	CListBox	m_llistaBitacora;
	CListCtrl	m_llistaNoClassificat;
	CListCtrl	m_llistaGenerat;
	CTreeCtrl	m_arbreClassificats;
	BOOL	m_usarArea;
	BOOL	m_usarCompacitat;
	BOOL	m_usarDMax;
	BOOL	m_usarDMin;
	BOOL	m_usarPerimetre;
	float	m_margeArea;
	float	m_margeCompacitat;
	float	m_margeDMax;
	float	m_margeDMin;
	float	m_margePerimetre;
	CString	m_nomClasse;
	BOOL	m_usarDMaxMin;
	float	m_margeDMaxMin;
	float	m_distanciaArea;
	float	m_distanciaCompacitat;
	float	m_distanciaDMax;
	float	m_distanciaDMaxMin;
	float	m_distanciaDMin;
	float	m_distanciaPerimetre;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDlgCfgCriteris)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CDlgCfgCriteris)
	virtual BOOL OnInitDialog();
	afx_msg void OnBotoObservar();
	afx_msg void OnBotoBuidar();
	afx_msg void OnBotoBorrar();
	afx_msg void OnBotoAfegirClasse();
	afx_msg void OnBotoBorrarClasse();
	afx_msg void OnBotoClassificar();
	afx_msg void OnBotoDesclassificar();
	afx_msg void OnBotoGenerar();
	afx_msg void OnBotoGuardar();
	afx_msg void OnClickLlistaNoclassificats(NMHDR* pNMHDR, LRESULT* pResult);
	afx_msg void OnBotoAutomatic();
	afx_msg void OnBotoBorrarTot();
	afx_msg void OnBotoRenomenar();
	afx_msg void OnBotoBorrarMissatges();
	afx_msg void OnTancar();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	void * m_sistema;
	// Afegeix un missatge al quadre de missatges del quadre de
	// dileg.
	void afegirBitacora(CString cadena);
	// Determina si els objectes generats o1 i o2 intersecten
	// per algun dels criteris. Si s aix, retorna true i indica
	// en el quadre de missatges entre quins objectes i per
	// quines caracterstiques s'ha produit la col.lisi.
	bool mirarInterseccio(CAuxObjecteGenerat *o1, CAuxObjecteGenerat *o2);
	// S'empra com a identificador per a generar noms
	// automtics de classes.
	int m_numClasse;
	// Le segents tres funcions determinen si els valors a i
	// b sn semblants, essent c el marge de semblana.
	bool semblant(double a, double b, float c);
	bool semblant(long a, long b, float c);
	bool semblant(int a, int b, float c);
	// Determina si els objectes especfics p1 i p2 sn
	// semblants segons els marges de semblana definits en
	// el quadre de dileg.
	bool semblants(CAuxPropietatObjecte *p1, CAuxPropietatObjecte *p2);
	// La vista.
	void * m_vista;
	// El nom de l'arxiu on es guarda l'estat de l'objecte
	CString m_nomArxiu;
	// Carrega l'estat de l'objecte
	void carregar();
	// Guarda l'estat de l'objecte
	void guardar();
	// Objecte identificaci usat per a emmagatzemar les
	// definicions dels objectes en el format .dob
	CScrIdentificacio m_identificacio;
	// Omple l'element d'interfcie llista d'objectes generats
	// a partir de la llista d'objectes generats.
	void omplirLlistaGenerat();
	// Funcions d'actualitzaci de maxims i mnims emprada
	// per a trobar el mxim i el mnim d'una srie de valors.
	void actualitzaMaxMin(double v, double *min, double *max);
	void actualitzaMaxMin(long v, long *min, long *max);
	void actualitzaMaxMin(int v, int *min, int *max);
	// Els objectes generats.
	CLlistaObjectesGenerats m_objectes;
	// Omple l'objecte grfic Arbre de Classes a partir de la
	// llista de Classes.
	void omplirArbreClasses();
	// La llista de classes.
	CLlistaClasses m_classes;
	// L'objecte de configuraci de processat d'imatge.
	CDlgCfgImatge * m_cimg;
	// El dispositiu cmara.
	CDspCamara * m_camara;
	// Omple l'objecte d'interfcie de llista de No Classificats
	// a partir de les dades pertinents.
	void omplirLlistaNoClassificats();
	// Transforma la llista d'objectes generats en una llista
	// d'objectes tal com ho fa servir l'objecte d'identificaci.
	// Aquesta conversi s necessria nicament perqu
	// l'objecte identificaci ja t implementat el mtode
	// per a generar un fitxer en format .dob a partir
	// d'una llista d'objectes.
	void transformarObjectesGeneratsEnObjectes(CLlistaObjectesGenerats *og, CLlistaObjectes *o);
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DIALEGCREACIOCRITERIS_H__82ACBAA2_09F4_11D4_B60D_00C026C084F2__INCLUDED_)
