/////////////////////////////////////////////////////////////////
//
// Arxiu      : DlgCfgImatge.cpp
//
// Descripci : Arxiu de capalera de la classe que mant
//				i permet solicitar a l'usuari els parmetres
//				de configuraci del processat d'imatge.
//				La solicitud de dades a l'usuari, aix com la
//				seva vosualitzaci es fan mitjanant un
//				quadre de dileg.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "DlgCfgImatge.h"
#include "Sistema.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CDlgCfgImatge::CDlgCfgImatge(CWnd* pParent /*=NULL*/)
	: CDialog(CDlgCfgImatge::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgCfgImatge)
	m_emfatitzarHistograma = FALSE;
	m_altFinestra = 460;
	m_ampleFinestra = 620;
	m_llindarBinaritzacio = 120;
	m_ampladaSuavitzat = 30;
	m_minAmuntAvall = 5;
	m_numDilatacions = 2;
	m_numErosions = 2;
	m_xFinestra = 10;
	m_yFinestra = 10;
	m_formaProcessat = 2;
	m_tamanyMascara = 3;
	m_estilSiluetes = 2;
	m_ampladaSuavitzatAM = 20;
	m_minAmuntAvallAM = 10;
	//}}AFX_DATA_INIT
}

void CDlgCfgImatge::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgCfgImatge)
	DDX_Check(pDX, IDC_CHECK_EMFATITZAR, m_emfatitzarHistograma);
	DDX_Text(pDX, IDC_EDIT_ALT_FINESTRA, m_altFinestra);
	DDX_Text(pDX, IDC_EDIT_AMPLE_FINESTRA, m_ampleFinestra);
	DDX_Text(pDX, IDC_EDIT_BINARITZACIO_LLINDAR, m_llindarBinaritzacio);
	DDX_Text(pDX, IDC_EDIT_HISTSUAVITZAT, m_ampladaSuavitzat);
	DDX_Text(pDX, IDC_EDIT_MIN, m_minAmuntAvall);
	DDX_Text(pDX, IDC_EDIT_NUM_DILATACIONS, m_numDilatacions);
	DDV_MinMaxUInt(pDX, m_numDilatacions, 0, 15);
	DDX_Text(pDX, IDC_EDIT_NUM_EROSIONS, m_numErosions);
	DDV_MinMaxUInt(pDX, m_numErosions, 0, 15);
	DDX_Text(pDX, IDC_EDIT_X_FINESTRA, m_xFinestra);
	DDX_Text(pDX, IDC_EDIT_Y_FINESTRA, m_yFinestra);
	DDX_Radio(pDX, IDC_RADIO_PROCES, m_formaProcessat);
	DDX_Text(pDX, IDC_EDIT_MASCARA, m_tamanyMascara);
	DDV_MinMaxUInt(pDX, m_tamanyMascara, 2, 10);
	DDX_Radio(pDX, IDC_RADIO_VIS_PROP, m_estilSiluetes);
	DDX_Text(pDX, IDC_EDIT_HISTSUAVAM, m_ampladaSuavitzatAM);
	DDX_Text(pDX, IDC_EDIT_MINAM, m_minAmuntAvallAM);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDlgCfgImatge, CDialog)
	//{{AFX_MSG_MAP(CDlgCfgImatge)
	ON_BN_CLICKED(IDC_BOTO_APLICAR, OnBotoAplicar)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//
// Inicialitzaci de l'objecte.
// En primer lloc obt el nom de l'arxiu a on hi ha les dades, 
// a partir del punter al sistema. Desprs carrega l'esmentat
// arxiu.
//
void CDlgCfgImatge::inicialitzar(void *sistema)
{
	m_nomArxiu = ((CSistema *)sistema)->m_arxiuConfiguracioImatge;
	carregar();	
	Create(IDD_CONFIGURAR_IMATGE);
}

//
// Carrega l'estat de l'objecte des de l'arxiu de disc, el 
// nom del qual est en m_nomArxiu.
//
void CDlgCfgImatge::carregar()
{
	CFile fitxer;
	if (!fitxer.Open(m_nomArxiu, CFile::modeRead)) return;
	CArchive arxiu(&fitxer, CArchive::load);

	arxiu >> m_altFinestra ;
	arxiu >> m_ampladaSuavitzat ;
	arxiu >> m_ampleFinestra ;
	arxiu >> m_emfatitzarHistograma ;
	arxiu >> m_formaProcessat ;
	arxiu >> m_llindarBinaritzacio ;
	arxiu >> m_minAmuntAvall ;
	arxiu >> m_numDilatacions ;
	arxiu >> m_numErosions ;
	arxiu >> m_tamanyMascara ;
	arxiu >> m_xFinestra ;
	arxiu >> m_yFinestra ;
	arxiu >> m_estilSiluetes ;
	arxiu >> m_ampladaSuavitzatAM ;
	arxiu >> m_minAmuntAvallAM;

	arxiu.Close();
	fitxer.Close();
}

//
// Guarda les dades de l'objecte en l'arxiu el nom del qual
// est en la variable m_nomArxiu
//
void CDlgCfgImatge::guardar()
{
	CFile fitxer(m_nomArxiu, CFile::modeWrite | CFile::modeCreate);
	CArchive arxiu(&fitxer, CArchive::store);

	arxiu << m_altFinestra ;
	arxiu << m_ampladaSuavitzat ;
	arxiu << m_ampleFinestra ;
	arxiu << m_emfatitzarHistograma ;
	arxiu << m_formaProcessat ;
	arxiu << m_llindarBinaritzacio ;
	arxiu << m_minAmuntAvall ;
	arxiu << m_numDilatacions ;
	arxiu << m_numErosions ;
	arxiu << m_tamanyMascara ;
	arxiu << m_xFinestra ;
	arxiu << m_yFinestra ;
	arxiu << m_estilSiluetes ;
	arxiu << m_ampladaSuavitzatAM ;
	arxiu << m_minAmuntAvallAM;

	arxiu.Close();
	fitxer.Close();
}

//
// Realitza les tasques de finalitzaci de l'objecte, entre
// elles, guardar l'estat de l'objecte a disc.
//
void CDlgCfgImatge::finalitzar()
{
	guardar();
}

//
// Com que el quadre de dileg de configuraci de la imatge
// s no modal, es pot tenir actiu mentres es fan altres coses.
// Si es vol que les dades que hi hem introduit siguin emprades
// per l'aplicaci es pitja el bot Aplicar. A l'esmentat
// event hi respon la segent funci, que, cridant a UpdateData(true)
// aconsegueix que les dades que hi ha dins dels controls
// del quadre de dileg es copin dins de la variable
// associada.
//
void CDlgCfgImatge::OnBotoAplicar() 
{
	UpdateData(true);	
}

void CDlgCfgImatge::alternar()
{
	if (!estaActiu()) 
		ShowWindow(SW_SHOW);
	else 
		ShowWindow(SW_HIDE);
}

BOOL CDlgCfgImatge::estaActiu()
{
	return IsWindowVisible();
}

void CDlgCfgImatge::OnCancel() 
{
	ShowWindow(SW_HIDE);
}
