/////////////////////////////////////////////////////////////////
//
// Arxiu      : DlgVisHistograma.cpp
//
// Descripci : Arxiu d'implementaci de la classe de visualitzaci
//				de l'histograma d'una imatge.
//				El funcionament de l'objecte es basa en un timer.
//				Cada cert temps, especificat per l'usuari en
//				la configuraci general, es captura un
//				fotograma, es calcula l'histograma, es suavitza
//				segons els parmetres especificats per l'usuari
//				en la configuraci del processament d'imatge, 
//				se'n calculen els llindars de thresholding, 
//				tamb segons els parmetres especificats per
//				l'usuari en la configuraci del processat
//				d'imatge, i, finalment, es representen tant
//				els histogrames com els llindars de thresholding.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "DlgVisHistograma.h"
#include "Sistema.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CDlgVisHistograma::CDlgVisHistograma(CWnd* pParent /*=NULL*/)
	: CDialog(CDlgVisHistograma::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgVisHistograma)
	//}}AFX_DATA_INIT
}


void CDlgVisHistograma::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgVisHistograma)
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDlgVisHistograma, CDialog)
	//{{AFX_MSG_MAP(CDlgVisHistograma)
	ON_WM_PAINT()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//
// Inicialitzaci de l'objecte
// S'obt un punter a la cmara, a la configuraci del processament
// d'imatge i a la configuraci general.
// Tamb s'inicialitza la imatge dins la qual es captura el
// fotograma (del qual es calcula l'histograma).
//
void CDlgVisHistograma::inicialitzar(void *sistema)
{
	Create(IDD_HISTOGRAMA);
	m_camara=&(((CSistema *)sistema)->m_camara);
	m_imatge.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, (((CSistema *)sistema)->m_imgconfig).m_xFinestra, (((CSistema *)sistema)->m_imgconfig).m_yFinestra, (((CSistema *)sistema)->m_imgconfig).m_ampleFinestra, (((CSistema *)sistema)->m_imgconfig).m_altFinestra);
	m_configuracio = &(((CSistema *)sistema)->m_imgconfig);
	m_configGeneral = &(((CSistema *)sistema)->m_genconfig);
}

//
// Determina si la finestra de l'histograma s visible
//
BOOL CDlgVisHistograma::estaActiu()
{
	return IsWindowVisible();
}

//
// Alterna l'estat de la finestra entre visible i no visible
// Quan s'activa la finestra, tamb s'activa el timer. Quan
// es desactiva la finestra, es desactiva el timer.
// El timer s'encarrega de calcular i dibuixar l'histograma 
// cada cert temps, especificat per l'usuari.
//
void CDlgVisHistograma::alternar()
{
	if (!estaActiu()) {
		ShowWindow(SW_SHOW);
		calcularHistogrames();
		SetTimer(HIS_TIMER_ACTUALITZACIO, m_configGeneral->m_tempsRefresc, NULL);
	}
	else {
		ShowWindow(SW_HIDE);
		KillTimer(HIS_TIMER_ACTUALITZACIO);
	}
}

//
// Mtide OnPaint. Es crida quan la finestra necessita ser
// redibuixada. Aqu dins hi va el codi que representa
// grficament l'histograma, l'histograma suavitzat i els
// llindars de thresholding.
//
void CDlgVisHistograma::OnPaint() 
{
	CPaintDC dc(this);
	int i;
	CPen elNegre(PS_SOLID, 1, RGB(0,0,0));
	CPen elVermell(PS_SOLID, 1, RGB(255,0,0));
	CPen *elVell = dc.SelectObject( &elNegre );

	// Dibuix dels histogrames
	for (i=0;i<IMG_NUM_COLORS;i++) {
		dc.SelectObject( &elNegre );
		dc.MoveTo(i+15, 115);
		dc.LineTo(i+15,115-(((m_histograma[i])*90)/9999));
		dc.MoveTo(i+15, 235);
		dc.LineTo(i+15,235-(((m_histogramaSuavitzat[i])*90)/9999));
	}

	// Dibuix dels llindars de thresholding
	for (i=0;i<m_llindars->GetSize();i++) {
		dc.SelectObject( &elVermell );
		dc.MoveTo((*m_llindars)[i]+15, 115);
		dc.LineTo((*m_llindars)[i]+15, 25);
		dc.MoveTo((*m_llindars)[i]+15, 235);
		dc.LineTo((*m_llindars)[i]+15, 145);
	}

	dc.SelectObject(elVell);
}

//
// Cpia d'histogrames. Un histograma no s ms que un array
// de IMG_NUM_COLORS sencers.
//
void CDlgVisHistograma::copiarHistograma(int * origen, int * desti)
{
	int i;
	for (i=0;i<IMG_NUM_COLORS;i++) 
		desti[i]=origen[i];
}

//
// Clcul dels histogrames i dels llindars. Aix comprn:
// * Captura del fotograma
// * Clcul del histograma (mtode propi de CPriImatge)
// * Cpia del histograma
// * Suavitzat de l'histograma. (mtode propi de CPriImatge)
// * Localitzaci dels llindars de thresholding de l'histograma
//   suavitzat (mtode de CPriImatge)
//
void CDlgVisHistograma::calcularHistogrames()
{
	m_camara->posarEnMemoria(&m_imatge);
	m_imatge.calcularHistograma(9999, m_configuracio->m_emfatitzarHistograma);
	copiarHistograma(m_imatge.obtenirHistograma(), m_histograma);
	m_imatge.suavitzarHistograma(m_configuracio->m_ampladaSuavitzat);
	copiarHistograma(m_imatge.obtenirHistograma(), m_histogramaSuavitzat);
	m_imatge.localitzarMinims(m_configuracio->m_minAmuntAvall, m_configuracio->m_minAmuntAvall);
	m_llindars=m_imatge.obtenirLlindars();
}

//
// Funci de resposta al timer. Quan s'activa el timer, 
// es calculen els histogrames i es redibuixa la finestra
// A ms, si la finestra no s visible, es desactiva el timer
// (si no es visualitza no fa falta consumir temps de CPU
// en el seu clcul)
//
void CDlgVisHistograma::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent==HIS_TIMER_ACTUALITZACIO) {
		calcularHistogrames();
		RedrawWindow();
		if (!estaActiu()) KillTimer(HIS_TIMER_ACTUALITZACIO);
	}
}

