/////////////////////////////////////////////////////////////////
//
// Arxiu      : DlgVisHistograma.h
//
// Descripci : Arxiu d'implementaci de la classe de visualitzaci
//				de l'histograma d'una imatge.
//				El funcionament de l'objecte es basa en un timer.
//				Cada cert temps, especificat per l'usuari en
//				la configuraci general, es captura un
//				fotograma, es calcula l'histograma, es suavitza
//				segons els parmetres especificats per l'usuari
//				en la configuraci del processament d'imatge, 
//				se'n calculen els llindars de thresholding, 
//				tamb segons els parmetres especificats per
//				l'usuari en la configuraci del processat
//				d'imatge, i, finalment, es representen tant
//				els histogrames com els llindars de thresholding.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#if !defined(AFX_DIALEGHISTOGRAMA_H__AAEA13B2_F3FD_11D3_B5DB_00C026C084F2__INCLUDED_)
#define AFX_DIALEGHISTOGRAMA_H__AAEA13B2_F3FD_11D3_B5DB_00C026C084F2__INCLUDED_

//
// Arxius de capalera
//
#include "DspCamara.h"	
#include "PriImatge.h"	
#include "DlgCfgImatge.h"
#include "DlgCfgGeneral.h"
#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//
// Constants de la classe
//
#define HIS_TIMER_ACTUALITZACIO 1

class CDlgVisHistograma : public CDialog
{
public:
	// Constructor
	CDlgVisHistograma(CWnd* pParent = NULL);
	// Alterna l'estat del quadre de dileg entre visible
	// i invisible.
	void alternar();
	// Determina si el quadre de dileg s visible.
	BOOL estaActiu();
	// Inicialitza l'objecte.
	void inicialitzar(void *sistema);

// Dialog Data
	//{{AFX_DATA(CDlgVisHistograma)
	enum { IDD = IDD_HISTOGRAMA };
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CDlgVisHistograma)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CDlgVisHistograma)
	afx_msg void OnPaint();
	afx_msg void OnTimer(UINT nIDEvent);
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	// Punter als llindars calculats per l'histograma actual
	CIntArray * m_llindars;
	// Punter a la configuraci general, a la qual s'especifica
	// la freqncia d'actualitzaci de l'histograma.
	CDlgCfgGeneral * m_configGeneral;
	// Punter a les dades de configuraci del processat d'imatge, 
	// del qual se n'extreuen els parmetres de creaci de
	// l'histograma (emfatitzat, amplada del suavitzat, mnim 
	// de pujades o baixades per a crear llindar, ...)
	CDlgCfgImatge * m_configuracio;
	// La imatge de la qual se'n calcula l'histograma.
	CPriImatge m_imatge;
	// Calcula l'histograma i l'histograma suavitzat
	void calcularHistogrames();
	// Copia un histograma des de origen cap a desti
	void copiarHistograma(int *origen, int *desti);
	// L'histograma suavitzat
	int m_histogramaSuavitzat[IMG_NUM_COLORS];
	// L'histograma
	int m_histograma[IMG_NUM_COLORS];
	// Punter al dispositiu Camara.
	CDspCamara *m_camara;
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_DIALEGHISTOGRAMA_H__AAEA13B2_F3FD_11D3_B5DB_00C026C084F2__INCLUDED_)
