/////////////////////////////////////////////////////////////////
//
// Arxiu      : DlgVisIdentificats.cpp
//
// Descripci : Visualitza objectes prviament identificats.
//				Per cada objecte en mostra les dades de la
//				classe a la qual pertany, s a dir, el nom
//				i els parmetres d'agafada (DX, DY i Rotaci).
//				Quan es fa click sobre un dels objectes, 
//				es passa a la classe vista la informaci
//				necessria per a que superposi a la imatge
//				de l'objecte el sistema de coordenades de
//				l'objecte en qesti. 
//				El sistema de coordenades es construeix
//				assumint un eix X en la direcci del vector
//				d'agafada per defecte, i un eix Y perpendicular
//				a ell i sobre el pl de la pantalla.
//				L'escala del sistema s tal que la longitud del
//				vector d'agafada per defecte es considera la
//				unitat.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "DlgVisIdentificats.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

CDlgVisIdentificats::CDlgVisIdentificats(CWnd* pParent /*=NULL*/)
	: CDialog(CDlgVisIdentificats::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDlgVisIdentificats)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}

void CDlgVisIdentificats::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDlgVisIdentificats)
	DDX_Control(pDX, IDC_LLISTA_OBJECTES, m_listCtrlObjectes);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CDlgVisIdentificats, CDialog)
	//{{AFX_MSG_MAP(CDlgVisIdentificats)
	ON_NOTIFY(NM_CLICK, IDC_LLISTA_OBJECTES, OnClickLlistaObjectes)
	ON_BN_CLICKED(IDC_BOTO_AGAFAR, OnBotoAgafar)
	ON_BN_CLICKED(IDC_BOTO_MOURE0, OnBotoMoure0)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//
// El mtode OnInitDialog() s crida just abans de mostrar
// el quadre de dileg.
// El que es fa s crear les columnes del quadre de dileg i anar
// inserint els elements de m_llistaObjectes en el control, 
// m_listCtrlObjectes.
//
BOOL CDlgVisIdentificats::OnInitDialog() 
{
	LV_ITEM li;
	int i;

	m_listCtrlObjectes.SubclassDlgItem(IDC_LLISTA_OBJECTES, this);
	m_listCtrlObjectes.DeleteAllItems();

	m_listCtrlObjectes.InsertColumn(1, "Etiqueta", LVCFMT_LEFT, 140);
	m_listCtrlObjectes.InsertColumn(2, "DX", LVCFMT_LEFT, 90);
	m_listCtrlObjectes.InsertColumn(3, "DY", LVCFMT_LEFT, 90);
	m_listCtrlObjectes.InsertColumn(4, "R", LVCFMT_LEFT, 90);

	li.mask=LVIF_TEXT;
	li.pszText=new char[50];

	for (i=0;i<m_llistaObjectes.GetSize();i++) {

		li.iItem=i;
		li.iSubItem=0;
		strcpy(li.pszText, m_llistaObjectes[i].m_objecte->obtenirNom());
		m_listCtrlObjectes.InsertItem(&li);

		li.iItem=i;
		li.iSubItem=1;
		sprintf(li.pszText, "%f", m_llistaObjectes[i].m_objecte->obtenirDespXAgafada());
		m_listCtrlObjectes.SetItem(&li);

		li.iItem=i;
		li.iSubItem=2;
		sprintf(li.pszText, "%f", m_llistaObjectes[i].m_objecte->obtenirDespYAgafada());
		m_listCtrlObjectes.SetItem(&li);
		
		li.iItem=i;
		li.iSubItem=3;
		sprintf(li.pszText, "%f", m_llistaObjectes[i].m_objecte->obtenirRotacioAgafada());
		m_listCtrlObjectes.SetItem(&li);
	}

	return TRUE;  
}

//
// La segent funci s cridada quan es fa click sobre
// un objecte de la llista.
// En primer lloc es determina sobre quin dels elements s'ha
// fet click, i se n'extreuen les dades de posici i orientaci.
// A partir d'aqu, s'indica a la vista qu s el que ha
// de dibuixar.
//
void CDlgVisIdentificats::OnClickLlistaObjectes(NMHDR* pNMHDR, LRESULT* pResult) 
{
	int elItem, xc, yc, xmn, ymn, dx, dy, xr, yr, xo, yo;
	double distancia, rotacio, orientacio, alfa;
	CPoint p;
	// Determinam sobre quin item s'ha fet click.
	elItem=m_listCtrlObjectes.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	if (elItem==-1) return;
	// Extreiem les propietats de l'objecte
	xc=m_llistaObjectes[elItem].m_xCentre;
	yc=m_llistaObjectes[elItem].m_yCentre;
	xmn=m_llistaObjectes[elItem].m_xDMin;
	ymn=m_llistaObjectes[elItem].m_yDMin;
	orientacio=m_llistaObjectes[elItem].m_orientacio;
	rotacio=(m_llistaObjectes[elItem].m_objecte->obtenirRotacioAgafada())*2*3.14159/360;
	distancia=sqrt((xmn-xc)*(xmn-xc)+(ymn-yc)*(ymn-yc));
	alfa=rotacio-orientacio;

	p=traslladar((rotar((CPoint((int)(-distancia), 0)),(int)orientacio)), xc, yc);
	xmn=p.x;
	ymn=p.y;
	
	dx=(int)(((m_llistaObjectes[elItem].m_objecte->obtenirDespXAgafada())*distancia)/100);
	dy=(int)(((m_llistaObjectes[elItem].m_objecte->obtenirDespYAgafada())*distancia)/100);

	p=traslladar((rotar((traslladar((rotar((traslladar(CPoint(xmn, ymn), -xc, -yc)), alfa)),-dx, -dy)), orientacio)), xc, yc);
	xr=p.x;
	yr=p.y;
	
	p=traslladar((rotar((traslladar((rotar((traslladar(CPoint(xc, yc), -xc, -yc)), alfa)),-dx, -dy)), orientacio)), xc, yc);
	xo=p.x;
	yo=p.y;

	// S'indica a la vista qu ha de dibuixar.
	m_laVista->m_xc=xc;
	m_laVista->m_yc=yc;
	m_laVista->m_xmn=xmn;
	m_laVista->m_ymn=ymn;
	m_laVista->m_xa=xr;
	m_laVista->m_ya=yr;
	m_laVista->m_xo=xo;
	m_laVista->m_yo=yo;
	m_laVista->m_dibuixarOrientacio=true;
	m_laVista->RedrawWindow();
	*pResult = 0;
}

//
// Quan es tanca el quadre de dileg s'ha d'indicar a la
// vista que deixi de dibuixar res referent als objectes
// identificats, si s que ho estava fent.
//
CDlgVisIdentificats::~CDlgVisIdentificats()
{
	m_laVista->m_dibuixarOrientacio=false;
	m_laVista->RedrawWindow();
}

//
// Funcions de rotaci i trasllaci.
// S'han implementat a partir de la classe CPoint, de la forma
// en que es veu per a facilitar la concatenaci d'operacions.
//
CPoint CDlgVisIdentificats::rotar(CPoint punt, double angle)
{
	return (CPoint((int)(cos(angle)*punt.x - sin(angle)*punt.y),(int)(sin(angle)*punt.x + cos(angle)*punt.y)));
}

CPoint CDlgVisIdentificats::traslladar(CPoint punt, int dx, int dy)
{
	return (CPoint((int)(punt.x+dx), (int)(punt.y+dy)));
}

//
// Aquesta funci s'activa quan l'usuari pitja sobre el bot
// 'Senyalar'.
//
void CDlgVisIdentificats::OnBotoAgafar() 
{
	int elItem, xc, yc, xmn, ymn;
	double orientacio;
	CPoint p;
	// En primer lloc es comprova quin dels elements de la
	// llista est seleccionat. Si no n'hi ha cap, retorna.
	elItem=m_listCtrlObjectes.GetNextItem(-1, LVNI_ALL | LVNI_SELECTED);
	if (elItem==-1) return;
	// S'obtenen les dades de l'objecte seleccionat.
	xc=m_llistaObjectes[elItem].m_xCentre;
	yc=m_llistaObjectes[elItem].m_yCentre;
	xmn=m_llistaObjectes[elItem].m_xDMin;
	ymn=m_llistaObjectes[elItem].m_yDMin;
	orientacio=m_llistaObjectes[elItem].m_orientacio;

	// Es calcula el vector d'agafada.
	m_laVista->m_sistema.calcularAgafada(m_llistaObjectes[elItem].m_objecte, &xc, &yc, &orientacio, xmn, ymn);

	// Es transforma al sistema de coordenades del robot.
	m_laVista->m_sistema.pantallaARobot(&xc, &yc, &orientacio);
	m_laVista->m_sistema.m_robot.posarDiferit();
	m_laVista->m_sistema.m_robot.velocitat(30);
	// Es mou el robot a sobre de l'objecte
	m_laVista->m_sistema.m_robot.moure(xc, yc, ROB_Z_DEFECTE, ROB_PITCH_DEFECTE, (int)orientacio);
	m_laVista->m_sistema.m_robot.executarPrograma();
	m_laVista->m_sistema.m_robot.posarDirecte();
}

// Envia el robot a la posici 0 quan es pitja el bot de
// 'MOVE 0'.
void CDlgVisIdentificats::OnBotoMoure0() 
{
	m_laVista->m_sistema.m_robot.posarDirecte();
	m_laVista->m_sistema.m_robot.moure(0);
}
