/////////////////////////////////////////////////////////////////
//
// Arxiu      : DspCamara.cpp
//
// Descripci : Arxiu d'implementaci de la classe d'interfcie
//				entre l'aplicaci i el dispositiu extern 
//				cmara.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "iv4.h"
#include "DspCamara.h"
#include "Sistema.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CDspCamara::CDspCamara()
{
	// Inicialitzaci de l'objecte a l'estat de "Sense errors"
	m_error=0;
	m_gravant=false;
	m_inicialitzada=false;
}

CDspCamara::~CDspCamara()
{
}

void CDspCamara::iniciarGravacio()
{
	// Cridada a la funci de la llibreria de la cmara
	if (!m_gravant) {
		IV4StartGrab(IV4_Continue_Grab); 
		m_gravant=true;
	}
}

void CDspCamara::aturarGravacio()
{
	if (m_gravant) {
		IV4StopGrab();
		IV4StartGrab(IV4_Single_Grab);
		m_gravant=false;
	}
}

void CDspCamara::capturarFotograma()
{
	// Cridada a la funci de la llibreria de la cmara
	// S'atura la gravacio (per si de cas s'estava gravant)
	if (m_gravant) {
		IV4StopGrab();
		m_gravant=false;
	}
	else
		IV4StartGrab(IV4_Single_Grab);
}

void CDspCamara::moureFinestra(int x, int y)
{
	// Cridada a la funci de la llibreria de la cmara
	// Aquesta funci s'ha de cridar quan es generi un 
	// event de moviment de la finestra que cont la imatge
	// capturada per la cmara.
	IV4SetImageWndOnScreen(x, y);
}

void CDspCamara::inicialitzar(void *sistema, int x , int y)
{
	int tmp;

	if (m_inicialitzada) return;
	
	// S'obre la capturadora. Aix no inicialitza res.
	// En cas d'error es retorna el codi d'error
	// corresponent.
	tmp=IV4Open(IV4_SDK_VERSION);

	if (tmp!=IV4_OK && tmp!=IV4_ERROR) {
		m_error=1;
		return;
	}

	// Inicialitzaci de la capturadora.
	if (IV4Init(NULL)!=IV4_OK) {
		m_error=2;
		return;
	}

	// Configuraci de la capturadora.
	// Stndard PAL
	if (IV4SetTVStandard(IV4_Camera_PAL)!=IV4_OK) {
		AfxMessageBox("Error IV4SetTVStandard.");
		return;
	}

	if (IV4SetGrabWindow(IV4_Frame_Mode | IV4_Grab_Odd, 0x0055, 0x0010, CAM_AMPLE_DEFECTE, CAM_ALT_DEFECTE)!=IV4_OK) {
		AfxMessageBox("Error IV4SetGrabWindow.");
		return;
	}

	if (IV4SetVideoFrame(IV4_Single_Image_Mode, 0, 0, 0)!=IV4_OK) {
		AfxMessageBox("Error IV4SetVideoFrame.");
		return;
	}

	if (IV4SetImageFrameOnVideoFrame(0, 0, CAM_AMPLE_DEFECTE, CAM_ALT_DEFECTE)!=IV4_OK) {
		AfxMessageBox("Error IV4SetImageFrameOnVideoFrame.");
		return;
	}

	if (IV4SetImageWndOnScreen(0,0)!=IV4_OK) {
		AfxMessageBox("Error IV4SetImageWndOnScreen");
		return;
	}

	// Definici del Overlay. El Overlay s el mtode que 
	// proporciona la cmara per a poder posar la imatge dins
	// d'una finestra. De fet, sols s dna la sensaci de qu
	// s aix.
	// Amb el Overlay, el que es fa s que la imatge capturada
	// sols es mostra sobre els pixels de la pantalla que sn
	// d'un determinat color. Si es fa que l'interior de la
	// finestra (de windows) a on ho volem visualitzar sigui d'un
	// color que no es troba en cap altre punt de la pantalla
	// (un color poc com) i es combina aix amb "moureFinestra"
	// sembla que tenim la imatge dins de la finestra.
	// La primera funci defineix quin s aquest color.
	// La segona indica que volem fer el que s'ha descrit abans.
	if (IV4SetRGBColorKey(CAM_R_OVERLAY, CAM_G_OVERLAY, CAM_B_OVERLAY)!=IV4_OK) {
		AfxMessageBox("Error IV4SetRGBColorKey");
		return;
	}
						  
	if (IV4SetOverlayMode(IV4_Color_Key_Mode)!=IV4_OK) {
		AfxMessageBox("Error IV4SetOverlayMode.");
		return;
	}

	m_gravant=false;
	m_inicialitzada=true;
}

void CDspCamara::finalitzar()
{
    // Tanca la capturadora de video i retorna l'error pertinent
	// si no ha anat b.

	aturarGravacio();
	IV4SetOverlayMode(IV4_No_Overlay_Mode);
	if (IV4Close()!=IV4_OK) 
		m_error = 3;
	m_gravant=false;
}

int CDspCamara::obtenirError()
{
	// Retorna el codi d'error
	return m_error;
}


bool CDspCamara::estaGravant()
{
	// Retorna l'estat de la cmara
	return m_gravant;
}

//
// Passa una imatge des del frame buffer de la cmara
// a un objecte de classe CPriImatge.
// Tot es fa a travs de les funcions que ofereix
// el SDK de la cmara.
//
void CDspCamara::posarEnMemoria(CPriImatge * laImg)
{
	int i, j;
	TPixel **laMatriu=laImg->obtenirMatriu();
	int ample=laImg->obtenirAmpleImatge();
	int alt=laImg->obtenirAltImatge();

	unsigned short tmp[2000];

	IV4WaitForVSync();
	for (i=0;i<alt;i++) {
		if (IV4RdPixelLine(0, i, ample, (void *)tmp)!=IV4_OK) {
			m_error = CAM_ERROR_ESCRIVINT;
			AfxMessageBox("Error llegint del frame buffer");
			return ;
		}
		for (j=0;j<ample;j++) 
			laMatriu[i][j]=(TPixel)(tmp[j] & 0x00FF);
	}
	IV4WaitForVSync();
}

//
// Passa una imatge des d'un objecte de la classe CPriImatge
// cap al frame buffer de la cmara.
// Tot es fa a travs de les funcions que ofereix
// el SDK de la cmara.
//
void CDspCamara::llegirDeMemoria(CPriImatge * laImg)
{
	int i, j;
	TPixel **laMatriu=laImg->obtenirMatriu();
	int ample=laImg->obtenirAmpleImatge();
	int alt=laImg->obtenirAltImatge();
	TPixel *laLinia=new TPixel[ample];

	IV4WaitForVSync();
	for (i=0;i<alt;i++) {
		for (j=0;j<ample;j++) 
		laLinia[j]=laMatriu[i][j]|0x8000;
		if (IV4WtPixelLine(0, i, ample, (void *)laLinia)!=IV4_OK) {
			m_error = CAM_ERROR_LLEGINT;			
			AfxMessageBox("Error escrivint en el Frame Buffer");
			return;
		}
	}
	IV4WaitForVSync();
}
