/////////////////////////////////////////////////////////////////
//
// Arxiu      : DspCamara.h
//
// Descripci : Arxiu de capalera de la classe d'interfcie
//				entre l'aplicaci i el dispositiu extern 
//				cmara.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#if !defined(AFX_CAMARA_H__24E55366_9209_11D3_B579_00C026C084F2__INCLUDED_)
#define AFX_CAMARA_H__24E55366_9209_11D3_B579_00C026C084F2__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#include "PriImatge.h"
#include "iv4.h"

//
// Definici de constants del programa
//
#define CAM_ALT_DEFECTE 480
#define CAM_AMPLE_DEFECTE 640
#define CAM_R_OVERLAY 255
#define CAM_G_OVERLAY 0
#define CAM_B_OVERLAY 255

//
// Definici dels errors
//
#define CAM_ERROR_NO             0
#define CAM_ERROR_OBERTURA       1
#define CAM_ERROR_INICIALITZACIO 2
#define CAM_ERROR_TANCAMENT      3
#define CAM_ERROR_ESCRIVINT      4
#define CAM_ERROR_LLEGINT        5

class CDspCamara  
{
public:
	// Constructor
	CDspCamara();
	// Inicialitza la cmara. Configura la cmara amb els
	// parmetres que se li indiquen. Alguns parmetres es
	// configuren amb valors per defecte. Un ajustament
	// mes fi de la cmara requerir modificar el codi.
	void inicialitzar(void *sistema, int x=CAM_AMPLE_DEFECTE, int y=CAM_ALT_DEFECTE);
	// Captura un nic fotograma.
	void capturarFotograma();
	// Atura la gravaci i deixa en el Frame Buffer el darrer 
	// fotograma capturat.
	void aturarGravacio();
	// Comena la gravaci.
	void iniciarGravacio();
	// Copia una imatge des d'un objecte de classe CPriImatge
	// cap al frame buffer de la cmara.
	void llegirDeMemoria(CPriImatge *laImg);
	// Copia una imatge des del frame buffer de la cmara cap
	// a un objecte de tipus CPriImatge.
	void posarEnMemoria(CPriImatge *laImg);
	// Retorna true o false segons la cmara estigui o no en 
	// estat de gravaci.
	bool estaGravant(); 
	// Finalitza l'utilitzaci de la cmara.
	void finalitzar();	
	// Mou la finestra de visualitzaci de l'imatge capturada 
	// per la cmara.
	void moureFinestra(int x, int y);
	// Retorna el codi del darrer error que s'ha produit.
	int obtenirError();	
	// Destructor
	virtual ~CDspCamara();
private:
	// Estat intern de l'objecte
	// Est la capturadora inicialitzada?
	bool m_inicialitzada;
	// Estam gravant?
	bool m_gravant;
	// S'ha produit algun error?
	int m_error;	
};

#endif // !defined(AFX_CAMARA_H__24E55366_9209_11D3_B579_00C026C084F2__INCLUDED_)

