/////////////////////////////////////////////////////////////////
//
// Arxiu      : DspRobot.cpp
//
// Descripci : La classe DspRobot proporciona mtodes que
//				serveixen d'interfcie entre l'aplicaci
//				i el controlador del robot.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "DspRobot.h"
#include "Sistema.h"
#include <limits.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CDspRobot dialog


CDspRobot::CDspRobot(CWnd* pParent /*=NULL*/)
	: CDialog(CDspRobot::IDD, pParent)
{
	//{{AFX_DATA_INIT(CDspRobot)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
}


void CDspRobot::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CDspRobot)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CDspRobot, CDialog)
	//{{AFX_MSG_MAP(CDspRobot)
	ON_MESSAGE(WM_COMM_RXCHAR, OnCommunication)		
		// NOTE: the ClassWizard will add message map macros here
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDspRobot message handlers

void CDspRobot::definirPunt(int pt, int x, int y, int z, int pitch, int roll)
{
	char tmp[100];
	sprintf(tmp, "TEACH %d%s", pt, m_sl);
	processarComanda(tmp);
	sprintf(tmp, "%d%s", x, m_sl);
	processarComanda(tmp);
	sprintf(tmp, "%d%s", y, m_sl);
	processarComanda(tmp);
	sprintf(tmp, "%d%s", z, m_sl);
	processarComanda(tmp);
	sprintf(tmp, "%d%s", pitch, m_sl);
	processarComanda(tmp);
	sprintf(tmp, "%d%s", roll, m_sl);
	processarComanda(tmp);
}

void CDspRobot::moure(int p)
{
	char tmp[200]; 
	sprintf(tmp, "MOVE %d%s", p, m_sl);
	processarComanda(tmp);
}

void CDspRobot::moure(int x, int y, int z, int pitch, int roll)
{
	// Aquesta versi del mtode "moure" permet moure
	// el robot directament a les coordenades
	// especificades. Per aix, es defineix un punt temporal
	// i es mou cap al punt temporal.
	// S'ha d'evitar que els programes ACL que sn dins del
	// controlador facin servir aquest punt.
	definirPunt(ROB_PUNT_TEMPORAL, x, y, z, pitch, roll);
	moure(ROB_PUNT_TEMPORAL);
}

void CDspRobot::velocitat(int v)
{
	char tmp[200]; 
	sprintf(tmp, "SPEED %d%s", v, m_sl);
	processarComanda(tmp);
}

void CDspRobot::home()
{
	char tmp[200]; 
	sprintf(tmp, "HOME%s", m_sl);
	processarComanda(tmp);
}

void CDspRobot::enviarComanda(const char * comanda)
{
	char tmp[200]; 
	sprintf(tmp, "%s%s", comanda, m_sl);
	processarComanda(tmp);
}

void CDspRobot::posarDirecte()
{
	m_instruccions.RemoveAll();
	m_directe=true;
}

void CDspRobot::posarDiferit()
{
	m_instruccions.RemoveAll();
	m_directe=false;
}

void CDspRobot::processarComanda(const char * comanda)
{
	if (m_directe) executarComanda(comanda);
	else m_instruccions.Add(comanda);
}

void CDspRobot::executarComanda(const char * comanda)
{
	m_port.WriteToPort(comanda, true, ">", 1);
}

void CDspRobot::executarPrograma()
{
	int i;
	for (i=0;i<m_instruccions.GetSize();i++)
		executarComanda(m_instruccions[i]);
	m_instruccions.RemoveAll();
}

void CDspRobot::inicialitzar(void *sistema)
{
	m_directe=true;             // Estat inicial : directe
	m_instruccions.RemoveAll(); // Es buida el buffer d'instruccions
	m_recepcio.RemoveAll();		// Es buida el buffer de recepci
	m_tempStr=_T("");
	m_numPort=(((CSistema *)sistema)->m_robconfig.m_numport);
	m_port.InitPort(this, 4096, m_numPort+1); 
								// Comunicaci a 4096 bps
	m_port.StartMonitoring();	// Es comena a monitoritzar el 
								// port de comunicacions al qual
								// est connectat el robot.
	m_sl[0]=ROB_SALT_LINIA;			// Salt de lnia, carcter que
								// el robot interpreta com 
								// "executa comanda"
	m_sl[1]=0;
	m_dialegANotificar=NULL;	// Punter a la finestra a la qual
								// es notifiquen els missatges
								// rebuts des del robot, si s
								// que es vol notificar a alg.
	m_noMostrar=(((CSistema *)sistema)->m_robconfig.m_nomostrar);
								// Carcters que no s'han de 
								// visualitzar en el terminal.
	m_charMin=(((CSistema *)sistema)->m_robconfig.m_asciiminim);
								// Carcter mnim a visualitzar.
	Create(IDD_STUFF);	// El dileg ha d'estar creat per tal que
						// respongui als events generats per 
						// CSerialPort.
}

bool CDspRobot::obtenirPunt(int pt, int * x, int * y, int * z, int * pitch, int * roll)
{
	char tmp[200];
	int i;
	sprintf(tmp, "LISTPV %d%s", pt, m_sl);
	executarComanda(tmp);

	//
	// Obtenci de la coordenada X
	//
	i=posicioEnString('X', m_recepcio[m_recepcio.GetSize()-1]);
	if (i==-1) 
		return false;	// No s'ha trobar el carcter "X". 
						// Probablement sigui degut a que 
						// el controlador hagi retornat
						// un missatge d'error, de punt no
						// definit.
	if (!obtenirCadena(i+2, m_recepcio[m_recepcio.GetSize()-1], tmp)) return false; 
						// Es bota els espais que hi ha a partir
						// de la posici i+2, i posa la cadena
						// que va des de la posici obtinguda
						// botant els espais fins a trobar un 
						// nou espai dins de tmp.
	*x=atoi(tmp);

	//
	// Obtenci de la coordenada Y
	//
	i=posicioEnString('Y', m_recepcio[m_recepcio.GetSize()-1]);
	if (i==-1) 
		return false;	
	if (!obtenirCadena(i+2, m_recepcio[m_recepcio.GetSize()-1], tmp)) return false; 
	*y=atoi(tmp);

	//
	// Obtenci de la coordenada Z
	//
	i=posicioEnString('Z', m_recepcio[m_recepcio.GetSize()-1]);
	if (i==-1) 
		return false;
	if (!obtenirCadena(i+2, m_recepcio[m_recepcio.GetSize()-1], tmp)) return false; 
	*z=atoi(tmp);

	//
	// Obtenci de la coordenada Pitch
	//
	i=posicioEnString('P', m_recepcio[m_recepcio.GetSize()-1]);
	if (i==-1) 
		return false;	
	if (!obtenirCadena(i+2, m_recepcio[m_recepcio.GetSize()-1], tmp)) return false; 
	*pitch=atoi(tmp);

	//
	// Obtenci de la coordenada Roll
	//
	i=posicioEnString('R', m_recepcio[m_recepcio.GetSize()-1]);
	if (i==-1) 
		return false;	
	if (!obtenirCadena(i+2, m_recepcio[m_recepcio.GetSize()-1], tmp)) return false; 
	*roll=atoi(tmp);

	return true;
}

LONG CDspRobot::OnCommunication(WPARAM ch, LPARAM port)
// Funci de resposta al missatge de windows enviat per la
// classe CSerialPort indicant que ha rebut un carcter.
{
	if (ch!=(WPARAM)m_sl[0]) { 
		// Mentre no es rebi un salt de lnia es van emmagatzemant
		// els carcters rebuts dins de la cadena temporal
		// m_tempStr.
		m_tempStr+=ch;
	}
	else {
		// Quan s'ha rebut un carcter de salt de lnia es
		// guarda tota la cadena temporal en el buffer de
		// recepci i s'envia un missatge (de Windows) a la 
		// finestra a la qual s'ha indicat que es fes.
		m_recepcio.Add(m_tempStr);
		m_tempStr=_T("");
		if (m_dialegANotificar!=NULL)
			::PostMessage((m_dialegANotificar)->m_hWnd, WM_ROB_REBUT, (WPARAM)0, (LPARAM)0);
	}
	return 0;
}


CString CDspRobot::obtenirRecepcio()
{
	int i, j;
	CString tmp, tmp2(" ", 300);
	tmp=m_recepcio[m_recepcio.GetSize()-1];
	j=0;
	for (i=0;i<tmp.GetLength();i++) {
		if (tmp[i]!=m_sl[0] && tmp[i]>=m_charMin && !caracterEnString(tmp[i], m_noMostrar)) {
			tmp2.SetAt(j, tmp[i]);
			j++;
		}
	}
	tmp2.SetAt(j, (char)0);
	return tmp2;
}


bool CDspRobot::caracterEnString(char ch, CString st)
{
	int i;
	for (i=0;i<st.GetLength();i++)
		if (st[i]==ch) return true;
	return false;
}

int CDspRobot::posicioEnString(char ch, CString st)
{
	int i;
	for (i=0;i<st.GetLength();i++)
		if (st[i]==ch) return i;
	return -1;
}

bool CDspRobot::obtenirCadena(int pos, CString s, char * tmp)
{
	int i, j;
	for (i=pos;s[i]==' ' && i<s.GetLength();i++);
	if (i==s.GetLength()) return false;
	j=0;
	for (j=0;j+i<s.GetLength() && s[j+i]!=' ';j++)
		tmp[j]=s[j+i];
	tmp[j]=0;
	return true;
}

void CDspRobot::obrir()
{
	char tmp[200]; 
	sprintf(tmp, "OPEN%s", m_sl);
	processarComanda(tmp);
}

void CDspRobot::tancar()
{
	char tmp[200]; 
	sprintf(tmp, "CLOSE%s", m_sl);
	processarComanda(tmp);
}

void CDspRobot::modificarDialegANotificar(CWnd * nouDialeg)
{
	m_dialegANotificar=nouDialeg;
}

void CDspRobot::abortar()
{
	char tmp[200];
	sprintf(tmp, "a%s", m_sl);
	executarComanda(tmp);
}

void CDspRobot::enviarPrograma(CString nomArxiu, CString nomPrograma)
{
	CString liniaText=_T("");
	CFile fitxer;
	if (!fitxer.Open(nomArxiu, CFile::modeRead)) return;
	CArchive arxiu(&fitxer, CArchive::load);
	bool directe=m_directe;
	bool sortir=false;
	int pos=0;
	char tmp[100];
	m_directe=true;

	sprintf(tmp, "EDIT %s", nomPrograma.Left(5));
	enviarComanda(tmp);
	enviarComanda("Y");
	// Poden passar dues coses:
	// * Que el programa no existeixi. D'aquesta manera, el 
	//   "Y" respon a la pregunta "Do you want to create it?"
	// * Que el programa existeixi. D'aquesta manera el sistema
	//   respondr a "Y" amb un missatge d'error, per continuar
	//   en mode de recepci de comandes.
	while (arxiu.ReadString(liniaText) && !sortir) {
		pos++;
		sprintf(tmp, "%s%s", liniaText, m_sl);
		m_port.WriteToPort(tmp, true, "?", 1);
		if (m_recepcio[m_recepcio.GetSize()-1].Find("Unrecognized")!=-1) {
			sprintf(tmp, "Error en la linia %d. Comanda ACL no vlida", pos);
			AfxMessageBox(tmp);
			sortir=true;
		}
	}
	enviarComanda("EXIT");
	// Si s'ha produit algun error ACL, el borram. S'ha decidit fer
	// aix perqu l'error ACL podria provocar una actuaci
	// dolenta pel robot (colisi, ...)
	if (sortir) { 
		sprintf(tmp, "REMOVE %s", nomPrograma.Left(5));
		enviarComanda(tmp);
		enviarComanda("YES");
	}

	m_directe=directe;

	arxiu.Close();
	fitxer.Close();
}

void CDspRobot::executarPrograma(CString nom)
{
	char tmp[200];
	posarDirecte();
	sprintf(tmp, "RUN %s", nom);
	enviarComanda(tmp);
}

void CDspRobot::executarPrograma(CString nom, int temps)
{
	executarPrograma(nom);
	Sleep(temps*1000);
}

void CDspRobot::executarPrograma(CString nom, int temps, CString respostaEsperada)
{
	char comanda[100];
	sprintf(comanda, "RUN %s%s", nom, m_sl);
	m_port.WriteToPort(comanda, true, respostaEsperada, temps);
}

void CDspRobot::activarCinta()
{
	char comanda[100];
	sprintf(comanda, "SET OUT[1]=1%s", m_sl);
	executarComanda(comanda);

}

void CDspRobot::desactivarCinta()
{
	char comanda[100];
	sprintf(comanda, "SET OUT[1]=0%s", m_sl);
	executarComanda(comanda);
}
