/***************** IV4.H ********************/
/* this file contains macro definitions and */
/* function prototypes used in IV-400 SDK   */
/********************************************/

#ifndef _IV4_H_
#define _IV4_H_

#include "portable.h"

#define IV4_SDK_VERSION             0x0152

// flags used by IV1GetSAA7110Control() 
// and IV4SetSAA7110Control()
#define IV4_CS_Channel              50
#define IV4_Channel_Red             0
#define IV4_Channel_Green           1
#define IV4_Channel_Blue            2
#define IV4_Channel_SVideo          3
#define IV4_Channel_Current         4
#define IV4_Channel_SVideo2         5
#define IV4_Force_Channel_Switch    0x80

#define IV4_CS_Saturation           51
#define IV4_CS_Hue                  52

#define IV4_CS_AGC                  53
#define IV4_AGC_On                  0
#define IV4_AGC_Off                 1

#define IV4_CS_Gain                 54
#define IV4_Gain_N3                 0x00
#define IV4_Gain_0                  0x0F
#define IV4_Gain_3                  0x1F
#define IV4_Gain_6                  0x2F
#define IV4_Gain_9                  0x3F

#define IV4_CS_Flag_PAL             55
#define IV4_TVStandard_NTSC         0
#define IV4_TVStandard_PAL          1

#define IV4_CS_VCR_Mode             56
#define IV4_VCR_Mode_Off            0
#define IV4_VCR_Mode_On             1

#define IV4_CS_BW_Mode              57
#define IV4_BW_Mode_Off             2
#define IV4_BW_Mode_On              1

#define IV4_CS_Contrast             58
#define IV4_CS_Brightness           59

// for IV-450 Pro only
#define IV4_CS_Top                  60
#define IV4_CS_Bottom               61

// new error codes   
#define IV4_OK                      OK
#define IV4_ERROR                   GEN_ERROR
#define IV4_INVALID_VGA_MODE        -31
#define IV4_PCIBUS_NOT_FOUND        -32      
#define IV4_BOARD_NOT_FOUND         -33
#define IV4_PARAM_ERR               -34
#define IV4_TIMEOUT_ERR             -35
#define IV4_INSUFFICIENT_MEMORY     -36
#define IV4_NOT_SUPPORTED           -37

// flags used by IV4StartGrab()
#define IV4_Continue_Grab           0x0001
#define IV4_Single_Grab             0x0002
#define IV4_Trigger_Grab            0x0080
#define IV4_Skip_CameraDetection    0x0004

// flags used by IV4SetGrabWindow() & IV4GetGrabWindow()
#define IV4_Grab_Odd                0x0004
#define IV4_Grab_Even               0x0008
#define IV4_Grab_Next               0x0010
#define IV4_Field_Mode              0x0020
#define IV4_Frame_Mode              0x0040
#define IV4_NonInterlace_Mode       0x0100

// flags used by IV4SetPixelMode() & IV4GetPixelMode()
// flags used by IV4RdPixelRect() & IV4WtPixelRect()
#define IV4_8bit_Mode               0x01
#define IV4_16bit_Mode              0x02

// flags used by IV4RdPixelRect() & IV4WtPixelRect()
#define IV4_VInverse_Mode           0x04
#define IV4_HInverse_Mode           0x08

// flags used by IV4SetOverlayMode() & IV4GetOverlayMode()
#define IV4_Opaque_Overlay_Mode     0x00000000L
#define IV4_No_Overlay_Mode         0x01000000L
#define IV4_Color_Key_Mode          0x05000000L
#define IV4_Chroma_Key_Mode         0x06000000L   

// flags used by IV4SetZoomFactors() & IV4GetZoomFactors()
#define IV4_Zoom_1x                 1
#define IV4_Zoom_2x                 2
#define IV4_Zoom_4x                 4

// flags used by IV4SetScaleFactors() & IV4GetScaleFactors()
#define IV4_HScale_1                0x00000000L
#define IV4_HScale_2                0x33333333L     
#define IV4_HScale_4                0x3F3F3F3FL     
#define IV4_HScale_8                0x3FFF3FFFL
#define IV4_HScale_16               0x3FFFFFFFL
#define IV4_VScale_1                0x00000000L
#define IV4_VScale_2                0x55555555L
#define IV4_VScale_4                0x77777777L
#define IV4_VScale_8                0x7F7F7F7FL
#define IV4_VScale_16               0x7FFF7FFFL

// flags used by IV4SetVideoFrame() & IV4GetVideoFrame()
#define IV4_Single_Image_Mode       1
#define IV4_Multi_Image_Mode        2 

// flags used by IV4SetTVStandard() & IV4GetTVStandard()
#define IV4_Camera_NTSC             0
#define IV4_Camera_PAL              1 

// used by IV4BMPLoad(), IV4TGALoad(), IV4TIFFLoad()
#define IV4_Load_As_Is            	1
#define IV4_Load_As_BW            	2    

// used by IV4BMPSave(), IV4TGASave(), IV4TIFFSave()
#define IV4_Save_As_Is            	4
#define IV4_Save_As_BW            	5    
#define IV4_Save_As_RGB            	6
#define IV4_Save_As_YUV            	7

// used by IV4TGASave()
#define IV4_Save_As_RGB16          	8
#define IV4_Save_As_RGB24          	9
#define IV4_Save_As_RGB32          	10

// default Video Info. for NTSC and PAL
#define IV4_NTSC_Width_Default      0x0280                    
#define IV4_PAL_Width_Default       0x0300
#define IV4_NTSC_Height_Default     0x01E0                                        
#define IV4_PAL_Height_Default      0x0240          

// this is the size of the config buffer returned by IV4InquireConfig()
// this number will not be changed for ever
#define IV4_InitConfig_Size         0x0200   

#if defined(_MSDOS_)
typedef struct RECT {
	int top;
	int left;
	int right;
	int bottom;
} RECT;
#endif
// used by IV4InquireBoardInfo()
typedef struct _IV4BoardInfo      
{                                                                                
    ULONG SubSystemID;              // contains board revision information
                                    // not available for ViRGE
    UWORD ChipDeviceID;             // 0x5631 for ViRGE, 0x8A01 for ViRGE DX/GX
    BYTE  ChipRevisionID;           // varies with stepping
    UWORD SDKVersion;               // 0xMMmm: MM major version, mm minor version
    BYTE  ExternalTriggerable;      // 1 yes, 0 no
    BYTE  BoardMemorySize;          // 2, 4 MB
    UWORD CurVGAMode;               // current VGA mode
    UWORD ScreenWidth;              // depends on current VGA mode
    UWORD ScreenHeight;             // depends on current VGA mode
    UWORD ScreenDepth;              // depends on current VGA mode
    BYTE  IRQNumber;                // IRQ number
    float Mclock;                   // MCLOCK frequency
    // following variables have been added since SDK Ver. 1.42
    BYTE  SVideoMode;               // 0 - single SVideo, 1 - double SVideo 
    RECT  WorkingArea;              // used for multi monitor support
    // following variables have been added since SDK Ver. 1.51
    UWORD KernelVersion;            // 0xMMmm: MM major version, mm minor version
} IV4BoardInfo;

#define IV4_Single_SVideo           0
#define IV4_Double_SVideo           1


#if defined(_CPLUSPLUS_)
extern "C" {
#endif

// General Board Control Functions
int MTPROCALL IV4Open(UWORD SDKVersion);
UWORD MTPROCALL IV4InquireSDKVersion(void);
int MTPROCALL IV4InquireBoardInfo(IV4BoardInfo FAR *info);
int MTPROCALL IV4Init(BYTE FAR *initFlags);
int MTPROCALL IV4InquireConfig(BYTE FAR *config);
int MTPROCALL IV4Close(void);

// Grabbing Control Functions
int MTPROCALL IV4StopGrab(void);
int MTPROCALL IV4StartGrab(UWORD flag);
int MTPROCALL IV4WaitForFrameEnd(void);
int MTPROCALL IV4WaitForVSync(void);
int MTPROCALL IV4WaitForVGAVRetrace(void);

// Overlay Control Functions
int MTPROCALL IV4SetOverlayMode(ULONG mode);
int MTPROCALL IV4GetOverlayMode(ULONG FAR *mode);
int MTPROCALL IV4SetRGBColorKey(BYTE r, BYTE g, BYTE b);
int MTPROCALL IV4GetRGBColorKey(BYTE FAR *r, BYTE FAR *g, BYTE FAR *b);
int MTPROCALL IV4SetYUVChromaKey(BYTE yl, BYTE ul, BYTE vl, 
                                 BYTE yu, BYTE uu, BYTE vu);
int MTPROCALL IV4GetYUVChromaKey(BYTE FAR *yl, BYTE FAR *ul, BYTE FAR *vl, 
                                 BYTE FAR *yu, BYTE FAR *uu, BYTE FAR *vu);

// Grab Window, Video Frame, Image Window Control Functions
int MTPROCALL IV4SetGrabWindow(UWORD mode, UWORD sx, UWORD sy, UWORD dx, UWORD dy);
int MTPROCALL IV4GetGrabWindow(UWORD FAR *mode, UWORD FAR *sx, UWORD FAR *sy, UWORD FAR *dx, UWORD FAR *dy);
int MTPROCALL IV4SetScaleFactors(ULONG Hfactor, ULONG Vfactor);
int MTPROCALL IV4GetScaleFactors(ULONG FAR *Hfactor, ULONG FAR *Vfactor);
int MTPROCALL IV4SetVideoFrame(UWORD mode, ULONG startOffset, UWORD dx, UWORD dy);
int MTPROCALL IV4GetVideoFrame(UWORD FAR *mode, ULONG FAR *startOffset, UWORD FAR *dx, UWORD FAR *dy);
int MTPROCALL IV4SetImageFrameOnVideoFrame(UWORD sx, UWORD sy, UWORD dx, UWORD dy);
int MTPROCALL IV4GetImageFrameOnVideoFrame(UWORD FAR *sx, UWORD FAR *sy, UWORD FAR *dx, UWORD FAR *dy);
int MTPROCALL IV4SetZoomFactors(UWORD HFactor, UWORD VFactor);
int MTPROCALL IV4GetZoomFactors(UWORD FAR *HFactor, UWORD FAR *VFactor);
int MTPROCALL IV4SetImageWndOnScreen(short sx, short sy);
int MTPROCALL IV4GetImageWndOnScreen(short FAR *sx, short FAR *sy, UWORD FAR *dx, UWORD FAR *dy);

// Video Adjustment Control Functions
int MTPROCALL IV4SetTVStandard(BYTE camera);
int MTPROCALL IV4GetTVStandard(BYTE FAR *camera);
int MTPROCALL IV4SetChannel(BYTE channel);
int MTPROCALL IV4GetChannel(BYTE FAR *channel);
int MTPROCALL IV4SetPixelMode(BYTE mode);
int MTPROCALL IV4GetPixelMode(BYTE FAR *mode);
int MTPROCALL IV4SetHue(char hue);
int MTPROCALL IV4GetHue(char FAR *hue);
int MTPROCALL IV4SetSaturation(BYTE sat);
int MTPROCALL IV4GetSaturation(BYTE FAR *sat);
int MTPROCALL IV4SetVCRMode(BYTE mode);
int MTPROCALL IV4GetVCRMode(BYTE FAR *mode);
int MTPROCALL IV4SetAGC(BYTE AGC);
int MTPROCALL IV4GetAGC(BYTE FAR *AGC);
int MTPROCALL IV4SetGain(BYTE channel, BYTE gain);
int MTPROCALL IV4GetGain(BYTE channel, BYTE FAR *gain);
int MTPROCALL IV4SetContrast(BYTE contrast);
int MTPROCALL IV4GetContrast(BYTE FAR *contrast);
int MTPROCALL IV4SetBrightness(BYTE brightness);
int MTPROCALL IV4GetBrightness(BYTE FAR *brightness);

// VRAM Access Control Functions
int MTPROCALL IV4RdPixelLine(UWORD sx, UWORD sy, UWORD dx, void FAR *buf);
int MTPROCALL IV4WtPixelLine(UWORD sx, UWORD sy, UWORD dx, void FAR *buf);
int MTPROCALL IV4SetArea2Value(UWORD sx, UWORD sy, UWORD dx, UWORD dy, 
                                BYTE Yval, BYTE Uval, BYTE Vval);
int MTPROCALL IV4RdPixelRect(UWORD sx, UWORD sy, UWORD dx, UWORD dy,
							 void FAR *buf, UWORD PixelMode);
int MTPROCALL IV4WtPixelRect(UWORD sx, UWORD sy, UWORD dx, UWORD dy,
							 void FAR *buf, UWORD PixelMode);

// Look Up Table Control Functions
int MTPROCALL IV4LUTEnable(void);
int MTPROCALL IV4LUTDisable(void);
int MTPROCALL IV4LUTLoad(BYTE addr, UWORD n, BYTE FAR *value);
int MTPROCALL IV4SelectLUTBank(BYTE CurBank);

// File Operation Functions
int MTPROCALL IV4InquireBMPVals(char FAR *name, UWORD FAR *px, UWORD FAR *py, UWORD FAR *bp);
int MTPROCALL IV4InquireTGAVals(char FAR *name, UWORD FAR *px, UWORD FAR *py, UWORD FAR *bp);
int MTPROCALL IV4InquireTIFFVals(char FAR *name, UWORD FAR *px, UWORD FAR *py, UWORD FAR *bp);
int MTPROCALL IV4BMPLoad(char FAR *name,  UWORD sx,UWORD sy, UWORD dsx, UWORD dsy, 
                        UWORD dx, UWORD dy, UWORD flag);
int MTPROCALL IV4BMPSave(char FAR *name,  UWORD sx, UWORD sy, 
                        UWORD dx, UWORD dy, UWORD flag);
int MTPROCALL IV4TGALoad(char FAR *name,  UWORD sx,UWORD sy, UWORD dsx, UWORD dsy, 
                        UWORD dx, UWORD dy, UWORD flag);
int MTPROCALL IV4TGASave(char FAR *name,  UWORD sx, UWORD sy, 
                        UWORD dx, UWORD dy, UWORD flag);
int MTPROCALL IV4TIFFLoad(char FAR *name,  UWORD sx,UWORD sy, UWORD dsx, UWORD dsy, 
                        UWORD dx, UWORD dy, UWORD flag);
int MTPROCALL IV4TIFFSave(char FAR *name,  UWORD sx, UWORD sy, 
                        UWORD dx, UWORD dy, UWORD flag);

// color conversion Functions 
void MTPROCALL IV4Rgb2Yuv(void FAR *input, void FAR *output,
						  ULONG length, UWORD PixelSize, UWORD ByteOrder);
void MTPROCALL IV4Yuv2Rgb(void  FAR *input, void FAR *output,
						  ULONG length, UWORD PixelSize, UWORD ByteOrder);


void MTPROCALL IV4Rgb2YuvCCIR601_422(ULONG FAR *input, ULONG FAR *output,
						             ULONG length, UWORD PixelSize, UWORD flag);
void MTPROCALL IV4Yuv2Rgb24bitDIBCCIR601_422(ULONG FAR *input, ULONG FAR *output,
									         UWORD width, UWORD height);
void MTPROCALL IV4Yuv2Rgb24bitCCIR601_422(ULONG FAR *input, ULONG FAR *output, ULONG length, UWORD flag);
void MTPROCALL IV4Rgb2YCCIR601(ULONG FAR *input, ULONG FAR *output, ULONG length, BYTE reverse);
void MTPROCALL IV4Yuv2Rgb32bitCCIR601_422(ULONG FAR *input, ULONG FAR *output, ULONG length, UWORD flag);
void MTPROCALL IV4Yuv2Rgb32bitCCIR601_YYUV(ULONG FAR *input, ULONG FAR *output, ULONG length);

#if defined(_MSDOS_)
typedef void (FAR * PIV4CALLBACK)(UWORD);          
#define IV4CALLBACK void
#elif defined (WIN32)
typedef void (MTPROCALL * PIV4CALLBACK)(UWORD);          
#define IV4CALLBACK void MTPROCALL
#elif defined(_WINDOWS) || defined(_OS2)
typedef FARPROC PIV4CALLBACK;
#define IV4CALLBACK void _export CALLBACK
#endif

// event, trigger, strobe functions

// used by IV4SetStrobe(), IV4EnableEvent(), IV4EnableTrigger()
#define    	IV4_ACTIVE_HIGH    	0x0002
#define    	IV4_ACTIVE_LOW    	0x0001

// used by IV4EnableTrigger()
#define    	IV4_TRG_GRAB_FRAME	0x0004
#define    	IV4_TRG_GRAB_FIELD	0x0008
#define    	IV4_TRG_GRAB_NINT	0x0100
#define    	IV4_TRG_GRAB_SINGLE	0x0200
#define    	IV4_TRG_GRAB_CONT	0x0010
#define    	IV4_TRG_GRAB_STOP	0x0020                                                    

// used by IV4SetStrobe()
#define         IV4_HW_WIDTH            0x0040
#define         IV4_SW_WIDTH            0x0080
#define         IV4_HS_TIMING           0x0800
#define    	IV4_VS_TIMING    	0x1000
#define    	IV4_GRAB_SINGLE		0x2000
#define    	IV4_START_ODD		0x4000
#define    	IV4_START_EVEN    	0x8000
#define    	IV4_START_NEXT    	0x0400

// parameters of Trigger, GrabDone, Event callback functions
#define	IV4_Notify_Event        0x01
#define	IV4_Notify_Trigger      0x02
#define	IV4_Notify_GrabDone     0x04
#define	IV4_Notify_StrobeDone   0x08

// flags used by IV4InquireField(), and callback function of IV4SingleCapture()
#define IV4_Odd_Field           0x10
#define IV4_Even_Field          0x20

// functions found in IV4HLGRP.C
int MTPROCALL IV4EnableStrobe(UWORD flag, ULONG delay, ULONG width, 
                              PIV4CALLBACK StrobeDoneCallback);
int MTPROCALL IV4SetStrobe();
int MTPROCALL IV4ResetStrobe();
int MTPROCALL IV4DisableStrobe();
int MTPROCALL IV4EnableEvent(UWORD flag, PIV4CALLBACK EventNotifyCallback);
int MTPROCALL IV4DisableEvent();
int MTPROCALL IV4ResetEvent();
int MTPROCALL IV4GenerateEvent();
int MTPROCALL IV4EnableTrigger(UWORD flag, PIV4CALLBACK TriggerNotifyCallback);
int MTPROCALL IV4DisableTrigger();
int MTPROCALL IV4ResetTrigger();
int MTPROCALL IV4GenerateTrigger();                                   
int MTPROCALL IV4InquireField(void);
int MTPROCALL IV4SingleCapture(PIV4CALLBACK CaptureDoneCallback);
int MTPROCALL IV4IsCaptureDone(void);

#if defined(_MSDOS_)
// flag used by IV4EnableHWCursor()
#define IV4_DEFAULT_CURSOR   0x01
#define IV4_TIMEGLASS_CURSOR 0x02

int MTPROCALL IV4EnableHWCursor(void);
int MTPROCALL IV4DisableHWCursor(void);
int MTPROCALL IV4LoadHWCursor(UWORD CursorType, ULONG addr);
int MTPROCALL IV4LoadHWCursorEx(BYTE FAR *CAndBitmap, BYTE FAR *CXorBitmap, ULONG addr);
int MTPROCALL IV4ShowHWCursor(void);
int MTPROCALL IV4HideHWCursor(void);
int MTPROCALL IV4PositionHWCursor(int x, int y);
int MTPROCALL IV4InitMouseWithHWCursor(void);
int MTPROCALL IV4ClearMouseCallback(void);
int MTPROCALL IV4PositionMouse(int x, int y);

#endif

int MTPROCALL IV4SetZoomFactorsEx(UWORD w0, UWORD w1, UWORD h0, UWORD h1);
int MTPROCALL IV4GetZoomFactorsEx(UWORD FAR *w0, UWORD FAR *w1, UWORD FAR *h0, UWORD FAR *h1);

int MTPROCALL IV4SetIndexKey(BYTE key);
int MTPROCALL IV4GetIndexKey(BYTE FAR *key);

#define IV4_ANALOG_INPUT            0x0001
#define IV4_DIGITAL_INPUT           0x0002

#define IV4_COLOR_VIDEO             0x0004
#define IV4_BW_VIDEO                0x0008

#define IV4_SELECT_SAA7110          0x0010
#define IV4_SELECT_BT252            0x0020

#define IV4_COMPOSITE_VIDEO         0x0040
#define IV4_VIDEO_COMPOSITE_SYNC    0x0080
#define IV4_VIDEO_HSYNC_VSYNC       0x0100

#define IV4_CLAMP_ON_HS             0x0200
#define IV4_CLAMP_ON_BP             0x0400
#define IV4_BT_CLAMP_FOR_SAA7110    0x0800

#define IV4_USE_PLL_CLOCK           0x1000
#define IV4_USE_EXTERNAL_CLOCK      0x2000

// default SAA7110 configuration
#define IV4_SAA7110_CONFIG_0        IV4_ANALOG_INPUT | IV4_SELECT_SAA7110

// default SAA7110 configuration, but use BT252 as clampping
#define IV4_SAA7110_CONFIG_1        IV4_SAA7110_CONFIG_0 | IV4_BT_CLAMP_FOR_SAA7110

// default BT252 configuration
#define IV4_BT252_CONFIG_0          IV4_ANALOG_INPUT | IV4_SELECT_BT252

// default BT252 configuration
#define IV4_BT252_CONFIG_1          IV4_BT252_CONFIG_0 | IV4_VIDEO_HSYNC_VSYNC | IV4_USE_EXTERNAL_CLOCK

int MTPROCALL IV4SetBoardConfig(ULONG flags);
int MTPROCALL IV4GetBoardConfig(ULONG FAR *flags);

int MTPROCALL IV4SetTopBottom(BYTE channel , BYTE top, BYTE bottom);
int MTPROCALL IV4GetTopBottom(BYTE channel , BYTE FAR *top, BYTE FAR *bottom);

int MTPROCALL IV4ResetHWStrobe(void);

#define IV4_HDecimation_StopGrab        0xFFFFFFFFL
#define IV4_HDecimation_StartGrab       0x12345678L

void MTPROCALL IV4SetHDecimation(ULONG dValue);

int MTPROCALL IV4DetectCamera(BYTE channel);
int MTPROCALL IV4EnableCameraNotify(PIV4CALLBACK CameraCallback);
int MTPROCALL IV4DisableCameraNotify(void);

#define	IV4_Camera_Disconnected         0x01
#define	IV4_Camera_Reconnected          0x02


// used for CompressMethod in IV4JPEGSave()
#define IV4_JPEG_LOSSLESS		0		// JPEG lossless compression
#define IV4_JPEG_LOSSY			1		// JPEG baseline compression

int MTPROCALL IV4JPEGSave(char FAR *name,  UWORD sx, UWORD sy, 
                          UWORD dx, UWORD dy, UWORD flag,
                          BYTE CompressMethod, BYTE QualityFactor,
                          BYTE FAR *Comments, UWORD cLength);

int MTPROCALL IV4JPEGLoad(char FAR *name,  UWORD sx,UWORD sy, UWORD dsx, UWORD dsy, 
                          UWORD dx, UWORD dy, UWORD flag,
                          BYTE FAR *Comments, UWORD FAR *cLength);
int MTPROCALL IV4InquireJPEGVals(char FAR *name, UWORD FAR *px, UWORD FAR *py, UWORD FAR *bp,
                          BYTE FAR *Comments, UWORD FAR *cLength);

#if defined(__WATCOMC__)
void MTPROCALL IV4_DOS4GW_Begin_Graphic(void);
void MTPROCALL IV4_DOS4GW_End_Graphic(void);
#endif

#if defined(_CPLUSPLUS_)
}
#endif

#endif
