/////////////////////////////////////////////////////////////////
//
// Arxiu      : PriAnalisiMoviment.cpp
//
// Descripci : Arxiu d'implementaci de la classe que realitza
//				part de la funcionalitat del processament
//				digital d'imatge.
//				Aquesta classe s'encarrega de la detecci de
//				moviment i de determinar quan un objecte est
//				totalment dins del camp de visi de la cmara.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "PriAnalisiMoviment.h"
#include "sistema.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//
// Constructor estndard
//
CPriAnalisiMoviment::CPriAnalisiMoviment(CWnd* pParent /*=NULL*/)
	: CDialog(CPriAnalisiMoviment::IDD, pParent)
{
	//{{AFX_DATA_INIT(CPriAnalisiMoviment)
	m_areaVisible = 0;
	m_estat = 0;
	m_comentari = _T("");
	//}}AFX_DATA_INIT
}

void CPriAnalisiMoviment::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CPriAnalisiMoviment)
	DDX_Text(pDX, IDC_EDIT_AREA_VISIBLE, m_areaVisible);
	DDX_Text(pDX, IDC_EDIT_ESTAT, m_estat);
	DDX_Text(pDX, IDC_EDIT_COMENTARI, m_comentari);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CPriAnalisiMoviment, CDialog)
	//{{AFX_MSG_MAP(CPriAnalisiMoviment)
	ON_WM_TIMER()
	ON_BN_CLICKED(IDC_BOTO_ACTIVAR, OnBotoActivar)
	ON_BN_CLICKED(IDC_BOTO_DESACTIVAR, OnBotoDesactivar)
	ON_BN_CLICKED(IDC_BOTO_CINTAON, OnBotoCintaon)
	ON_BN_CLICKED(IDC_BOTO_CINTAOF, OnBotoCintaof)
	ON_BN_CLICKED(IDC_BOTO_INICI, OnBotoInici)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

//
// Inicialitzaci de l'objecte. 
// S'accedeix a tots els parmetres de configuraci i als
// dispositius externs a travs de 'sistema'.
//
void CPriAnalisiMoviment::inicialitzar(void *sistema)
{
	Create(IDD_ANALISI_MOVIMENT);
	// Obtenci dels distints elements amb que ha
	// de tractar l'anlisi del moviment a partir
	// del punter al sistema. 
	// Es fa el casting (CSistema *) ja que
	// sistema s'ha passat com a (void *).
	m_robot=&(((CSistema *)sistema)->m_robot);
	m_camara=&(((CSistema *)sistema)->m_camara);
	m_cella=&(((CSistema *)sistema)->m_cellconfig);
	m_config=&(((CSistema *)sistema)->m_imgconfig) ;
	// Obtenci dels parmetres de control de 
	// l'autmat finit.
	m_nMaxInici=m_cella->m_nMaxInici;
	m_nMaxDet=m_cella->m_nMaxDeteccio;
	m_tolerancia=m_cella->m_tolerancia;
	// Clcul de les dimensions de l'rea a observar
	// per a la detecci de moviment.
	int ampladaAnalitzada = m_cella->m_amplada;
	int xf = m_config->m_xFinestra;
	int yf = m_config->m_yFinestra;
	int wf = m_config->m_ampleFinestra;
	int hf = m_config->m_altFinestra;
	int xf2 = xf+wf-ampladaAnalitzada;
	int yf2 = yf ;
	int wf2 = ampladaAnalitzada;
	int hf2 = hf ;
	// Inicialitzaci de l'estat de l'autmat.
	m_estat=0;
	m_areaVisible=0;
	m_n=0;
	m_imgn=0;
	m_img1=0;
	m_img2=0;
	m_imatge.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, xf2, yf2, wf2, hf2);
	m_alt=hf2;
	m_ample=wf2;
	m_xinicial=xf2;
	m_yinicial=yf2;
	m_matriu=m_imatge.obtenirMatriu();
	m_comentari=_T("");
	m_sistema=sistema;
	// Inicialitzaci de l'execuci continua.
	// L'execuci contnua permet conservar 
	// els valors de les variables de l'script que
	// s'executa entre execucions, fins que no
	// es torni a cridar la funci iniciarExecucioContinua().
	((CSistema *)m_sistema)->m_interpret.iniciarExecucioContinua();
	SetTimer(ANM_TIMER_ANALISI, m_cella->m_tempsRefresc, NULL);
}

//
// Funci cridada pel timer. S'encarrega de:
// * capturar el fotograma actual
// * fer el threshold
// * calcular el valor a partir del qual treballa l'autmat. 
// * executar l'autmat
//
void CPriAnalisiMoviment::OnTimer(UINT nIDEvent) 
{
	if (nIDEvent==ANM_TIMER_ANALISI) {
		// Es captura el fotograma
		m_camara->capturarFotograma();
		m_camara->posarEnMemoria(&m_imatge);
		// Es realitza el threshold
		//m_imatge.threshold(9999, m_config->m_emfatitzarHistograma, m_config->m_ampladaSuavitzat, m_config->m_minAmuntAvall, m_config->m_minAmuntAvall);
		m_imatge.threshold(9999, true, m_config->m_ampladaSuavitzatAM, m_config->m_minAmuntAvallAM, m_config->m_minAmuntAvallAM);
		// Es compten els pixels...
		comptarPixels();
		// S'actualitza m_img1 (valor observat per al
		// fotograma actual)
		m_img1=m_areaVisible;
		// S'avana cap a un nou estat depenent de l'estat
		// actual i dels parmetres.
		nouEstat();
		UpdateData(false);
	}
	else
		CDialog::OnTimer(nIDEvent);
}

//
// S'encarrega de comptar el nmero de pixels que no
// formen part del fons de la imatge. Es suposa que
// el fons de la imatge s del color 0. El valor aix
// obtingut s el que es t en compte a l'hora de comparar
// dues imatges (i determinar si est entrant un objecte
// en el camp de visi)
//
void CPriAnalisiMoviment::comptarPixels()
{
	int i, j;
	m_areaVisible=0;
	for (i=m_yinicial;i<m_yinicial+m_alt;i++)
		for (j=m_xinicial;j<m_xinicial+m_ample;j++) {
			if (m_matriu[i][j]!=0) {
				m_areaVisible++;
			}
		}
}
// 
// Actualitza l'estat de l'autmat i actua en conseqncia.
// L'autmat respon al segent esquema:
//
// Inicialitzaci: n=0 ; Estat inicial:0
// Estat 0 : *	si n<nMaxInici i el fotograma actual s
// (inicial.)	semblant a l'antarior llavors n++ i 
//				Estat=0;
//			 *	si n<nMaxInici i el fotograma actual no
//				s semblant a l'anterior, llavors fer n=0 i
//				Estat=0;
//			 *	si n>=nMaxInici llavors imatgeNormal=
//				imatgeActual i Estat=1;
// Estat 1 : *	si imatgeActual s semblant a imatgeNormal
// (sense		llavors Estat=1;
// objectes) *	si imatgeActual no s semblant a imatgeNormal
//				llavors fer n=0 i Estat=2;
// Estat 2 : *	si imatgeActual no s semblant a imatgeNormal
// (possible	i n<nMaxDet llavors n++ i Estat=2;
// objecte)  *	si imatgeActual s semblant a imatgeNormal i
//				n<nMaxDet, llavors Estat=1
//				si imatgeActual s semblant a imatgeNormal i
//				n>nMaxDet llavors **actuar** i Estat=1;

void CPriAnalisiMoviment::nouEstat()
{
	switch (m_estat) {
	case 0:
		// Estat d'inicialitzaci. El sistema est en 
		// aquest estat fins que no ha trobat
		// m_nMaxInici fotogrames seguits amb un
		// recompte semblant de pixels que no sn
		// del fons.
		m_comentari="Inicialitzant";
		if (!semblant(m_img1, m_img2, m_tolerancia)) 
			m_n=0;
		else if (m_n<=m_nMaxInici)
			m_n++;
		if (m_n>m_nMaxInici) {
			m_imgn=m_img1;
			m_comentari="Inicialitzat";
			m_estat=1;
			KillTimer(ANM_TIMER_ANALISI);
		}
		break;
	case 1:
		// S'est en estat d'observaci mentres el
		// recompte de pixels sigui semblant 
		// al recompte obtingut en l'etapa d'inicialitzaci.
		m_comentari="Observant";
		if (!semblant(m_img1, m_imgn, m_tolerancia)) {
			m_n=0;
			m_estat=2;
		}
		break;
	case 2:
		// Si, estant en l'estat 2, el recompte de pixels
		// torna a ser semblant al de la inicialitzaci
		// es torna a l'estat 1, a no ser que s'hagi
		// estat durant m_nMaxDetecci fotogrames en
		// l'estat 2. En aquest cas, vol dir que hi
		// ha un objecte a l'abast de la cmara.
		if (semblant(m_img1, m_imgn, m_tolerancia) && m_n>m_nMaxDet) {
			KillTimer(ANM_TIMER_ANALISI);
			actuar();
			SetTimer(ANM_TIMER_ANALISI, m_cella->m_tempsRefresc, NULL);	
			m_estat=1;
		} 
		else if (semblant(m_img1, m_imgn, m_tolerancia))
				m_estat=1;
		else if (!semblant(m_img1, m_imgn, m_tolerancia) && m_n<=m_nMaxDet)
			m_n++;
		break;
	}
	m_img2=m_img1;
}

//
// Determina si dos nmeros sencers sn semblants segons un
// cert marge de tolerncia. 
// Aquesta funci s'empra per a determinar si dues imatges, 
// a partir del valor que es calcula a partir d'elles, 
// sn semblants.
//
bool CPriAnalisiMoviment::semblant(long int im1, long int im2, long int tolerancia)
{
	if (im1>im2-tolerancia && im1<im2+tolerancia)
		return true;
	else
		return false;
}

//
// Activa el timer. Suposa que el sistema est inicialitzat.
//
void CPriAnalisiMoviment::OnBotoActivar() 
{
	inicialitzarConfiguracio();
	SetTimer(ANM_TIMER_ANALISI, m_cella->m_tempsRefresc, NULL);	
}

//
// Desactiva el timer
//
void CPriAnalisiMoviment::OnBotoDesactivar() 
{
	KillTimer(ANM_TIMER_ANALISI);	
}

//
// Alterna la visualitzaci de la finestra d'anlisi
//
void CPriAnalisiMoviment::alternar()
{
	if (!estaActiu()) {
		ShowWindow(SW_SHOW);
	}
	else {
		KillTimer(ANM_TIMER_ANALISI);
		ShowWindow(SW_HIDE);
	}

}

//
// Determina si la finestra est activa.
//
BOOL CPriAnalisiMoviment::estaActiu()
{
	return IsWindowVisible();
}

//
// Recullen l'event de OK i Cancel per a evitar que es tanqui
// la finestra en pitjar Return o Escape.
//
void CPriAnalisiMoviment::OnOK()
{
	return;
}

void CPriAnalisiMoviment::OnCancel()
{
	ShowWindow(SW_HIDE);
}

//
// Respon a l'event de pitjar el bot d'activaci de la cinta
// i l'activa.
//
void CPriAnalisiMoviment::OnBotoCintaon() 
{
	m_robot->activarCinta();	
}

//
// Respon a l'event de pitjar el bot de desactivaci de la cinta
// i la desactiva.
//
void CPriAnalisiMoviment::OnBotoCintaof() 
{
	m_robot->desactivarCinta();
}

//
// Executa l'script configurat. Aquesta funci es crida quan
// s'ha determinat que l'objecte est totalment dins del camp
// de visi i s'ha d'actuar en conseqncia.
//
void CPriAnalisiMoviment::executarScript()
{
	int i, xc, yc, per, xmn, ymn, dmn, xmx, ymx, dmx;
	double comp, orien, tinici, tfi, tduracio;
	CPoint p;
	long area;
	CAuxObjecte *objecte;

	tinici = clock();

	if (((CSistema *)m_sistema)->m_cellconfig.m_nomArxiuScript==_T("")) {
		static char BASED_CODE szFilter[] = "Scripts d'actuaci de BRAS (*.bsc)|*.bsc|Tots els Arxius (*.*)|*.*||";
		CFileDialog fdlg(true, NULL, NULL, OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, szFilter);
		if (fdlg.DoModal()==IDOK) 
			((CSistema *)m_sistema)->m_cellconfig.m_nomArxiuScript=fdlg.GetPathName();
		else return;
	}

	// Es crea una imatge amb les mides adequades.
	CPriImatge laImatge, laImatgeTemporal;
	CPriImgMarques marques;
	CPriRegio regio;
	laImatge.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_config->m_xFinestra, m_config->m_yFinestra, m_config->m_ampleFinestra, m_config->m_altFinestra);
	laImatgeTemporal.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_config->m_xFinestra, m_config->m_yFinestra, m_config->m_ampleFinestra, m_config->m_altFinestra);
	marques.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_config->m_xFinestra, m_config->m_yFinestra, m_config->m_ampleFinestra, m_config->m_altFinestra);
	regio.inicialitzar(IMG_AMPLEI_DEFECTE, IMG_ALTI_DEFECTE, m_config->m_xFinestra, m_config->m_yFinestra, m_config->m_ampleFinestra, m_config->m_altFinestra);
	// Es captura el fotograma
	m_camara->capturarFotograma();
	m_camara->posarEnMemoria(&laImatge);
	// Posa la imatge adequade en l'espai temporal segons el que
	// s'hagi determinat en les preferncies.
	switch (m_config->m_estilSiluetes) {
	case 0 : break;
	case 1 : break;
	case 2 : laImatgeTemporal=laImatge; break;
	}
	// Realitza el procs de la imatge
	((CSistema *)m_sistema)->realitzarProcesImatge(&laImatge);
	// Crea la imatge de marques
	laImatge.crearMarques(&marques);

	((CSistema *)m_sistema)->m_interpret.m_objectes.RemoveAll();
	// Itera i per cada regi n'obt les propietats.
	for (i=1;i<marques.obtenirNumMarques();i++) {
		marques.crearRegio(i, &regio);
		area = regio.obtenirArea();
		regio.obtenirCentreGravetat(&xc, &yc);
		// Dibuixa el contorn sols si el volem veure.
		if (m_config->m_estilSiluetes==0)
			per = regio.obtenirPerimetre(NULL);
		else 
			per = regio.obtenirPerimetre(&laImatgeTemporal);

		comp = regio.obtenirCompacitat();
		regio.obtenirDMin(&xmn, &ymn, &dmn);
		regio.obtenirDMax(&xmx, &ymx, &dmx);

		if (m_config->m_estilSiluetes!=0) {
			laImatgeTemporal.dibuixarPunt(xc, yc, 2);
			laImatgeTemporal.dibuixarPunt(xmn, ymn, 2);
			laImatgeTemporal.dibuixarPunt(xmx, ymx, 2);
		}

		orien = regio.obtenirOrientacio();
		objecte=((CSistema *)m_sistema)->m_identificacio.identificarObjecte(area, per, dmn, dmx, comp);

		if (objecte==NULL) {
			((CSistema *)m_sistema)->pantallaARobot(&xc, &yc, &orien);
			((CSistema *)m_sistema)->m_interpret.m_objectes.Add(CAuxPropietatObjecte("INDEFINIT", per, xc, yc, area, xmn, xmx, ymn, ymx, dmn, dmx, comp, orien));
		}
		else { 
			((CSistema *)m_sistema)->calcularAgafada(objecte, &xc, &yc, &orien, xmn, ymn);
			((CSistema *)m_sistema)->pantallaARobot(&xc, &yc, &orien);
			((CSistema *)m_sistema)->m_interpret.m_objectes.Add(CAuxPropietatObjecte(objecte->obtenirNom(), per, xc, yc, area, xmn, xmx, ymn, ymx, dmn, dmx, comp, orien));
		}
	}
	if (m_config->m_estilSiluetes!=0) m_camara->llegirDeMemoria(&laImatgeTemporal);

	tfi = clock();
    tduracio = (double)(tfi - tinici);

	FILE *fp;
	fp=fopen("tonilog.log", "a+");
	fprintf(fp, "Temps: %f\n", tduracio/(CLOCKS_PER_SEC));
	fclose(fp);


	((CSistema *)m_sistema)->m_interpret.posarTempsTranscorregut((float)(tduracio/CLOCKS_PER_SEC));
	((CSistema *)m_sistema)->m_interpret.actuacioDesdeAnalisiMoviment();
	((CSistema *)m_sistema)->m_interpret.executarContinu(((CSistema *)m_sistema)->m_cellconfig.m_nomArxiuScript);
}

//
// Respon a l'event de ptjar el bot d'inicialitzar. 
// Posa l'autmat enestat d'inicialitzaci i inicia l'execuci
// contnua (s a dir, es perden els valors de les variables
// internes del script).
//
void CPriAnalisiMoviment::OnBotoInici() 
{
	m_estat=0;
	m_n=0;
	m_tempsMitja=0;
	m_tempsTotal=0;
	m_numMostres=0;
	inicialitzarConfiguracio();
	((CSistema *)m_sistema)->m_interpret.iniciarExecucioContinua();
	SetTimer(ANM_TIMER_ANALISI, m_cella->m_tempsRefresc, NULL);	
}

//
// Es crida aquesta funci quan s'ha determinat que l'objecte est
// enquadrat
//
void CPriAnalisiMoviment::actuar()
{
	if (m_cella->m_aturarCinta) m_robot->desactivarCinta();
	m_comentari="Objecte trobat!";
	UpdateData(false);
	executarScript();
	if (m_cella->m_aturarCinta) m_robot->activarCinta();
}

void CPriAnalisiMoviment::inicialitzarConfiguracio()
{
	m_nMaxInici=m_cella->m_nMaxInici;
	m_nMaxDet=m_cella->m_nMaxDeteccio;
	m_tolerancia=m_cella->m_tolerancia;
	// Clcul de les dimensions de l'rea a observar
	// per a la detecci de moviment.
	int ampladaAnalitzada = m_cella->m_amplada;
	int xf = m_config->m_xFinestra;
	int yf = m_config->m_yFinestra;
	int wf = m_config->m_ampleFinestra;
	int hf = m_config->m_altFinestra;
	int xf2 = xf+wf-ampladaAnalitzada;
	int yf2 = yf ;
	int wf2 = ampladaAnalitzada;
	int hf2 = hf ;
	// Modificaci de l'objecte que emmagatzema la imatge
	// observada en la detecci de moviment.
	m_imatge.posarXFinestra(xf2);
	m_imatge.posarYFinestra(yf2);
	m_imatge.posarAmpleFinestra(wf2);
	m_imatge.posarAltFinestra(hf2);
	// Inicialitzaci de l'estat de l'autmat.
	m_alt=hf2;
	m_ample=wf2;
	m_xinicial=xf2;
	m_yinicial=yf2;
}
