/////////////////////////////////////////////////////////////////
//
// Arxiu      : PriAnalisiMoviment.h
//
// Descripci : Arxiu de capalera de la classe que implementa
//				part de la funcionalitat del processament
//				digital d'imatge.
//				Aquesta classe s'encarrega de la detecci de
//				moviment i de determinar quan un objecte est
//				totalment dins del camp de visi de la cmara.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////


#if !defined(AFX_ANALISIMOVIMENT_H__36F70402_03BD_11D4_B5FE_00C026C084F2__INCLUDED_)
#define AFX_ANALISIMOVIMENT_H__36F70402_03BD_11D4_B5FE_00C026C084F2__INCLUDED_

//
// Arxius de capalera.
//
#include <time.h>			// Per al clcul de temps
#include "PriImatge.h"		// Classe on s'emmagatzema la imatge
							// i s'hi realitza el processat.
#include "DspCamara.h"		// El dispositiu cmara
#include "DlgCfgImatge.h"	// Per a accedir als parmetres
							// de configuraci del processat
							// d'imatge
#include "DlgCfgCella.h"	// Per a accedir als parmetres de
							// configuraci de la cel.la
#include "DspRobot.h"		// El dispositiu robot.

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

#define ANM_TIMER_ANALISI 2

class CPriAnalisiMoviment : public CDialog
{
public:
	// Constructor standard per a tot objecte derivat de CDialog
	CPriAnalisiMoviment(CWnd* pParent = NULL); 
	// Inicialitza l'objecte. Es rep un punter al sistema, per
	// a poder obtenir, a travs seu, tots els parmetres de
	// configuraci necessaris, aix com accs als dispositius
	// externs (cmara i robot).
	void inicialitzar(void *sistema);
	// Alterna entre l'estat de visible i de no visible.
	void alternar();
	// Determina si la finestra est visible o no
	BOOL estaActiu();
	// Es redefineixen OnCancel i OnOK per a impedir que
	// es pugui sortir pitjant ESC o que Intro s'interpreti
	// com a OK.
	void OnCancel();
	void OnOK();

// Dialog Data
	//{{AFX_DATA(CPriAnalisiMoviment)
	enum { IDD = IDD_ANALISI_MOVIMENT };
	long	m_areaVisible;
	UINT	m_estat;
	CString	m_comentari;
	//}}AFX_DATA


// Overrides
	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CPriAnalisiMoviment)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

protected:

	// Generated message map functions
	//{{AFX_MSG(CPriAnalisiMoviment)
	afx_msg void OnTimer(UINT nIDEvent);
	afx_msg void OnBotoActivar();
	afx_msg void OnBotoDesactivar();
	afx_msg void OnBotoCintaon();
	afx_msg void OnBotoCintaof();
	afx_msg void OnBotoInici();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
private:
	float m_numMostres;
	float m_tempsTotal;
	float m_tempsMitja;
	void inicialitzarConfiguracio();
	// Realitza l'actuaci pertinent una vegada s'ha
	// decidit que l'objecte est totalment visible.
	void actuar();
	// Fa els canvis d'estat i actua en conseqncia.
	// L'autmat que s'implementa respon a les segents
	// especificacions:
	// Inicialitzaci: n=0 ; Estat inicial:0
	// Estat 0 : *	si n<nMaxInici i el fotograma actual s
	// (inicial.)	semblant a l'antarior llavors n++ i 
	//				Estat=0;
	//			 *	si n<nMaxInici i el fotograma actual no
	//				s semblant a l'anterior, llavors fer n=0 i
	//				Estat=0;
	//			 *	si n>=nMaxInici llavors imatgeNormal=
	//				imatgeActual i Estat=1;
	// Estat 1 : *	si imatgeActual s semblant a imatgeNormal
	// (sense		llavors Estat=1;
	// objectes) *	si imatgeActual no s semblant a imatgeNormal
	//				llavors fer n=0 i Estat=2;
	// Estat 2 : *	si imatgeActual no s semblant a imatgeNormal
	// (possible	i n<nMaxDet llavors n++ i Estat=2;
	// objecte)  *	si imatgeActual s semblant a imatgeNormal i
	//				n<nMaxDet, llavors Estat=1
	//				si imatgeActual s semblant a imatgeNormal i
	//				n>nMaxDet llavors **actuar** i Estat=1;
	void nouEstat();
	// Executa l'script definit en la configuraci.
	void executarScript();
	// Determina si im1 i im2 sn semblants, atenent-se a
	// la tolerncia indicada.
	bool semblant(long int im1, long int im2, long int tolerancia);
	// Compta el nmero de pixels que no formen part
	// del fons de la imatge. Es suposa que el fons de la
	// imatge s del color 0.
	void comptarPixels();
	// Valors a partir dels quals es fa l'anlisi de la
	// imatge.
	int m_yinicial;
	int m_xinicial;
	int m_alt;
	int m_ample;
	// La matriu de pixels de la imatge
	TPixel ** m_matriu;
	// El punter al sistema.
	void * m_sistema;
	// El robot sobre el qual s'ha d'actuar.
	CDspRobot * m_robot;
	// Configuracions vries.
	CDlgCfgCella * m_cella;
	CDlgCfgImatge * m_config;
	CDspCamara * m_camara;
	// Imatge sobre la qual es treballa. Est declarada
	// aqu per a evitar les solicituds d'espai cada 
	// vegada que es necessiti.
	CPriImatge m_imatge;
	// Parmtres que determinen el funcionament de l'autmat
	long int m_tolerancia;
	int m_n;
	long int m_img2;
	long int m_img1;
	long int m_imgn;
	// Nmero de vegades consecutives que l'autmat ha de passar
	// per l'estat d'inicialitzaci per a considerar que est 
	// correctament inicialitzat.
	int m_nMaxInici;	
	// Nmero de vegades consecutives que l'autmat s'ha de 
	// trobar en l'estat de 'possible objecte' per a considerar
	// que, efectivament, hi ha un objecte a la vista i que 
	// est enquadrat en l'rea de visi de la cmara.
	int m_nMaxDet;		
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Developer Studio will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_ANALISIMOVIMENT_H__36F70402_03BD_11D4_B5FE_00C026C084F2__INCLUDED_)
