/////////////////////////////////////////////////////////////////
//
// Arxiu      : PriImatge.cpp
//
// Descripci : CPriImatge s una classe derivada de CPriImg.
//				Els objectes d'aquesta classe estan pensats per
//				a contenir imatges capturades per la cmara.
//				La classe disposa de mtodes per a la realitzaci
//				de les tasques necessries del processament
//				de la imatge: binaritzaci, clcul de l'histograma,
//				suavitzat de l'histograma, thresholding, erosio,
//				dilataci i anlisi de connectivitat.
//				El resultat de l'anlisi de connectivitat es pot
//				passar a un objecte de la classe CPriImgMarques
//				per a continuar amb el processament.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "PriImatge.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CPriImatge::CPriImatge()
{
}

CPriImatge::~CPriImatge()
{
	if (m_histograma!=NULL) {
		delete m_histograma;
	}
}

int * CPriImatge::obtenirHistograma()
{
	if (m_histogramaCalculat) return m_histograma;
	else return NULL;
}

void CPriImatge::calcularHistograma(int valorMaxim, BOOL emfatitzat)
{
	int i, Max, x, y;
	
	if (m_histograma==NULL) {
		m_error=IMG_NO_MEMORIA_HISTOGRAMA;
		return;
	}
	
	for (i=0;i<IMG_NUM_COLORS;i++)
		m_histograma[i]=0;

	normalitzarFinestra();

	// Clcul de l'histograma
	Max=0;
	for (y=m_yFinestra;y<m_yFinestra+m_altFinestra;y++) {
		for (x=m_xFinestra;x<m_xFinestra+m_ampleFinestra;x++) {
			i=laMatriu[y][x];
			m_histograma[i]++;
			if (m_histograma[i]>Max) Max=m_histograma[i];
		}
	}
	
	// Emfasi de l'histograma si s'escau.
	if (emfatitzat) {
		for (i=0;i<IMG_NUM_COLORS;i++)
			if (m_histograma[i])
				m_histograma[i]=(int)sqrt((float)m_histograma[i]);
		Max=(int)sqrt((float)Max);
	}

	// Normalitzaci de l'histograma d'acort al mxim maxHist
	for (i=0;i<IMG_NUM_COLORS;i++)
		m_histograma[i]=(int)(((float)m_histograma[i] * valorMaxim) / Max);

	// Marcam a la classe que l'histograma s'ha calculat
	m_histogramaCalculat=true;
}

void CPriImatge::suavitzarHistograma(int ampladaSuavitzat)
{
	int i, gv, cen;
	long h;
	int *histSuavitzat;

	// Si l'histograma no est calculat, no fa res.
	if (m_histogramaCalculat) {
	
		if (m_histograma==NULL) {
			m_error=IMG_NO_MEMORIA_HISTOGRAMA;
			return;	
		}
		
		// Solicita memria per l'histograma suavitzat.
		histSuavitzat = new int[IMG_NUM_COLORS];
		if (histSuavitzat==NULL) {
			m_error=IMG_NO_MEMORIA_SUAVITZAT_HISTOGRAMA;
			return;
		}
		
		// Inicialitza l'histograma suavitzat.
		cen = ampladaSuavitzat / 2;
		for (gv=0;gv<IMG_NUM_COLORS;gv++) histSuavitzat[gv]=0;
	
		// A cada valor de l'histograma suavitzat hi assigna
		// la mitjana de els ampladaSuavitzat valors que
		// envolten l'esmentada posici en l'histograma original.
		for (gv=0;gv<IMG_NUM_COLORS-ampladaSuavitzat;gv++) {
			h=0;
			for (i=gv;i<gv+ampladaSuavitzat;i++) 
				h+=(long)m_histograma[i];
			histSuavitzat[gv+cen]=(int)(h/ampladaSuavitzat);
		}
	
		// S'assigna a m_histograma l'histograma suavitzat.
		delete m_histograma;

		m_histograma=histSuavitzat;
	}
	else return; 
}

// Es recorre la imatge pixel a pixel. Es compara el valor
// dels pixels amb un llindar. Si s major o igual al llindar,
// es canvia el color a blanc. Si s menor, es canvia a negre.
void CPriImatge::binaritzar(TPixel llindar)
{
	int i, j;
	normalitzarFinestra();
	for (i=m_yFinestra;i<m_yFinestra+m_altFinestra;i++)
		for (j=m_xFinestra;j<m_xFinestra+m_ampleFinestra;j++) 
			if (laMatriu[i][j]<llindar) laMatriu[i][j]=IMG_COLOR_NEGRE;
			else laMatriu[i][j]=IMG_COLOR_BLANC;
}

// Operador de cpia. Permet copiar tot el contingut d'un objecte
// CPriImatge a un altre.
CPriImatge & CPriImatge::operator =(const CPriImatge & laImg)
{
	int i;
	CPriImg::operator=(laImg);
	if (laImg.m_histograma!=NULL) {
		m_histograma=new int[IMG_NUM_COLORS];
		for (i=0;i<IMG_NUM_COLORS;i++)
			m_histograma[i]=laImg.m_histograma[i];
	}
	m_histogramaCalculat=laImg.m_histogramaCalculat;
	m_llindars.RemoveAll();
	for (i=0;i<laImg.m_llindars.GetSize();i++)
		m_llindars.Add(laImg.m_llindars[i]);
	return *this;
}

// Compta el nmero d'indexos de l'histograma pels quals
// el valor de l'histograma va baixant.
int CPriImatge::comptarHistoBaixen(int inicial)
{
	int i, Dist;

	if (m_histograma[inicial]<=m_histograma[inicial+1]) return 0;

	Dist = inicial;

	for (i=inicial;i<IMG_NUM_COLORS-1;i++)
		if (m_histograma[i]>m_histograma[i+1])
			Dist = i;
		else
			if (m_histograma[i]<m_histograma[i+1]) break;

	return (Dist-inicial);
}

// Compta el nmero d'indexos de l'histograma pels quals
// el valor de l'histograma va pujant.
int CPriImatge::comptarHistoPujen(int inicial)
{
	int i, Dist;

	if (m_histograma[inicial]>=m_histograma[inicial+1]) return 0;

	Dist = inicial;

	for (i=inicial;i<IMG_NUM_COLORS-1;i++)
		if (m_histograma[i]<m_histograma[i+1])
			Dist = i;
		else
			if (m_histograma[i]>m_histograma[i+1]) break;

	return (Dist-inicial);
}

// Localitza les valls de l'histograma, les quals s'empraran
// com a llindars en el procs de thresholding.
void CPriImatge::localitzarMinims(int minAvall, int minAmunt)
{
	int i, d, u, Avall, Amunt;

	m_llindars.SetAtGrow(0,0);
	i=1;
	for (d=0;d<IMG_NUM_COLORS;d++) {
		Avall=comptarHistoBaixen(d);
		// Si el nmero d'indexos que baixen s major que el
		// valor mnim especificat...
		if (Avall>=minAvall) {
			for (u=d+Avall;u<IMG_NUM_COLORS;u++) {
				Amunt=comptarHistoPujen(u);
				// Si el nmero d'indexos que pujen s major que
				// el valor mnim expecificat
				if (Amunt>=minAmunt) {
					// Tenim una vall. 
					m_llindars.SetAtGrow(i,d+Avall+(u-d-Avall)/2);
					i++;
					d=u+Amunt;
					break;
				}
			}
		}
	}
	m_llindars.SetAtGrow(i,IMG_NUM_COLORS-1);
}

void CPriImatge::threshold(int maxHisto, BOOL emfatHisto, int ampleSuav, int minAmunt, int minAvall)
{
	int i, x, y, gv;

	// Es buida l'array de llindars.
	m_llindars.RemoveAll();

	// El calcula l'histograma.
	// El primer parmetre fa referncia al valor
	// mxim que pot agafar l'histograma i el
	// segon indica si es vol emfatitzar l'histograma.
	calcularHistograma(maxHisto, emfatHisto);

	// Es realitza el suavitzat de l'histograma.
	suavitzarHistograma(ampleSuav);
	
	// Localitza els mnims de l'histograma, valors
	// que s'empraran com a llindars en el clcul del
	// threshold.
	// Els mnims s'emmagatzemem en l'array m_llindars.
	localitzarMinims(minAvall, minAmunt);

	// Nmero de valors mnims obtinguts.
	int n=m_llindars.GetSize();

	// Normalitza la finestra de treball.
	// Simplement comprova si aquesta es surt de la pantalla
	// i ho corregeix si fa falta.
	normalitzarFinestra();

	// Recorre tota la imatge, i per cada pixel de la imatge
	// comprova dins de quin dels llindars cau. Desprs
	// assigna al pixel el valor del llindar.
	for (y=m_yFinestra; y<m_yFinestra+m_altFinestra; y++)
		for (x=m_xFinestra; x<m_xFinestra+m_ampleFinestra;x++) 
			for (i=0;i<n-1;i++) {
				gv=(int)laMatriu[y][x];
				if (m_llindars[i]<gv && gv<=m_llindars[i+1]) {
					laMatriu[y][x]=(TPixel)i;
					break;
				}
			}

	// Es marca que el threshold ha estat calculat.
	m_thresholdCalculat=true;
}

void CPriImatge::ferThresholdVisible()
{
	int i, j;

	// Si el threshold no est calculat, es retorna
	// sense fer res.
	if (!m_thresholdCalculat) return;

	// Normalitza la finestra de treball.
	// Simplement comprova si aquesta es surt de la pantalla
	// i ho corregeix si fa falta.
	normalitzarFinestra();

	// Recorre tota la imatge. Per a cada pixel
	// canvia el seu valor pel que li correspon
	// segons m_llindars..
	for (j=m_yFinestra;j<m_yFinestra+m_altFinestra;j++)
		for (i=m_xFinestra;i<m_xFinestra+m_ampleFinestra;i++)
			laMatriu[j][i]=(TPixel)m_llindars[laMatriu[j][i]];
}

TPixel ** CPriImatge::obtenirMatriu()
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	return (CPriImg::obtenirMatriu());
}

void CPriImatge::posarPixel(int x, int y, TPixel p)
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	CPriImg::posarPixel(x, y, p);	
}


void CPriImatge::posarAmpleImatge(int ample)
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	CPriImg::posarAmpleImatge(ample);
}

void CPriImatge::posarAltImatge(int alt)
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	CPriImg::posarAltImatge(alt);
}

void CPriImatge::posarAmpleFinestra(int amplef)
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	CPriImg::posarAmpleFinestra(amplef);
}

void CPriImatge::posarAltFinestra(int altf)
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	CPriImg::posarAltFinestra(altf);
}

void CPriImatge::posarXFinestra(int x)
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	CPriImg::posarXFinestra(x);
}

void CPriImatge::posarYFinestra(int y)
{
	m_histogramaCalculat=false;
	m_thresholdCalculat=false;
	CPriImg::posarYFinestra(y);
}

void CPriImatge::erosionar(int tamanyMascara)
{
	int i, j, k, l, x, y, NumThres, n, x_tmp, y_tmp, x2_tmp, y2_tmp;
	TPixel **temp;

	// Si el threshold no est calculat no s'inicia el procs
	// d'erosi.
	if (!m_thresholdCalculat) return;

	// Si ja tenim una mscara del mateix tamany que la que
	// es necessita, no fa falta recalcular-la.
	if (!m_mascaraCalculada || m_mascara.tamany!=tamanyMascara)
		calcularMascara(tamanyMascara);

	// Solicitud de memria per a una imatge temporal.
	temp = new TPixelPtr[m_altImatge];
	if (temp == NULL) {
		m_error=IMG_NO_MEMORIA_IMATGE_ERO;
		return;
	}
	for (i=0;i<m_altImatge;i++) {
		temp[i]= new TPixel[m_ampleImatge];
		if (temp[i] == NULL) {
			m_error=IMG_NO_MEMORIA_IMATGE_ERO;
			return;
		}
	}

	// Realitzaci del procs d'erosi.
	NumThres=m_llindars.GetSize();
	
	normalitzarFinestra();
	
	x_tmp = m_xFinestra;
	y_tmp = m_yFinestra;
	x2_tmp = m_ampleFinestra + m_xFinestra;
	y2_tmp = m_altFinestra + m_yFinestra;

	for (i=0;i<m_altImatge;i++)
		for (j=0;j<m_ampleImatge;j++)
			temp[i][j]=0;

	// Es passa un rectangle de nxn per tota la imatge.
	// Si a les coordenades (x,y) de la imatge tot el
	// rectangle s pl, a la coordenada (x,y) de la imatge
	// dest es posa un pixel. Si no es pl, no es posa.
	for (n=0;n<NumThres;n++) {
		for (j=y_tmp;j<y2_tmp;j++) {
			for (i=x_tmp;i<x2_tmp;i++) {
				for (k=0;k<m_mascara.tamany;k++) {
					for (l=0;l<m_mascara.tamany;l++) {
						x=i+m_mascara.x[k][l];
						y=j+m_mascara.y[k][l];
						if (y>=y_tmp && x>=x_tmp && y<y2_tmp && x<x2_tmp) 
							if (laMatriu[y][x]!=(TPixel)n) goto Fora;
					}
				}
				temp[j][i]=(TPixel)n;
Fora: ;
			}
		}
	}
	alliberarMatriu();
	laMatriu=temp;
}

void CPriImatge::calcularMascara(int tamanyMascara)
{
	int i, j, meitat;

	eliminarMascara();

	m_mascara.tamany=tamanyMascara;

	m_mascara.x=new TIntPtr[tamanyMascara];
	if (m_mascara.x==NULL) 
		m_error=IMG_NO_MEMORIA_MASCARA;

	m_mascara.y=new TIntPtr[tamanyMascara];
	if (m_mascara.y==NULL)
		m_error=IMG_NO_MEMORIA_MASCARA;

	for (i=0;i<tamanyMascara;i++) {
		m_mascara.x[i]=new int[tamanyMascara];
		m_mascara.y[i]=new int[tamanyMascara];
		if (m_mascara.x[i]==NULL || m_mascara.y[i]==NULL)
			m_error=IMG_NO_MEMORIA_MASCARA;
	}
	
	// Creaci de la mscara.
	// P.ex. mscara de 3x3:
	// +----+----+----+
	// | -1 |  0 |  1 |
	// | -1 | -1 | -1 |
	// +----+----+----+
	// | -1 |  0 |  1 |
	// |  0 |  0 |  0 |
	// +----+----+----+
	// | -1 |  0 |  1 |
	// |  1 |  1 |  1 |
	// +----+----+----+


	meitat=((tamanyMascara%2)==0)?(tamanyMascara/2):((tamanyMascara-1)/2);

	for (i=0;i<tamanyMascara;i++) 
		for (j=0;j<tamanyMascara;j++) {
			m_mascara.x[i][j]=i-meitat;
			m_mascara.y[i][j]=j-meitat;
		}
}

void CPriImatge::eliminarMascara()
{
	int i;

	if (m_mascara.x!=NULL) {
		for (i=0;i<m_mascara.tamany;i++) 
			if (m_mascara.x[i]!=NULL) 
				delete m_mascara.x[i];
		delete m_mascara.x;
	}

	if (m_mascara.y!=NULL) {
		for (i=0;i<m_mascara.tamany;i++)
			if (m_mascara.y[i]!=NULL) {
					delete m_mascara.y[i];
			}
			delete m_mascara.y;
	}

	m_mascara.tamany=0;
}

void CPriImatge::dilatar(int tamanyMascara)
{
	int i, j, k, l, x, y, NumThres, n, x_tmp, y_tmp, x2_tmp, y2_tmp;
	TPixel **temp;

	// Si el threshold no est calculat no s'inicia el procs
	// de dilataci.
	if (!m_thresholdCalculat) return;

	// Si ja tenim una mscara del mateix tamany que la que
	// es necessita, no fa falta recalcular-la.
	if (!m_mascaraCalculada || m_mascara.tamany!=tamanyMascara)
		calcularMascara(tamanyMascara);

	// Solicitud de memria per a una imatge temporal.
	temp = new TPixelPtr[m_altImatge];
	if (temp == NULL) {
		m_error=IMG_NO_MEMORIA_IMATGE_DIL;
		return;
	}
	for (i=0;i<m_altImatge;i++) {
		temp[i]= new TPixel[m_ampleImatge];
		if (temp[i] == NULL) {
			m_error=IMG_NO_MEMORIA_IMATGE_DIL;
			return;
		}
	}

	// Realitzaci del procs de dilataci.
	NumThres=m_llindars.GetSize();
	
	normalitzarFinestra();
	
	x_tmp = m_xFinestra;
	y_tmp = m_yFinestra;
	x2_tmp = m_ampleFinestra + m_xFinestra;
	y2_tmp = m_altFinestra + m_yFinestra;

	for (i=0;i<m_altImatge;i++)
		for (j=0;j<m_ampleImatge;j++)
			temp[i][j]=0;

	// Es passa un rectangle de nxn per tota la imatge.
	// Si a les coordenades (x,y) de la imatge hi ha un o ms
	// pixels dins del rectangle, a la coordenada (x,y) de la imatge
	// dest es posa un pixel. En cas contrari, no es posa.
	for (n=0;n<NumThres;n++) {
		for (j=y_tmp;j<y2_tmp;j++) {
			for (i=x_tmp;i<x2_tmp;i++) {
				for (k=0;k<m_mascara.tamany;k++) {
					for (l=0;l<m_mascara.tamany;l++) {
						x=i+m_mascara.x[k][l];
						y=j+m_mascara.y[k][l];
						if (y>=y_tmp && x>=x_tmp && y<y2_tmp && x<x2_tmp) 
							if (laMatriu[y][x]==(TPixel)n) {
								temp[j][i]=(TPixel)n;
								goto Fora;
							}
					}
				}
Fora:;
 
			}
		}
	}
	alliberarMatriu();
	laMatriu=temp;
}

// Realitza l'anlisi de connectivitat.
// Es necessari que s'hagi fet el threshold abans.
void CPriImatge::crearMarques(CPriImgMarques * imgm)
{
	int x, y, marca, xu, yu, xc, yc, xl, yl, U, C, L, Um, Lm;
	int xf1, xf2, yf1, yf2;
	TPixel **markIm;
	CAuxLlistaEquivalencia le;

	normalitzarFinestra();
	if (imgm->obtenirAmpleImatge()!=m_ampleImatge || imgm->obtenirAltImatge()!=m_altImatge) {
		m_error=IMG_TAMANY_MARQUES_INCORRECTE;
		return;
	}
	imgm->posarXFinestra(m_xFinestra);
	imgm->posarYFinestra(m_yFinestra);
	imgm->posarAmpleFinestra(m_ampleFinestra);
	imgm->posarAltFinestra(m_altFinestra);

	markIm=imgm->obtenirMatriu();

	xf1=m_xFinestra;
	xf2=m_xFinestra+m_ampleFinestra;
	yf1=m_yFinestra;
	yf2=m_yFinestra+m_altFinestra;

	for (x=xf1;x<xf2;x++) {
		laMatriu[yf1][x]=0;
		laMatriu[yf2-1][x]=0;
	}

	for (y=yf1;y<yf2;y++) {
		laMatriu[y][xf1]=0;
		laMatriu[y][xf2-1]=0;
	}

	marca=0;
	le.posarValor(marca, marca);

	for (y=yf1+1;y<yf2-1;y++) {
		for (x=xf1+1;x<xf2-1;x++) {
			xu=x;   yu=y-1;
			xc=x;   yc=y;
			xl=x-1; yl=y;
			U=(int)laMatriu[yu][xu];
			C=(int)laMatriu[yc][xc];
			L=(int)laMatriu[yl][xl];
			if (C) {
				if (C==U && C!=L) {
					markIm[yc][xc]=markIm[yu][xu];
				} else {
					if (C==L && C!=U) {
						markIm[yc][xc]=markIm[yl][xl];
					} else {
						if (C==L && C==U) {
							Lm=markIm[yl][xl];
							Um=markIm[yu][xu];
							markIm[yc][xc]=Lm;
							if (Lm!=Um) {
								Lm=le.obtenirDarreraMarca(Lm);
								Um=le.obtenirDarreraMarca(Um);
								le.posarValor(Lm, Um);
							}
						} else {
							marca++;
							markIm[yc][xc]=marca;

							le.posarValor(marca, marca);
						}
					}
				}
			}
		}
	}

	imgm->posarNumMarques(le.arreglarLlista());

	for (y=yf1;y<yf2;y++)
		for (x=xf1;x<xf2;x++)
			if (markIm[y][x])
				markIm[y][x]=le.obtenirValor(markIm[y][x]);
}

void CPriImatge::inicialitzar(int ample, int alt, int xf, int yf, int amplef, int altf)
{
	CPriImg::inicialitzar(ample, alt, xf, yf, amplef, altf);
	m_histogramaCalculat=false;
	m_histograma=new int[IMG_NUM_COLORS];
	if (m_histograma==NULL) {
		m_error=IMG_NO_MEMORIA_HISTOGRAMA;
		AfxMessageBox("No hi ha memria per a allotjar l'histograma");
		return;
	}
	m_llindars.RemoveAll();
	m_mascara.x=NULL;
	m_mascara.y=NULL;
	m_thresholdCalculat=false;
}

CIntArray * CPriImatge::obtenirLlindars()
{
	return &m_llindars;
}
