/////////////////////////////////////////////////////////////////
//
// Arxiu      : PriImatge.h
//
// Descripci : CPriImatge s una classe derivada de CPriImg.
//				Els objectes d'aquesta classe estan pensats per
//				a contenir imatges capturades per la cmara.
//				La classe disposa de mtodes per a la realitzaci
//				de les tasques necessries del processament
//				de la imatge: binaritzaci, clcul de l'histograma,
//				suavitzat de l'histograma, thresholding, erosio,
//				dilataci i anlisi de connectivitat.
//				El resultat de l'anlisi de connectivitat es pot
//				passar a un objecte de la classe CPriImgMarques
//				per a continuar amb el processament.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#if !defined(AFX_IMG_H__4E90EA72_EABC_11D3_B5CC_00C026C084F2__INCLUDED_)
#define AFX_IMG_H__4E90EA72_EABC_11D3_B5CC_00C026C084F2__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//
// Arxius de capalera
//
#include "PriImg.h"
#include "PriImgMarques.h"
#include "AuxLlistaEquivalencia.h"

//
// Array de sencers. Creix automticament.
//
typedef CArray<int, int> CIntArray;

//
// Estructura per a contenir una mscara d'erosi o
// dilataci.
//
typedef struct TMascara {
  int tamany;
  int **x;
  int **y;
} TStrEl; 

typedef int * TIntPtr; // Punter a sencer.


class CPriImatge : public CPriImg  
{
public:
	CIntArray * obtenirLlindars();
	void inicialitzar(int ample=IMG_AMPLEI_DEFECTE, int alt=IMG_ALTI_DEFECTE, int xf=IMG_XF_DEFECTE, int yf=IMG_YF_DEFECTE, int amplef=IMG_AMPLEF_DEFECTE, int altf=IMG_ALTF_DEFECTE);
	void crearMarques(CPriImgMarques *imgm);	
	void dilatar(int tamanyMascara);
	void posarYFinestra(int y);
	void posarXFinestra(int x);
	void posarAltFinestra(int altf);
	void posarAmpleFinestra(int amplef);
	void posarAltImatge(int alt);
	void posarAmpleImatge(int ample);
	void posarPixel(int x, int y, TPixel p);
	TPixel ** obtenirMatriu();
	void erosionar(int tamanyMascara);	// Realitza l'erosi de la
										// imatge sols si s un
										// threshold. Si ja hi
										// ha una mscara calculada
										// de tamanyMascara, no la
										// torna a calcular. En cas
										// contrari, la calcula.
	void ferThresholdVisible();
	void threshold(int maxHisto, BOOL emfatHisto, int ampleSuav, int minAmunt, int minAvall);
	CPriImatge & operator=(const CPriImatge &laImg);
	void binaritzar(TPixel llindar);
	void suavitzarHistograma(int ampladaSuavitzat);
								// Suavitza l'histograma segons
								// ampladaSuavitzat, en cas que
								// aquest estigui calculat. Si
								// no ho est, no fa res.
	void calcularHistograma(int valorMaxim, BOOL emfatitzat);
	int * obtenirHistograma();  // Si l'histograma ha estat 
								// calculat, el retorna. Si no
								// retorna NULL.
	void localitzarMinims(int minAvall, int minAmunt);
	CPriImatge();
	virtual ~CPriImatge();

private:
	void eliminarMascara();
	void calcularMascara(int tamanyMascara);
	bool m_mascaraCalculada;
	TStrEl m_mascara;
	bool m_thresholdCalculat;
	int comptarHistoPujen(int inicial);
	int comptarHistoBaixen(int inicial);
	CIntArray m_llindars;
	bool m_histogramaCalculat;
	int * m_histograma;
};

#endif // !defined(AFX_IMG_H__4E90EA72_EABC_11D3_B5CC_00C026C084F2__INCLUDED_)
