/////////////////////////////////////////////////////////////////
//
// Arxiu      : PriImg.cpp
//
// Descripci : CPriImg s la classe base per a CPriImatge, 
//				CPriImgMarques i CPriRegio. 
//				CPriImg s un contenidor de pixels, amb les
//				funcions bsiques d'accs.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BRAS.h"
#include "PriImg.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// El constructor de la classe no solicita espai per a la
// matriu, aix es fa en el mtode 'inicialitzar'. 
// S'ha optat per aquesta soluci ja que les classes de les
// quals se'n declaren objectes sn les derivades de 
// CPriImg, i quan es crea un objecte d'una classe derivada
// es crida automticament el constructor per defecte 
// de la classe base. 

CPriImg::CPriImg()
{
	laMatriu=NULL;
}

CPriImg::~CPriImg()
{
	alliberarMatriu();
}

int CPriImg::obtenirError()
{
	return m_error;
}

int CPriImg::obtenirAmpleImatge()
{
	return m_ampleImatge;
}

int CPriImg::obtenirAltImatge()
{
	return m_altImatge;
}

int CPriImg::obtenirAmpleFinestra()
{
	return m_ampleFinestra;
}

int CPriImg::obtenirAltFinestra()
{
	return m_altFinestra;
}

int CPriImg::obtenirXFinestra()
{
	return m_xFinestra;
}

int CPriImg::obtenirYFinestra()
{
	return m_yFinestra;
}

void CPriImg::posarAmpleImatge(int ample)
{
	m_ampleImatge=ample;
}

void CPriImg::posarAltImatge(int alt)
{
	m_altImatge=alt;
}

void CPriImg::posarAmpleFinestra(int amplef)
{
	m_ampleFinestra=amplef;
}

void CPriImg::posarAltFinestra(int altf)
{
	m_altFinestra=altf;
}

void CPriImg::posarXFinestra(int xf)
{
	m_xFinestra=xf;
}

void CPriImg::posarYFinestra(int yf)
{
	m_yFinestra=yf;
}

void CPriImg::posarPixel(int x, int y, TPixel p)
{
	if (posicioValida(x, y))
		laMatriu[y][x]=p;
}

TPixel CPriImg::obtenirPixel(int x, int y)
{
	if (posicioValida(x, y))
		return (laMatriu[y][x]);
	else
		return IMG_COLOR_NEGRE;
}

bool CPriImg::posicioValida(int x, int y)
{
	return (x>=0 && x<m_ampleImatge && y>=0 && y<m_altImatge);
}

CPriImg& CPriImg::operator =(const CPriImg & laImatge)
{
	int i, j;

	//
	// Cpia dels parmetres de la imatge
	//
	m_altFinestra=laImatge.m_altFinestra;
	m_altImatge=laImatge.m_altImatge;
	m_ampleFinestra=laImatge.m_ampleFinestra;
	m_ampleImatge=laImatge.m_ampleImatge;
	m_error=laImatge.m_error;
	m_xFinestra=laImatge.m_xFinestra;
	m_yFinestra=laImatge.m_yFinestra;

	//
	// Si la imatge dest de la cpia no est buida, la buidam.
	//
	if (laMatriu!=NULL) {
		for (i=0;i<m_altImatge;i++)
			if (laMatriu[i]!=NULL) 
				delete laMatriu[i];
		delete laMatriu;
	}

	// 
	// Es solicita memria per a la nova imatge. A mesura que 
	// es va solicitant la memria es realitza la cpia.
	//
	laMatriu = new TPixelPtr[m_altImatge];
	if (laMatriu == NULL) m_error=IMG_NO_MEMORIA_IMATGE;

	for (i=0;i<m_altImatge;i++) {
		laMatriu[i]= new TPixel[m_ampleImatge];
		if (laMatriu[i] == NULL) m_error=IMG_NO_MEMORIA_IMATGE;
		else {
			for (j=0;j<m_ampleImatge;j++)
				laMatriu[i][j]=laImatge.laMatriu[i][j];
		}

	}
	return *this;
}

void CPriImg::normalitzarFinestra()
{
	if ((m_ampleFinestra+m_xFinestra)>(m_ampleImatge-3)) 
		m_ampleFinestra=m_ampleImatge-m_xFinestra-4;
	if ((m_altFinestra+m_yFinestra)>(m_altImatge-3))
		m_altFinestra=m_altImatge-m_yFinestra-4;
	if (m_xFinestra<4) m_xFinestra=4;
	if (m_yFinestra<4) m_yFinestra=4;
}

TPixel ** CPriImg::obtenirMatriu()
{
	return laMatriu;
}

void CPriImg::alliberarMatriu()
{
	int i;
	if (laMatriu!=NULL) {
		for (i=0;i<m_altImatge;i++)
			if (laMatriu[i]!=NULL) delete[] laMatriu[i];
		delete laMatriu;
	}
}


void CPriImg::inicialitzar(int ample, int alt, int xf, int yf, int amplef, int altf)
{
	int i;
	
	m_altFinestra=altf;
	m_ampleFinestra=amplef;
	m_altImatge=alt;
	m_ampleImatge=ample;
	m_xFinestra=xf;
	m_yFinestra=yf;
	m_error=IMG_SENSE_ERROR;
	
	// La matriu  de pixels a on s'ha d'emmagatzemar la imatge
	// es solicita dinmicament.
	// En cas de problemes en aquesta solicitud dinmica es
	// genera el codi d'error corresponen i es surt del 
	// constructor.

	alliberarMatriu();
	laMatriu = new TPixelPtr[alt];
	if (laMatriu == NULL) {
		m_error=IMG_NO_MEMORIA_IMATGE;
		AfxMessageBox("No hi ha memria per a allotjar l'imatge");
		return;
	}
	for (i=0;i<alt;i++) {
		laMatriu[i]= new TPixel[ample];
		if (laMatriu[i] == NULL) {
			m_error=IMG_NO_MEMORIA_IMATGE;
			AfxMessageBox("No hi ha memria per a allotjar l'histograma");
			return;
		}
	}
	// Una imatge recen creada s totalment negre.
	omplir(IMG_COLOR_NEGRE);
}

void CPriImg::dibuixarPunt(int x, int y, int tam)
{
	int i, j;
	for (i=x-tam;i<x+tam;i++)
		for (j=y-tam;j<y+tam;j++)
			posarPixel(i, j, IMG_COLOR_BLANC);
}

void CPriImg::omplir(TPixel c)
{
	int i, j;
	for (i=0;i<m_altImatge;i++)
		for (j=0;j<m_ampleImatge;j++)
			laMatriu[i][j]=c;
}
