/////////////////////////////////////////////////////////////////
//
// Arxiu      : PriImg.h
//
// Descripci : CPriImg s la classe base per a CPriImatge, 
//				CPriImgMarques i CPriRegio. 
//				CPriImg s un contenidor de pixels, amb les
//				funcions bsiques d'accs.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#if !defined(AFX_IMATGE_H__735AC0F2_EA92_11D3_B5CB_00C026C084F2__INCLUDED_)
#define AFX_IMATGE_H__735AC0F2_EA92_11D3_B5CB_00C026C084F2__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//
// Definici de les constants
//

#define IMG_ALTF_DEFECTE   480
#define IMG_AMPLEF_DEFECTE 640
#define IMG_XF_DEFECTE      0
#define IMG_YF_DEFECTE      0
#define IMG_ALTI_DEFECTE   480
#define IMG_AMPLEI_DEFECTE 640
#define IMG_NUM_COLORS     256
#define IMG_OBJECTE        255
#define IMG_FONS            0
#define IMG_PI             3.14159

//
// Definici d'alguns colors
//

#define IMG_COLOR_NEGRE 0
#define IMG_COLOR_BLANC 255

//
// Definici dels errors
//

#define IMG_SENSE_ERROR						0
#define IMG_NO_MEMORIA_IMATGE				1
#define IMG_NO_MEMORIA_HISTOGRAMA			2
#define IMG_NO_MEMORIA_SUAVITZAT_HISTOGRAMA 3
#define IMG_NO_MEMORIA_MASCARA              4
#define IMG_NO_MEMORIA_IMATGE_ERO           5
#define IMG_NO_MEMORIA_IMATGE_DIL           8
#define IMG_NO_MEMORIA_IMATGE_CONN          6
#define IMG_NO_MEMORIA_IMATGE_REG           7
#define IMG_TAMANY_MARQUES_INCORRECTE       9
#define IMG_TAMANY_REGIO_INCORRECTE        10
#define IMG_TAMANY_IMATGE_INCORRECTE       11

//
// Definici de tipus
//

typedef unsigned short TPixel;
typedef TPixel *TPixelPtr;

class CPriImg  
{
public:
	void omplir (TPixel c);
	void dibuixarPunt(int x, int y, int tam);
	void inicialitzar(int ample=IMG_AMPLEI_DEFECTE, int alt=IMG_ALTI_DEFECTE, int xf=IMG_XF_DEFECTE, int yf=IMG_YF_DEFECTE, int amplef=IMG_AMPLEF_DEFECTE, int altf=IMG_ALTF_DEFECTE);
	// Operador de cpia
	CPriImg& operator=(const CPriImg& laImatge);
	//
	// Funcions d'accs i modificaci dels membres de
	// la classe.
	//
	TPixel obtenirPixel(int x, int y);
	void posarPixel(int x, int y, TPixel p);
	void posarYFinestra(int yf);
	void posarXFinestra(int xf);
	void posarAltFinestra(int altf);
	void posarAmpleFinestra(int amplef);
	void posarAltImatge(int alt);
	void posarAmpleImatge(int ample);
	int obtenirYFinestra();
	int obtenirXFinestra();
	int obtenirAltFinestra();
	int obtenirAmpleFinestra();
	int obtenirAltImatge();
	int obtenirAmpleImatge();
	int obtenirError();
	// La funci obtenirMatriu permet accedir directament a la
	// matriu de pixels. Aix romp, en part, l'encapsulament,
	// per s'ha optat per aquesta implementaci per tal d'accelerar
	// la cpia de dades des del buffer de la cmara a la
	// memria de l'ordinador. Veure la implementaci de
	// CDspCamara::llegirDeMemoria i CDspCamara::posarEnMemoria
	TPixel ** obtenirMatriu();

	// Constructor
	CPriImg();
	
	virtual ~CPriImg();

protected:
	void normalitzarFinestra();	// Comprova que la finestra de
								// de treball estigui totalment
								// continguda dins de la imatge.
								// Si no s aix la redueix per
								// tal que hi cpiga.
	bool posicioValida(int x, int y);
	void alliberarMatriu();
	int m_error;
	int m_altFinestra;
	int m_ampleFinestra;
	int m_xFinestra;
	int m_yFinestra;
	int m_altImatge;
	int m_ampleImatge;
	TPixel ** laMatriu;
};

#endif // !defined(AFX_IMATGE_H__735AC0F2_EA92_11D3_B5CB_00C026C084F2__INCLUDED_)
