// Regio.cpp: implementation of the CPriRegio class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BRAS.h"
#include "PriRegio.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CPriRegio::CPriRegio()
{
}

CPriRegio::~CPriRegio()
{
}

void CPriRegio::inicialitzar(int ample, int alt, int xf, int yf, int amplef, int altf)
{
	CPriImg::inicialitzar(ample, alt, xf, yf, amplef, altf);
	m_areaCalculada=false;
	m_centreCalculat=false;
	m_perimetreCalculat=false;
	m_distanciesCalculades=false;
	m_orientacioCalculada=false;
}

void CPriRegio::calcularArea()
{
	int x, y;
	long n=0;

	for (y=0;y<m_altImatge;y++)
		for (x=0;x<m_ampleImatge;x++)
			if (laMatriu[y][x]==IMG_OBJECTE) n++;

	m_area=n;
	m_areaCalculada=true;
}

long CPriRegio::obtenirArea()
{
	if (!m_areaCalculada) calcularArea();
	return m_area;
}

void CPriRegio::obtenirCentreGravetat(int * xc, int * yc)
{
	if (!m_centreCalculat) calcularCentreGravetat();
	*xc=m_xCentre;
	*yc=m_yCentre;
}

void CPriRegio::calcularCentreGravetat()
{
	int x, y;
	long xc, yc, n;

	n=obtenirArea();

	xc=0;
	yc=0;

	for (y=0;y<m_altImatge;y++)
		for (x=0;x<m_ampleImatge;x++)
			if (laMatriu[y][x]==IMG_OBJECTE) {
				yc+=y;
				xc+=x;
			}
	m_xCentre=(int)(xc/n);
	m_yCentre=(int)(yc/n);
	m_centreCalculat=true;
}

TPixel ** CPriRegio::obtenirMatriu()
{
	m_areaCalculada=false;
	m_centreCalculat=false;
	m_perimetreCalculat=false;
	m_distanciesCalculades=false;
	return CPriImg::obtenirMatriu();
}

void CPriRegio::calcularPerimetre()
{
	int r, c;
	TPixel **m;
	normalitzarFinestra();

	//m_contorn.~CPriImg();

	m_contorn.inicialitzar(m_ampleImatge, m_altImatge, m_xFinestra, m_yFinestra, m_ampleFinestra, m_altFinestra);

	m=m_contorn.obtenirMatriu();

	//
	// Es dibuixa el contorn
	//

	for (r=1;r<m_altImatge;r++)
		for (c=1;c<m_ampleImatge;c++)
			if (!laMatriu[r][c-1] && laMatriu[r][c]) m[r][c]=IMG_COLOR_BLANC;
			else if (laMatriu[r][c-1] && !laMatriu[r][c]) m[r][c-1]=IMG_COLOR_BLANC;

	for (r=1;r<m_altImatge;r++)
		for (c=1;c<m_ampleImatge;c++)
			if (!laMatriu[r-1][c] && laMatriu[r][c]) m[r][c]=IMG_COLOR_BLANC;
			else if (laMatriu[r-1][c] && !laMatriu[r][c]) m[r-1][c]=IMG_COLOR_BLANC;

	//
	// Es calcula el perimetre
	//
	m_perimetre=0;
	for (r=0;r<m_altImatge;r++) 
		for (c=0;c<m_ampleImatge;c++)
			if (m[r][c]==IMG_COLOR_BLANC) m_perimetre++;

	m_perimetreCalculat=true;
}

int CPriRegio::obtenirPerimetre(CPriImg * img)
{
	int i, j;
	if (!m_perimetreCalculat) calcularPerimetre();
	if (img==NULL) return m_perimetre;
	if (img->obtenirAltImatge()!=m_contorn.obtenirAltImatge() || img->obtenirAmpleImatge()!=m_contorn.obtenirAmpleImatge())
		return m_perimetre;
	for (i=0;i<m_contorn.obtenirAltImatge();i++)
		for (j=0;j<m_contorn.obtenirAmpleImatge();j++)
			if (m_contorn.obtenirPixel(j, i)==IMG_COLOR_BLANC)
				img->posarPixel(j, i, IMG_COLOR_BLANC);
	return m_perimetre;
}

double CPriRegio::obtenirCompacitat()
{
	if (!m_perimetreCalculat) calcularPerimetre();
	if (!m_areaCalculada) calcularArea();
	return (((double)m_perimetre*(double)m_perimetre)/(4.0*IMG_PI*(double)m_area));
}

void CPriRegio::calcularDMinDMax()
{
	int i, j, d;
	m_dMin=30000;
	m_dMax=0;
	if (!m_centreCalculat) calcularCentreGravetat();
	if (!m_perimetreCalculat) calcularPerimetre();
	for (i=0;i<m_contorn.obtenirAltImatge();i++)
		for (j=0;j<m_contorn.obtenirAmpleImatge();j++)
			if (m_contorn.obtenirPixel(j, i)==IMG_COLOR_BLANC) {
				d=(int)distancia(j, i, m_xCentre, m_yCentre);
				if (d<m_dMin) {
					m_dMin=d;
					m_xDMin=j;
					m_yDMin=i;
				}
				if (d>m_dMax) {
					m_dMax=d;
					m_xDMax=j;
					m_yDMax=i;
				}
			}
	m_distanciesCalculades=true;
}

double CPriRegio::distancia(double x1, double y1, double x2, double y2)
{
	double t1, t2;
	t1=(x1-x2)*(x1-x2);
	t2=(y1-y2)*(y1-y2);
	return sqrt(t1+t2);
}

void CPriRegio::obtenirDMin(int * x, int * y, int * d)
{
	if (!m_distanciesCalculades) calcularDMinDMax();
	*x=m_xDMin;
	*y=m_yDMin;
	*d=m_dMin;
}

void CPriRegio::obtenirDMax(int * x, int * y, int * d)
{
	if (!m_distanciesCalculades) calcularDMinDMax();
	*x=m_xDMax;
	*y=m_yDMax;
	*d=m_dMax;
}

double CPriRegio::obtenirOrientacio()
{
	//if (!m_centreCalculat) calcularCentreGravetat();
	//if (!m_distanciesCalculades) calcularDMinDMax();
	//return (atan2(m_yDMin-m_yCentre, m_xDMin-m_xCentre));
	//if (!m_orientacioCalculada) 
	calcularOrientacio();
	return m_orientacio;
}

void CPriRegio::calcularOrientacio()
{
	int i, j;
	long int a=0, b=0, c=0;
	if (!m_centreCalculat) calcularCentreGravetat();
	for (j=0;j<m_altImatge;j++) {
		for (i=0;i<m_ampleImatge;i++) {
			if (laMatriu[j][i]==IMG_COLOR_BLANC) {
				a+=(i-m_xCentre)*(i-m_xCentre);
				b+=(i-m_xCentre)*(j-m_yCentre);
				c+=(j-m_yCentre)*(j-m_yCentre);
			}
		}
	}
	b*=2;
	m_orientacio=(atan2(b, (a-c))/2)+(IMG_PI/2);
	m_orientacioCalculada=true;
}
