/////////////////////////////////////////////////////////////////
//
// Arxiu      : ScrIdentificacio.cpp
//
// Descripci : Aquesta classe permet classificar un objecte
//				en funci de les propietats que t.
//				A ms, un objecte d'aquesta classe pot carregar
//				i guardar arxius .dob.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "ScrIdentificacio.h"
#include "Sistema.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CScrIdentificacio::CScrIdentificacio()
{
}

CScrIdentificacio::~CScrIdentificacio()
{
}

//
// Carrega un arxiu en format .dob i el posa en una
// estructura de dades CLlistaObjectes.
// Realitza, per tant, un parsejat de l'arxiu en qesti
//
void CScrIdentificacio::carregarObjectes(CString nomArxiu)
{
	char missatge[200];

	m_tokenActual.posarDades(tkIndefinit);
	m_liniaActual=_T("");
	m_numLiniaActual=0;
	m_posicioActual=0;

	m_nomArxiu=nomArxiu;
	m_fitxer = new CFile;
	if (!m_fitxer->Open(nomArxiu, CFile::modeRead)) {
		delete m_fitxer;
		return;
	}
	m_arxiu = new CArchive(m_fitxer, CArchive::load);

	borrarObjectes();
	parseLLISTAOBJECTES();

	if (m_errorParsejant) {
		m_errorParsejant=false;
		if (m_objecteActual!=NULL) {
			delete m_objecteActual;
		}
		if (m_criteriActual!=NULL) {
			delete m_criteriActual;
		}
		if (m_elementExpressioActual!=NULL) {
			delete m_elementExpressioActual;
		}
		sprintf(missatge, "Error a linia %d caracter %d de l'arxiu de definici d'objectes: %s ", m_liniaError, m_columnaError, m_textError);
		AfxMessageBox(missatge);
		borrarObjectes();
	}

	m_arxiu->Close();
	m_fitxer->Close();
}

void CScrIdentificacio::obtenirToken(CAuxToken *elToken)
{
	char tmp[30];
	double valortmp;
	int i;

	if (m_posicioActual>=m_liniaActual.GetLength()-1) {
		m_posicioActual=0;
		m_numLiniaActual++;
		if (!m_arxiu->ReadString(m_liniaActual)) {
			elToken->posarDades(tkFinal);
			return;
		}
	}
	// Botam els espais.
	for (i=m_posicioActual;i<m_liniaActual.GetLength() && (m_liniaActual[i]==' ' || m_liniaActual[i]=='\n' || m_liniaActual[i]=='\f' || m_liniaActual[i]=='\r'|| m_liniaActual[i]=='\t');i++);
	m_posicioActual=i;

	// Llegim la cadena d'entrada
	for (i=m_posicioActual;i<m_liniaActual.GetLength() && m_liniaActual[i]!=' ' && m_liniaActual[i]!='\n' && m_liniaActual[i]!='\f' && m_liniaActual[i]!='\r' && m_liniaActual[i]!='\t';i++) 
		tmp[i-m_posicioActual]=m_liniaActual[i];
	tmp[i-m_posicioActual]=0;
	m_posicioActual=i;

	for (i=0;i<(int)strlen(tmp);i++)
		tmp[i]=toupper(tmp[i]);

	valortmp=atof(tmp);
	if (valortmp!=0) {
		elToken->posarDades(tkValor, tkIndefinit, tmp, valortmp);
		return;
	}
	for (i=0;i<m_tokens.GetSize();i++) {
		if (m_tokens[i].obtenirCadena()==(CString)tmp) {
			elToken->posarDades(m_tokens[i].obtenirToken(), m_tokens[i].obtenirSubToken(), (CString)tmp);
			return;
		}
	}

	elToken->posarDades(tkId, tkIndefinit, tmp);
}

void CScrIdentificacio::inicialitzar(void *sistema)
{

	//
	// Inicialitzacio de totes les variables d'estat i dels
	// objectes, criteris i expressions temporals.
	//
	m_errorParsejant=false;
	m_objecteActual=NULL;
	m_criteriActual=NULL;
	m_elementExpressioActual=NULL;
	m_textError=_T("");
	//
	// Es buida la llista d'objectes
	//
	borrarObjectes();

	//
	// Es realitza l'associaci entre les paraules clau i els 
	// tokens corresponents.
	//
	m_tokens.Add(CAuxToken("{", tkObrirClau));
	m_tokens.Add(CAuxToken("}", tkTancarClau));
	m_tokens.Add(CAuxToken(";", tkPuntiComa));
	m_tokens.Add(CAuxToken("AGAFADA", tkAgafada));
	m_tokens.Add(CAuxToken("CRITERIS", tkCriteris));
	m_tokens.Add(CAuxToken("DX", tkDespRot, tkDX));
	m_tokens.Add(CAuxToken("DY", tkDespRot, tkDY));
	m_tokens.Add(CAuxToken("R", tkDespRot, tkR));
	m_tokens.Add(CAuxToken("AREA", tkAtribut, tkArea));
	m_tokens.Add(CAuxToken("PERIMETRE", tkAtribut, tkPerimetre));
	m_tokens.Add(CAuxToken("DMIN", tkAtribut, tkDMin));
	m_tokens.Add(CAuxToken("DMAX", tkAtribut, tkDMax));
	m_tokens.Add(CAuxToken("COMPACITAT", tkAtribut, tkCompacitat));
	m_tokens.Add(CAuxToken(">", tkComparador, tkMajor));
	m_tokens.Add(CAuxToken("<", tkComparador, tkMenor));
	m_tokens.Add(CAuxToken("==", tkComparador, tkIgual));
	m_tokens.Add(CAuxToken("!=", tkComparador, tkDistint));
	m_tokens.Add(CAuxToken(">=", tkComparador, tkMajorIgual));
	m_tokens.Add(CAuxToken("<=", tkComparador, tkMenorIgual));
	m_tokens.Add(CAuxToken("+", tkOperacio, tkSuma));
	m_tokens.Add(CAuxToken("-", tkOperacio, tkResta));
	m_tokens.Add(CAuxToken("*", tkOperacio, tkProducte));
	m_tokens.Add(CAuxToken("/", tkOperacio, tkDivisio));
}


void CScrIdentificacio::parseLLISTAOBJECTES()
{
	//
	//  LLISTAOBJECTES -> tkNomObjecte tkObrirClau OBJECTE tkTancarClau LLISTAOBJECTES
    //                 |  tkFinal
	//

	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkId) {
		//
		// Posam el nom en el objecte temporal i l'inicialitzam
		//
		m_objecteActual=new CAuxObjecte();
		m_objecteActual->posarNom(m_tokenActual.obtenirCadena());
		m_objecteActual->posarDespXAgafada(0);
		m_objecteActual->posarDespYAgafada(0);
		m_objecteActual->posarRotacioAgafada(0);
		//
		//
		//
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()==tkObrirClau) {
			parseOBJECTE();
			if (m_errorParsejant) return;
			obtenirToken(&m_tokenActual);
			if (m_tokenActual.obtenirToken()==tkTancarClau) {
				//
				// Guardam l'objecte temporal en la llista d'objectes
				//
				m_objectes.Add(m_objecteActual);
				//
				//
				//
				parseLLISTAOBJECTES();
				if (m_errorParsejant) return;
			}
			else {
				posarErrorParsing(IDE_ERROR_OBRIRCLAU);
				return;
			}
		}
		else {
			posarErrorParsing (IDE_ERROR_OBRIRCLAU);
			return;
		}
	}
	else if (m_tokenActual.obtenirToken()==tkFinal) {
		return;
	}
	else {
		posarErrorParsing(IDE_ERROR_NOUOBJECTE_O_FIARXIU);
		return;
	}
}

void CScrIdentificacio::parseOBJECTE()
{
	//
    // OBJECTE -> tkAgafada tkObrirClau AGAFADA tkCriteris tkObrirClau CRITERIS
    //         |  tkCriteris tkObrirClau CRITERIS
	//

	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkAgafada) {
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()==tkObrirClau) {
			parseAGAFADA();
			if (m_errorParsejant) return;
			obtenirToken(&m_tokenActual);
			if (m_tokenActual.obtenirToken()==tkCriteris) {
				obtenirToken(&m_tokenActual);
				if (m_tokenActual.obtenirToken()==tkObrirClau) {
					parseCRITERIS();
					if (m_errorParsejant) return;
				} else {
					posarErrorParsing(IDE_ERROR_OBRIRCLAU);
					return;
				}

			} else {
				posarErrorParsing(IDE_ERROR_CRITERIS);
				return;
			}
		}
		else {
			posarErrorParsing(IDE_ERROR_OBRIRCLAU);
			return;
		}
	}
	else if (m_tokenActual.obtenirToken()==tkCriteris) {
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()==tkObrirClau) {
			parseCRITERIS();
			if (m_errorParsejant) return;
		} else {
			posarErrorParsing(IDE_ERROR_OBRIRCLAU);
			return;
		}
	} else {
		posarErrorParsing(IDE_ERROR_CRITERIS_O_AGAFADA);
		return;
	}
}

void CScrIdentificacio::parseAGAFADA()
{
	//
	// AGAFADA -> tkDespRot tkValor tkPuntiComa AGAFADA
    //         | tkTancarClau
	//

	enum TToken tmp;

	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkDespRot) {
		// Es guarda el subtoken actual per a emprar-lo ms
		// endavant
		tmp=m_tokenActual.obtenirSubToken();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()==tkValor) {
			// Ara s quan s'empra el subtoken guardat 
			// just abans.
			switch (tmp) {
			case tkDX : m_objecteActual->posarDespXAgafada(m_tokenActual.obtenirValor()); break;
			case tkDY : m_objecteActual->posarDespYAgafada(m_tokenActual.obtenirValor()); break;
			case tkR  : m_objecteActual->posarRotacioAgafada(m_tokenActual.obtenirValor()); break;
			}
			obtenirToken(&m_tokenActual);
			if (m_tokenActual.obtenirToken()==tkPuntiComa) {
				parseAGAFADA();
				if (m_errorParsejant) return;
			} else {
				posarErrorParsing(IDE_ERROR_PUNTICOMA);
				return;
			}
		} else {
			posarErrorParsing(IDE_ERROR_VALORNUMERIC);
			return;
		}
	} 
	else if (m_tokenActual.obtenirToken()==tkTancarClau) {
	} else {
		posarErrorParsing(IDE_ERROR_DESPROT_O_TANCARCLAU);
		return;
	}
}

void CScrIdentificacio::parseCRITERIS()
{
	//
	// CRITERIS -> tkAtribut tkComparador EXPRESSIO CRITERIS
	//          |  tkTancarClau
	//

	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkAtribut) {
		//
		// Posam el atribut pertinent i es crea el criteri actual
		//
		m_criteriActual=new CAuxCriteri();
		switch (m_tokenActual.obtenirSubToken()) {
		case tkArea : m_criteriActual->posarAtribut(atArea); break;
		case tkPerimetre : m_criteriActual->posarAtribut(atPerimetre); break;
		case tkDMin : m_criteriActual->posarAtribut(atDMin); break;
		case tkDMax : m_criteriActual->posarAtribut(atDMax); break;
		case tkCompacitat : m_criteriActual->posarAtribut(atCompacitat); break;
		}
		//
		//
		//
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()==tkComparador) {
			//
			// Posam el comparador pertinent
			//
			switch (m_tokenActual.obtenirSubToken()) {
			case tkMajor : m_criteriActual->posarComparacio(cmMajor); break;
			case tkMenor : m_criteriActual->posarComparacio(cmMenor); break;
			case tkIgual : m_criteriActual->posarComparacio(cmIgual); break;
			case tkDistint : m_criteriActual->posarComparacio(cmDistint); break;
			case tkMajorIgual: m_criteriActual->posarComparacio(cmMajorIgual); break;
			case tkMenorIgual: m_criteriActual->posarComparacio(cmMenorIgual); break;
			}
			//
			//
			//
			parseEXPRESSIO();
			if (m_errorParsejant) return;
			parseCRITERIS();
			if (m_errorParsejant) return;
		} else {
			posarErrorParsing(IDE_ERROR_COMPARADOR);
			return;
		}
	}
	else if (m_tokenActual.obtenirToken()==tkTancarClau) {
	} else {
		posarErrorParsing(IDE_ERROR_ATRIBUT_O_TANCARCLAU);
		return;
	}
}

void CScrIdentificacio::parseEXPRESSIO()
{
	//
	// EXPRESSIO -> tkAtribut EXPRESSIO2
	//           | tkValor EXPRESSIO2
	//

	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkAtribut) {
		//
		// Guardam l'atribut en questi dins de l'element actual
		//
		m_elementExpressioActual=new CAuxElementExpressio();
		m_elementExpressioActual->valor(false);
		switch (m_tokenActual.obtenirSubToken()) {
		case tkArea : m_elementExpressioActual->posarAtribut(atArea); break;
		case tkPerimetre : m_elementExpressioActual->posarAtribut(atPerimetre); break;
		case tkDMin : m_elementExpressioActual->posarAtribut(atDMin); break;
		case tkDMax : m_elementExpressioActual->posarAtribut(atDMax); break;
		case tkCompacitat : m_elementExpressioActual->posarAtribut(atCompacitat); break;			
		}
		m_elementExpressioActual->posarOperacio(opInici);
		m_criteriActual->obtenirExpressio()->Add(m_elementExpressioActual);
		m_elementExpressioActual=new CAuxElementExpressio();
		//
		//
		//
		parseEXPRESSIO2();
		if (m_errorParsejant) return;
	}
	else if (m_tokenActual.obtenirToken()==tkValor) {
		//
		// Guardam el valor en questi dins de l'element actual
		//
		m_elementExpressioActual=new CAuxElementExpressio();
		m_elementExpressioActual->valor(true);
		m_elementExpressioActual->posarValor(m_tokenActual.obtenirValor());
		m_elementExpressioActual->posarOperacio(opInici);
		m_criteriActual->obtenirExpressio()->Add(m_elementExpressioActual);
		m_elementExpressioActual=new CAuxElementExpressio();
		//
		//
		//
		parseEXPRESSIO2();
		if (m_errorParsejant) return;
	} else {
		posarErrorParsing(IDE_ERROR_VALORNUMERIC);
		return;
	}
}

void CScrIdentificacio::parseEXPRESSIO2()
{
	//
	// EXPRESSIO2 -> tkOperacio VALOR EXPRESSIO2
	//            | tkPuntiComa
	//

	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkOperacio) {
		//
		// Posam les dades per al nou element de l'expressi
		//
		switch (m_tokenActual.obtenirSubToken()) {
		case tkSuma : m_elementExpressioActual->posarOperacio(opSuma); break;
		case tkResta : m_elementExpressioActual->posarOperacio(opResta); break;
		case tkProducte : m_elementExpressioActual->posarOperacio(opProducte); break;
		case tkDivisio : m_elementExpressioActual->posarOperacio(opDivisio); break;
		}
		//
		//
		//
		parseVALOR();
		if (m_errorParsejant) return;
		parseEXPRESSIO2();
		if (m_errorParsejant) return;
	}
	else if (m_tokenActual.obtenirToken()==tkPuntiComa) {
		//
		// S'afegeix el criteri actual a l'objecte actual
		//
		m_objecteActual->obtenirCriteris()->Add(m_criteriActual);
		//
		//
		//
	} else {
		posarErrorParsing(IDE_ERROR_OPERACIO_O_PUNTICOMA);
		return;
	}
}

void CScrIdentificacio::parseVALOR()
{
	//
	// VALOR -> tkAtribut
    //       | tkValor
	//
	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkAtribut) {
		//
		// Posam l'atribut a l'element actual i l'afegim a la llista
		//
		m_elementExpressioActual->valor(false);
		switch (m_tokenActual.obtenirSubToken()) {
		case tkArea : m_elementExpressioActual->posarAtribut(atArea); break;
		case tkPerimetre : m_elementExpressioActual->posarAtribut(atPerimetre); break;
		case tkDMin : m_elementExpressioActual->posarAtribut(atDMin); break;
		case tkDMax : m_elementExpressioActual->posarAtribut(atDMax); break;
		case tkCompacitat : m_elementExpressioActual->posarAtribut(atCompacitat); break;			
		}
		m_criteriActual->obtenirExpressio()->Add(m_elementExpressioActual);
		m_elementExpressioActual=new CAuxElementExpressio();
		//
		//
		//
	}
	else if (m_tokenActual.obtenirToken()==tkValor) {
		//
		// Guardam el valor en questi dins de l'element actual
		//
		m_elementExpressioActual->valor(true);
		m_elementExpressioActual->posarValor(m_tokenActual.obtenirValor());
		m_criteriActual->obtenirExpressio()->Add(m_elementExpressioActual);
		m_elementExpressioActual=new CAuxElementExpressio();
		//
		//
		//
	} else {
		posarErrorParsing(IDE_ERROR_ATRIBUT_O_VALORNUMERIC);
		return;
	}
}

void CScrIdentificacio::posarErrorParsing(CString elError)
{
	m_errorParsejant=true;
	m_textError=elError;
	m_liniaError=m_numLiniaActual;
	m_columnaError=m_posicioActual-m_tokenActual.obtenirCadena().GetLength() + 1;
}

void CScrIdentificacio::borrarObjectes()
{
	int i;
	for (i=0;i<m_objectes.GetSize();i++) 
		if (m_objectes[i]!=NULL) {
			delete m_objectes[i];
		}
	m_objectes.RemoveAll();
}

void CScrIdentificacio::guardarObjectes(CString nomArxiu)
{
	int i, j, k;
	CLlistaCriteris *c;
	CLlistaElementsExpressio *l;
	char tmp[200], tmp2[50], tmp3[50];

	CString obrirClau=obtenirCadena(tkObrirClau);
	CString tancarClau=obtenirCadena(tkTancarClau);
	CString agafada=obtenirCadena(tkAgafada);
	CString criteris=obtenirCadena(tkCriteris);
	CString despX=obtenirCadena(tkDX);
	CString despY=obtenirCadena(tkDY);
	CString rotacio=obtenirCadena(tkR);
	CString area=obtenirCadena(tkArea);
	CString perimetre=obtenirCadena(tkPerimetre);
	CString dmin=obtenirCadena(tkDMin);
	CString dmax=obtenirCadena(tkDMax);
	CString compacitat=obtenirCadena(tkCompacitat);
	CString suma=obtenirCadena(tkSuma);
	CString resta=obtenirCadena(tkResta);
	CString producte=obtenirCadena(tkProducte);
	CString divisio=obtenirCadena(tkDivisio);
	CString igual=obtenirCadena(tkIgual);
	CString distint=obtenirCadena(tkDistint);
	CString major=obtenirCadena(tkMajor);
	CString menor=obtenirCadena(tkMenor);
	CString majorIgual=obtenirCadena(tkMajorIgual);
	CString menorIgual=obtenirCadena(tkMenorIgual);
	CString puntiComa=obtenirCadena(tkPuntiComa);
	
	m_nomArxiu=nomArxiu;
	m_fitxer = new CFile(nomArxiu, CFile::modeWrite | CFile::modeCreate);
	m_arxiu = new CArchive(m_fitxer, CArchive::store);

	for (i=0;i<m_objectes.GetSize();i++) {
		sprintf(tmp, "%s %s\n", m_objectes[i]->obtenirNom(), obrirClau);
		(*m_arxiu).WriteString(tmp);
		if (m_objectes[i]->obtenirDespXAgafada()!=0 || m_objectes[i]->obtenirDespYAgafada()!=0 || m_objectes[i]->obtenirRotacioAgafada()!=0) {
			sprintf(tmp, "\t%s %s\n", agafada, obrirClau);
			(*m_arxiu).WriteString(tmp);
			if (m_objectes[i]->obtenirDespXAgafada()!=0) {
				sprintf (tmp, "\t\t%s %f %s\n", despX, m_objectes[i]->obtenirDespXAgafada(), puntiComa);
				(*m_arxiu).WriteString(tmp);
			}
			if (m_objectes[i]->obtenirDespYAgafada()!=0) {
				sprintf (tmp, "\t\t%s %f %s\n", despY, m_objectes[i]->obtenirDespYAgafada(), puntiComa);
				(*m_arxiu).WriteString(tmp);
			}
			if (m_objectes[i]->obtenirRotacioAgafada()!=0) {
				sprintf (tmp, "\t\t%s %f %s\n", rotacio, m_objectes[i]->obtenirRotacioAgafada(), puntiComa);
				(*m_arxiu).WriteString(tmp);
			}
			sprintf(tmp, "\t%s\n", tancarClau);
			(*m_arxiu).WriteString(tmp);
		}
		sprintf(tmp, "\t%s %s \n", criteris, obrirClau);
		(*m_arxiu).WriteString(tmp);
		c=m_objectes[i]->obtenirCriteris();
		for (j=0;j<c->GetSize();j++) {
			switch ((*c)[j]->obtenirAtribut()) {
			case atArea : strcpy(tmp2, area); break;
			case atPerimetre : strcpy(tmp2, perimetre); break;
			case atDMin : strcpy (tmp2, dmin); break;
			case atDMax : strcpy(tmp2, dmax); break;
			case atCompacitat : strcpy(tmp2, compacitat); break;
			}
			switch ((*c)[j]->obtenirComparacio()) {
			case cmMajor : strcpy (tmp3, major); break;
			case cmMenor : strcpy (tmp3, menor); break;
			case cmIgual : strcpy (tmp3, igual); break;
			case cmDistint : strcpy (tmp3, distint); break;
			case cmMajorIgual : strcpy (tmp3, majorIgual); break;
			case cmMenorIgual : strcpy (tmp3, menorIgual); break;
			}
			sprintf (tmp, "\t\t%s %s", tmp2, tmp3);
			(*m_arxiu).WriteString(tmp);
			l=(*c)[j]->obtenirExpressio();
			for (k=0;k<l->GetSize();k++) {
				switch ((*l)[k]->obtenirOperacio()) {
				case opSuma : strcpy (tmp2, suma);break;
				case opResta : strcpy (tmp2, resta); break;
				case opProducte : strcpy (tmp2, producte); break;
				case opDivisio : strcpy (tmp2, divisio); break;
				case opInici : tmp2[0]='\0';
				}
				if ((*l)[k]->esValor()) {
					sprintf (tmp, "%s %f ", tmp2, (*l)[k]->obtenirValor());
				} else {
					switch ((*l)[k]->obtenirAtribut()) {
						case atArea : strcpy(tmp3, area); break;
						case atPerimetre : strcpy(tmp3, perimetre); break;
						case atDMin : strcpy (tmp3, dmin); break;
						case atDMax : strcpy(tmp3, dmax); break;
						case atCompacitat : strcpy(tmp3, compacitat); break;
					}
					sprintf (tmp, "%s %s ", tmp2, tmp3);
				}
				(*m_arxiu).WriteString(tmp);
			}
			sprintf(tmp, " %s\n", puntiComa);
			(*m_arxiu).WriteString(tmp);
		}
		sprintf(tmp, "\t%s \n%s \n", tancarClau, tancarClau);
		(*m_arxiu).WriteString(tmp);
	}

	m_arxiu->Close();
	m_fitxer->Close();
}

CString CScrIdentificacio::obtenirCadena(enum TToken tk)
{
	int i;
	for (i=0;i<m_tokens.GetSize();i++) 
		if (m_tokens[i].obtenirToken()==tk || m_tokens[i].obtenirSubToken()==tk) 
			return m_tokens[i].obtenirCadena();
	return CString("TOKEN_DESCONEGUT");
}

CAuxObjecte * CScrIdentificacio::identificarObjecte(long elArea, int elPerimetre, int laDMin, int laDMax, double laCompacitat)
{
	int i, j ;
	CLlistaCriteris *c;
	CLlistaElementsExpressio *l;
	double valorExpressio;
	bool avaluacioActual;
	
	for (i=0;i<m_objectes.GetSize();i++) {
		c=m_objectes[i]->obtenirCriteris();
		avaluacioActual=true;
		for (j=0;j<c->GetSize();j++) {
			l=(*c)[j]->obtenirExpressio();
			valorExpressio=avaluarExpressio(l, elArea, elPerimetre, laDMin, laDMax, laCompacitat);
			avaluacioActual = avaluarComparacio((*c)[j]->obtenirAtribut(), (*c)[j]->obtenirComparacio(), valorExpressio, elArea, elPerimetre, laDMin, laDMax, laCompacitat) && avaluacioActual;
		}
		if (avaluacioActual) return (m_objectes[i]);
	}
	return NULL;
}

double CScrIdentificacio::avaluarExpressio(CLlistaElementsExpressio * le, long elArea, int elPerimetre, int laDMin, int laDMax, double laCompacitat)
{
	int i;
	double resultat=0.0, actual=0.0;
	for (i=0;i<le->GetSize();i++) {
		if ((*le)[i]->esValor()) 
			actual=(*le)[i]->obtenirValor();
		else {
			switch ((*le)[i]->obtenirAtribut()) {
			case atArea : actual=elArea; break;
			case atPerimetre : actual=elPerimetre; break;
			case atDMin : actual=laDMin; break;
			case atDMax : actual=laDMax; break;
			case atCompacitat : actual=laCompacitat; break;
			default: actual=0.0; break;
			}
		}
		
		switch ((*le)[i]->obtenirOperacio()) {
		case opInici: resultat = actual; break;
		case opSuma: resultat += actual; break;
		case opResta : resultat -= actual; break;
		case opProducte : resultat *= actual; break;
		case opDivisio : if (actual!=0) resultat /= actual; break;
		default : resultat=0; break;
		}
	}
	return resultat;
}

bool CScrIdentificacio::avaluarComparacio(enum TAtribut elAtribut, enum TComparacio elComparador, double elValor, long elArea, int elPerimetre, int laDMin, int laDMax, double laCompacitat)
{
	double valor=0.0;
	switch (elAtribut) {
	case atArea : valor=elArea; break;
	case atPerimetre : valor = elPerimetre; break;
	case atDMin : valor = laDMin; break;
	case atDMax : valor = laDMax; break;
	case atCompacitat : valor = laCompacitat; break;
	default : return false; break;
	}

	switch (elComparador) {
	case cmIgual : return (valor == elValor); break;
	case cmDistint : return (valor != elValor); break;
	case cmMajor : return (valor > elValor); break;
	case cmMenor : return (valor < elValor); break;
	case cmMajorIgual : return (valor >= elValor); break;
	case cmMenorIgual : return (valor <= elValor); break;
	}

	return false;
}
