/////////////////////////////////////////////////////////////////
//
// Arxiu      : ScrInterpret.cpp
//
// Descripci : Aquesta classe proporciona mtodes per a
//				llegir arxius de script de l'entorn BRAS
//				(format bsc) i executar-los, interaccionant
//				amb un objecte de la classe CDspRobot.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

//
// Arxius de capalera
//
#include "stdafx.h"
#include "BRAS.h"
#include "ScrInterpret.h"
#include "Sistema.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

CScrInterpret::CScrInterpret()
{

}

CScrInterpret::~CScrInterpret()
{

}

void CScrInterpret::inicialitzar(void *sistema)
{
	m_primeraExecucioContinua=true;
	m_hihaMoviment=false;
	m_tinicial=0;
	m_tfinal=0;
	m_tempsTranscorregut=0;
	m_robot=&(((CSistema *)sistema)->m_robot);
	m_cella=&(((CSistema *)sistema)->m_cellconfig);
	//
	// Inicialitzacio de totes les variables d'estat
	//
	iniciarEstat();

	//
	// Es realitza l'associaci entre les paraules clau i els 
	// tokens corresponents.
	//
	m_tokens.Add(CAuxToken(";", tkPuntiComa));
	m_tokens.Add(CAuxToken(",", tkComa));
	m_tokens.Add(CAuxToken("SI", tkSi));
	m_tokens.Add(CAuxToken("ANAR", tkGoto));
	m_tokens.Add(CAuxToken("DEFINIRENROBOT", tkDefinirEnRobot));
	m_tokens.Add(CAuxToken("FINAL", tkFinalProg));
	m_tokens.Add(CAuxToken("=", tkAssignacio));
	m_tokens.Add(CAuxToken("TRASLLADAR", tkTraslladar));
	m_tokens.Add(CAuxToken("EXECUTARENROBOT", tkExecutarEnRobot));
	m_tokens.Add(CAuxToken("(", tkObrirParentesi));
	m_tokens.Add(CAuxToken(")", tkTancarParentesi));
	m_tokens.Add(CAuxToken(">", tkComparador, tkMajor));
	m_tokens.Add(CAuxToken("<", tkComparador, tkMenor));
	m_tokens.Add(CAuxToken("==", tkComparador, tkIgual));
	m_tokens.Add(CAuxToken("!=", tkComparador, tkDistint));
	m_tokens.Add(CAuxToken(">=", tkComparador, tkMajorIgual));
	m_tokens.Add(CAuxToken("<=", tkComparador, tkMenorIgual));
	m_tokens.Add(CAuxToken("+", tkOperacio, tkSuma));
	m_tokens.Add(CAuxToken("-", tkOperacio, tkResta));
	m_tokens.Add(CAuxToken("*", tkOperacio, tkProducte));
	m_tokens.Add(CAuxToken("/", tkOperacio, tkDivisio));
	m_tokens.Add(CAuxToken("AREA", tkAtribut, tkArea));
	m_tokens.Add(CAuxToken("PERIMETRE", tkAtribut, tkPerimetre));
	m_tokens.Add(CAuxToken("DMIN", tkAtribut, tkDMin));
	m_tokens.Add(CAuxToken("DMAX", tkAtribut, tkDMax));
	m_tokens.Add(CAuxToken("COMPACITAT", tkAtribut, tkCompacitat));
	m_tokens.Add(CAuxToken("ESTAT", tkEstat));
	m_tokens.Add(CAuxToken("ANARSUBRUTINA", tkGosub));
	m_tokens.Add(CAuxToken("TORNAR", tkReturn));
	m_tokens.Add(CAuxToken("OBRIR", tkObrir));
	m_tokens.Add(CAuxToken("TANCAR", tkTancar));
	m_tokens.Add(CAuxToken("MOURE", tkMoure));
	m_tokens.Add(CAuxToken("MISSATGE", tkMissatge));
	m_tokens.Add(CAuxToken("MOSTRARTEMPS", tkMostrarTemps));
	m_tokens.Add(CAuxToken("ACTIVARCINTA", tkActivarCinta));
	m_tokens.Add(CAuxToken("DEFINIRFIXEENROBOT", tkDefinirFixeEnRobot));
	m_tokens.Add(CAuxToken("DESACTIVARCINTA", tkDesactivarCinta));
}

void CScrInterpret::parsePUNT()
{
	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkId) {
		// Posar en m_puntActual la informaci del punt
		// simbolitzat per tkId (si est definit) o 
		// retornar error si no ho est.
		if (!m_generantEtiquetes) {
			if (!existeixPunt(m_tokenActual.obtenirCadena(), &m_puntActual)) {
				posarErrorParsing(INT_ERROR_PUNT_NODEFINIT);
				return;
			}
		}
		return;
	}
	if (m_tokenActual.obtenirToken()==tkObrirParentesi) {
		// Posar en m_puntActual la informaci del punt
		// que s'introdueix.
		obtenirToken(&m_tokenActual);
		m_puntActual.assignarCadena("TEMP");
		// X
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALORXVALID);
			return;
		}
		m_puntActual.assignarX((int)m_tokenActual.obtenirValor());
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkComa) {
			posarErrorParsing(INT_ERROR_ESPERA_COMA);
			return;
		}
		obtenirToken(&m_tokenActual);
		// Y
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALORYVALID);
			return;
		}
		m_puntActual.assignarY((int)m_tokenActual.obtenirValor());
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkComa) {
			posarErrorParsing(INT_ERROR_ESPERA_COMA);
			return;
		}
		obtenirToken(&m_tokenActual);
		// Z
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALORZVALID);
			return;
		}
		m_puntActual.assignarZ((int)m_tokenActual.obtenirValor());
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkComa) {
			posarErrorParsing(INT_ERROR_ESPERA_COMA);
			return;
		}
		// P
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALORPVALID);
			return;
		}
		m_puntActual.assignarPitch((int)m_tokenActual.obtenirValor());
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkComa) {
			posarErrorParsing(INT_ERROR_ESPERA_COMA);
			return;
		}
		// R
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALORRVALID);
			return;
		}
		m_puntActual.assignarRoll((int)m_tokenActual.obtenirValor());
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkTancarParentesi) {
			posarErrorParsing(INT_ERROR_ESPERA_TANCAPARENTESI);
			return;
		}
		return;
	}
	posarErrorParsing(INT_ERROR_ESPERA_PUNT);
	return;
}

void CScrInterpret::obtenirToken(CAuxToken * elToken)
{
	char tmp[30];
	double valortmp;
	int i;
	bool esValor;

novalinia:
	while ((m_posicioActual>=m_liniaActual.GetLength()-1)||(m_liniaActual.IsEmpty())||((m_liniaActual[m_posicioActual]=='/')&&(m_liniaActual[m_posicioActual+1]=='/'))) {
		m_posicioActual=0;
		m_numLiniaActual++;
		if (m_numLiniaActual>=m_script.GetSize()) {
			elToken->posarDades(tkFinal);
			return;
		}
		m_liniaActual=m_script[m_numLiniaActual];
	}
	// Botam els espais.
	for (i=m_posicioActual;i<m_liniaActual.GetLength() && (m_liniaActual[i]==' ' || m_liniaActual[i]=='\n' || m_liniaActual[i]=='\f' || m_liniaActual[i]=='\r'|| m_liniaActual[i]=='\t');i++);
	m_posicioActual=i;

	// Llegim la cadena d'entrada
	for (i=m_posicioActual;i<m_liniaActual.GetLength() && m_liniaActual[i]!=' ' && m_liniaActual[i]!='\n' && m_liniaActual[i]!='\f' && m_liniaActual[i]!='\r' && m_liniaActual[i]!='\t';i++) 
		tmp[i-m_posicioActual]=m_liniaActual[i];
	tmp[i-m_posicioActual]=0;
	m_posicioActual=i;

	esValor=true;
	for (i=0;i<(int)strlen(tmp);i++) {
		if (!esCaracterNumeric(tmp[i]))
			esValor=false;
		tmp[i]=toupper(tmp[i]);
	}
	
	if (esValor) {
		valortmp=atof(tmp);
		elToken->posarDades(tkValor, tkIndefinit, tmp, valortmp);
		return;
	}
	if (tmp[0]=='@') {
		elToken->posarDades(tkEtiqueta, tkIndefinit, tmp);
		return;
	}
	if (tmp[0]=='#') {
		elToken->posarDades(tkObjecte, tkIndefinit, tmp);
		return;
	}
	if (tmp[0]=='$') {
		elToken->posarDades(tkPuntRobot, tkIndefinit, ((CString)tmp).Right(((CString)tmp).GetLength()-1));
		return;
	}
	if (strlen(tmp)==0) goto novalinia;
	for (i=0;i<m_tokens.GetSize();i++) {
		if (m_tokens[i].obtenirCadena()==(CString)tmp) {
			elToken->posarDades(m_tokens[i].obtenirToken(), m_tokens[i].obtenirSubToken(), (CString)tmp);
			return;
		}
	}

	elToken->posarDades(tkId, tkIndefinit, tmp);
}

void CScrInterpret::posarErrorParsing(CString error)
{
	m_textError=error;
	m_errorParsejant=true;
	m_liniaError=m_numLiniaActual+1;
	m_posicioError=m_posicioActual - m_tokenActual.obtenirCadena().GetLength();
}

void CScrInterpret::parseEXPRESSIO2()
{
	TToken op;
	double vtmp;
	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkOperacio) {
		op=m_tokenActual.obtenirSubToken();
		vtmp=parseVALOR();
		if (m_errorParsejant) return;
		switch (op) {
		case tkSuma :
			m_valorActual += vtmp;
			break;
		case tkResta :
			m_valorActual -= vtmp;
			break;
		case tkProducte :
			m_valorActual *= vtmp;
			break;
		case tkDivisio :
			m_valorActual /= vtmp;
			break;
		default:
			posarErrorParsing(INT_ERROR_OPERACIO_DESCONEGUDA);
			return;
			break;
		}
		parseEXPRESSIO2();
		if (m_errorParsejant) return;
		return;
	}
	if (m_tokenActual.obtenirToken()!=tkTancarParentesi) {
		posarErrorParsing(INT_ERROR_ESPERA_TANCAPARENTESI);
		return;
	}
}

double CScrInterpret::parseVALOR()
{
	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()==tkValor) {
		return (m_tokenActual.obtenirValor());
	}
	if (m_tokenActual.obtenirToken()==tkAtribut) {
		switch (m_tokenActual.obtenirSubToken()) {
		case tkArea:
			return m_area;
			break;
		case tkPerimetre:
			return m_perimetre;
			break;
		case tkDMin:
			return m_dMin;
			break;
		case tkDMax:
			return m_dMax;
			break;
		case tkCompacitat:
			return m_compacitat;
			break;
		default:
			posarErrorParsing(INT_ERROR_ESPERA_VALOR);
			return 0.0;
			break;
		}
	}
	posarErrorParsing(INT_ERROR_ESPERA_VALOR);
	return 0.0;
}

void CScrInterpret::parseEXPRESSIO()
{
	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()!=tkObrirParentesi) {
		posarErrorParsing(INT_ERROR_ESPERA_OBREPARENTESI);
		return;
	}
	m_valorActual=parseVALOR();
	if (m_errorParsejant) return;
	parseEXPRESSIO2();
	if (m_errorParsejant) return;
}

void CScrInterpret::parseCONDICIO()
{
	double v1, v2;
	TToken tcmp;
	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()!=tkObrirParentesi) {
		posarErrorParsing(INT_ERROR_ESPERA_OBREPARENTESI);
		return;
	}
	obtenirToken(&m_tokenActual);
	switch (m_tokenActual.obtenirToken()) {
	case tkObjecte:
		m_condicioActual=(m_tokenActual.obtenirCadena().Right(m_tokenActual.obtenirCadena().GetLength()-1)==m_nomObjecte);
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkTancarParentesi) {
			posarErrorParsing(INT_ERROR_ESPERA_TANCAPARENTESI);
			return;
		}
		return;
		break;
	case tkEstat:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkComparador) {
			posarErrorParsing(INT_ERROR_ESPERA_COMPARADOR);
			return;
		}
		tcmp=m_tokenActual.obtenirSubToken();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALOR);
			return;
		}
		switch(tcmp) {
		case tkIgual :
			m_condicioActual=(m_estat==(int)m_tokenActual.obtenirValor());
			break;
		case tkDistint:
			m_condicioActual=(m_estat!=(int)m_tokenActual.obtenirValor());
			break;
		case tkMajor :
			m_condicioActual=(m_estat>(int)m_tokenActual.obtenirValor());
			break;
		case tkMenor :
			m_condicioActual=(m_estat<(int)m_tokenActual.obtenirValor());
			break;
		case tkMajorIgual :
			m_condicioActual=(m_estat>=(int)m_tokenActual.obtenirValor());
			break;
		case tkMenorIgual :
			m_condicioActual=(m_estat<=(int)m_tokenActual.obtenirValor());
			break;
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkTancarParentesi) {
			posarErrorParsing(INT_ERROR_ESPERA_TANCAPARENTESI);
			return;
		}
		return;
		break;
	case tkValor :
		m_valorActual=m_tokenActual.obtenirValor();
		parseEXPRESSIO2();
		if (m_errorParsejant) return;
		break;
	case tkAtribut :
		switch (m_tokenActual.obtenirSubToken()) {
		case tkArea:
			m_valorActual=m_area;
			break;
		case tkPerimetre:
			m_valorActual=m_perimetre;
			break;
		case tkDMin:
			m_valorActual=m_dMin;
			break;
		case tkDMax:
			m_valorActual=m_dMax;
			break;
		case tkCompacitat:
			m_valorActual=m_compacitat;
			break;
		default:
			posarErrorParsing(INT_ERROR_ESPERA_VALOR);
			return;
			break;
		}
		parseEXPRESSIO2();
		if (m_errorParsejant) return;
		v1=m_valorActual;
		break;
	default :
		posarErrorParsing(INT_ERROR_CONDICIO_NOVALIDA);
		return;
	}
	obtenirToken(&m_tokenActual);
	if (m_tokenActual.obtenirToken()!=tkComparador) {
		posarErrorParsing(INT_ERROR_ESPERA_COMPARADOR);
		return ;
	}
	tcmp=m_tokenActual.obtenirSubToken();
	parseEXPRESSIO();
	if (m_errorParsejant) return;
	v2=m_tokenActual.obtenirValor();
	switch (tcmp) {
		case tkMajor :
			m_condicioActual=v1>v2;
			break;
		case tkMenor :
			m_condicioActual=v1<v2;
			break;
		case tkIgual :
			m_condicioActual=v1==v2;
			break;
		case tkDistint :
			m_condicioActual=v1!=v2;
			break;
		case tkMajorIgual :
			m_condicioActual=v1>=v2;
			break;
		case tkMenorIgual :
			m_condicioActual=v1<=v2;
			break;
		default :
			break;
	}
}

void CScrInterpret::parseLLISTACOMANDES()
{
	int liniaSalt;
	CAuxPuntRobot ptmp;
	CString p1;
	char eix, missatge[100];
	double vtmp;
	float ttotal;
	int nouX, nouY, nouZ, nouPitch, nouRoll;

	obtenirToken(&m_tokenActual);
	switch (m_tokenActual.obtenirToken()) {
	//
	// Activaci de la cinta
	// =====================
	//
	// Envia l'ordre d'activaci de la cinta
	//
	case tkActivarCinta:
		if (!m_generantEtiquetes) 
			m_robot->activarCinta();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		break;
	//
	// Desactivaci de la cinta
	// =====================
	//
	// Envia l'ordre d'activaci de la cinta
	//
	case tkDesactivarCinta:
		if (!m_generantEtiquetes) 
			m_robot->desactivarCinta();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		break;
		
	//
	// Visualitzaci del temps transcorregut
	// =====================================
	//
	// El temps transcorregut s el temps que duim executant
	// script i el que prviament s'ha consumit amb l'anlisi.
	// El primer ha estat rebut com a parmetre, mentres que 
	// el segon es calcula a partir de la mesura inicial de temps
	// que es fa en iniciar l'script.
	//
	case tkMostrarTemps :
		if (!m_generantEtiquetes) {
			m_tfinal=(float)clock();
			sprintf(missatge, "Temps anlisi: %f\nTemps execuci script: %f\n", m_tempsTranscorregut, (m_tfinal-m_tinicial)/(CLOCKS_PER_SEC));
			AfxMessageBox(missatge);
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		break;
	//
	// Posar un missatge en pantalla
	// =============================
	//
	// Llegeix la cadena del missatge (que no pot contenir
	// espais) i la visualitza en pantalla.
	//
	case tkMissatge :
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkId) {
			posarErrorParsing(INT_ERROR_ESPERA_IDENTIFICADOR);
			return;
		}
		if (!m_generantEtiquetes) {
			AfxMessageBox(m_tokenActual.obtenirCadena());
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		break;
	//
	// Obrir la pina
	// ==============
	//
	// Envia cap al robot l'ordre d'obertura de la pina
	//
	case tkObrir:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			m_robot->obrir();
		}
		break;
	//
	// Tancar la pina
	// ===============
	//
	// Envia cap al robot l'ordre de tancament de la pina
	//
	case tkTancar:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			m_robot->tancar();
		}
		break;
	//
	// Anar a una subrutina
	// ====================
	//
	// Guarda en la pila la lnia posterior a l'actual, que s
	// aquella en que s'ha de continuar l'execuci en sortir de 
	// la subrutina. A continuaci realitza el salt.
	//
  	case tkGosub:
		if (!m_generantEtiquetes)
			m_pila.Add(m_numLiniaActual+1);
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkEtiqueta) {
			posarErrorParsing(INT_ERROR_GOTOETIQUETA);
			return;
		}
		if (!m_generantEtiquetes) {
			if (!etiquetaALinia(m_tokenActual.obtenirCadena(), &liniaSalt)) {
				posarErrorParsing(INT_ERROR_ETIQUETA_NODEFINIDA);
				return;
			}
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			saltar(liniaSalt);
		}
		break;
	//
	// Tornar de la subrutina
	// ======================
	//
	// Salta a la linia indicada al cim de la pila, i fa un pop
	// de la pila. No comprova si la pila s buida ni de quin
	// Gosub retorna.
	//
	case tkReturn :
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			if (m_pila.GetSize()==0) {
				posarErrorParsing(INT_ERROR_RETURN_WITHOUT_GOSUB);
				return;
			}
			liniaSalt=m_pila[m_pila.GetSize()-1];
			m_pila.RemoveAt(m_pila.GetSize()-1);
			saltar(liniaSalt);
		}
		break ;
	//
	// Assignaci d'estat
	// ==================
	//
	// Llegeix el valor numric que segueix a la paraula clau
	// estat, i l'assigna a la variable d'estat.
	//
	case tkEstat:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkAssignacio) {
			posarErrorParsing(INT_ERROR_ESPERA_ASSIGNACIO);
			return;
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALOR);
			return;
		}
		m_estat=(int)(m_tokenActual.obtenirValor());
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		break;
	//
	// Si CONDICIO anar ETIQUETA
	// =========================
	//
	// Avalua la condicio. Si s vlida, salta a ETIQUETA.
	// Si no ho s, continua a la linia posterior.
	//
	case tkSi:
		parseCONDICIO();
		if (m_errorParsejant) return;
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkGoto) {
			posarErrorParsing(INT_ERROR_ESPERA_IFGOTO);
			return;
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkEtiqueta) {
			posarErrorParsing(INT_ERROR_GOTOETIQUETA);
			return;
		}
		if (!m_generantEtiquetes) {
			if (!etiquetaALinia(m_tokenActual.obtenirCadena(), &liniaSalt)) {
				posarErrorParsing(INT_ERROR_ETIQUETA_NODEFINIDA);
				return;
			}
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes && m_condicioActual) {
			saltar(liniaSalt);
		}
		break;
	//
	// Saltar
	// ======
	//
	// Fa que la segent lnia que s'executi sigui la indicada
	// per l'etiqueta `que segueix al token tkGoto.
	//
	case tkGoto:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkEtiqueta) {
			posarErrorParsing(INT_ERROR_GOTOETIQUETA);
			return;
		}
		if (!m_generantEtiquetes) {
			if (!etiquetaALinia(m_tokenActual.obtenirCadena(), &liniaSalt)) {
				posarErrorParsing(INT_ERROR_ETIQUETA_NODEFINIDA);
				return;
			}
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			saltar(liniaSalt);
		}
		break;

	//
	// Definir Fixe en robot
	// =====================
	//
	// Defineix en el robot el punt indicat de l'mbit de l'script
	// amb el nmero indicat en l'mbit del robot. El punt no
	// es desplaa en cas que la cinta ho faci. S'empra aquesta
	// comanda per a definir punts fixos, independentment del
	// moviment de la cinta.
	//
	case tkDefinirFixeEnRobot:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkId) {
			posarErrorParsing(INT_ERROR_ESPERA_IDENTIFICADOR);
			return;
		}
		if (!m_generantEtiquetes) {
			if (!existeixPunt(m_tokenActual.obtenirCadena(), &ptmp)) {
				posarErrorParsing(INT_ERROR_PUNT_NODEFINIT);
				return;
			}
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntRobot) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTROBOT);
			return;
		}
		p1=m_tokenActual.obtenirCadena();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			m_robot->definirPunt(atoi(p1), ptmp.obtenirX(), ptmp.obtenirY(), ptmp.obtenirZ(), ptmp.obtenirPitch(), ptmp.obtenirRoll());
		}

		break;
	//
	// Definir en robot
	// ================
	//
	// Defineix en el robot el punt indicat de l'mbit de l'script
	// amb el nmero indicat en l'mbit del robot.
	//
	case tkDefinirEnRobot:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkId) {
			posarErrorParsing(INT_ERROR_ESPERA_IDENTIFICADOR);
			return;
		}
		if (!m_generantEtiquetes) {
			if (!existeixPunt(m_tokenActual.obtenirCadena(), &ptmp)) {
				posarErrorParsing(INT_ERROR_PUNT_NODEFINIT);
				return;
			}
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntRobot) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTROBOT);
			return;
		}
		p1=m_tokenActual.obtenirCadena();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			if (m_cella->m_aturarCinta)
				m_robot->definirPunt(atoi(p1), ptmp.obtenirX(), ptmp.obtenirY(), ptmp.obtenirZ(), ptmp.obtenirPitch(), ptmp.obtenirRoll());
			else {
				if (!m_hihaMoviment) {
					m_robot->definirPunt(atoi(p1), ptmp.obtenirX(), ptmp.obtenirY(), ptmp.obtenirZ(), ptmp.obtenirPitch(), ptmp.obtenirRoll());
				} 
				else {
					m_tfinal=(float)clock();
					ttotal=(float)(((m_tfinal-m_tinicial)/CLOCKS_PER_SEC)+m_tempsTranscorregut+m_cella->m_tempsAddicional);
					nouX=ptmp.obtenirX()+(int)((m_cella->m_vx)*ttotal);
					nouY=ptmp.obtenirY()+(int)((m_cella->m_vy)*ttotal);
					nouZ=ptmp.obtenirZ();
					nouPitch=ptmp.obtenirPitch();
					nouRoll=(int)(ptmp.obtenirRoll()+(atan2(ptmp.obtenirY(), ptmp.obtenirX())-atan2(nouY, nouX))*(3600/(2*3.14159)));
					m_robot->definirPunt(atoi(p1), nouX, nouY, nouZ, nouPitch, nouRoll);
				}
			}
		}
		break;
	//
	// Moure
	// =====
	//
	// Indica al robot que faci el moviment
	//
	case tkMoure:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkId) {
			posarErrorParsing(INT_ERROR_ESPERA_IDENTIFICADOR);
			return;
		}
		if (!m_generantEtiquetes) {
			if (!existeixPunt(m_tokenActual.obtenirCadena(), &ptmp)) {
				posarErrorParsing(INT_ERROR_PUNT_NODEFINIT);
				return;
			}
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			m_robot->moure(ptmp.obtenirX(), ptmp.obtenirY(), ptmp.obtenirZ(), ptmp.obtenirPitch(), ptmp.obtenirRoll());
		}
		break;
	case tkId:
		p1=m_tokenActual.obtenirCadena();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkAssignacio) {
			posarErrorParsing(INT_ERROR_ESPERA_ASSIGNACIO);
			return;
		}
		parsePUNT();
		if (m_errorParsejant) return;
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			assignarPunt(p1, m_puntActual);
		}
		break;
	case tkTraslladar:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkId) {
			posarErrorParsing(INT_ERROR_ESPERA_IDENTIFICADOR);
			return;
		}
		p1=m_tokenActual.obtenirCadena();
		if (!m_generantEtiquetes) {
			if (!existeixPunt(m_tokenActual.obtenirCadena(), &ptmp)) {
				posarErrorParsing(INT_ERROR_PUNT_NODEFINIT);
				return;
			}
		}
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkId) {
			posarErrorParsing(INT_ERROR_ESPERA_IDENTIFICADOR);
			return;
		}
		eix=m_tokenActual.obtenirCadena()[0];
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALORCONSTANT);
			return;
		}
		vtmp=m_tokenActual.obtenirValor();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			//
			// Traslladar el punt P1, les dades del qual estn
			// en ptmp, vtmp unitats en direcci a l'eix
			// indicat per eix
			//
			switch (eix) {
			case 'X':
				ptmp.assignarX(ptmp.obtenirX()+(int)vtmp);
				break;
			case 'Y':
				ptmp.assignarY(ptmp.obtenirY()+(int)vtmp);
				break;
			case 'Z':
				ptmp.assignarZ(ptmp.obtenirZ()+(int)vtmp);
				break;
			case 'P':
				ptmp.assignarPitch(ptmp.obtenirPitch()+(int)vtmp);
				break;
			case 'R':
				ptmp.assignarRoll(ptmp.obtenirRoll()+(int)vtmp);
				break;
			default:
				break;
			}
			assignarPunt(p1, ptmp);
		
		}
		break;
	case tkExecutarEnRobot:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkId) {
			posarErrorParsing(INT_ERROR_ESPERA_IDENTIFICADOR);
			return;
		}
		p1=m_tokenActual.obtenirCadena();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkValor) {
			posarErrorParsing(INT_ERROR_ESPERA_VALOR);
			return;
		}
		vtmp=m_tokenActual.obtenirValor();
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			m_robot->executarPrograma(p1, (int)vtmp, "#");
		}
		break;
	case tkFinalProg:
		obtenirToken(&m_tokenActual);
		if (m_tokenActual.obtenirToken()!=tkPuntiComa) {
			posarErrorParsing(INT_ERROR_ESPERA_PUNTICOMA);
			return;
		}
		if (!m_generantEtiquetes) {
			if (m_numObjecteActual>=m_objectes.GetSize()-1) 
				return;
			m_numObjecteActual++;
			definirVariablesExternes();
			if (!etiquetaALinia("@INICI", &liniaSalt)) {
				liniaSalt=0;
			}
			saltar(liniaSalt);
		}
		break;
	case tkEtiqueta:
		if (m_generantEtiquetes)  {
			if (!existeixEtiqueta(m_tokenActual.obtenirCadena())) 
				m_etiquetes.Add(CAuxEtiqueta(m_tokenActual.obtenirCadena(), m_numLiniaActual));
			else {
				posarErrorParsing(INT_ERROR_ETIQUETADEFINIDA);
				return;
			}
		}
		break;
	case tkFinal:
		return;
		break;
	default:
		posarErrorParsing(INT_ERROR_ESPERA_COMANDAVALIDA);
		return;
		break;
	}
	parseLLISTACOMANDES();
	if (m_errorParsejant) return;
}

void CScrInterpret::parsePROGRAMA()
{
	parseLLISTACOMANDES();
	if (m_errorParsejant) return;
}

void CScrInterpret::llegirArxiu(CString nomArxiu)
{
	CString tmp;

	m_script.RemoveAll();

	m_nomArxiu=nomArxiu;
	m_fitxer = new CFile;
	if (!m_fitxer->Open(nomArxiu, CFile::modeRead)) {
		delete m_fitxer;
		return;
	}
	m_arxiu = new CArchive(m_fitxer, CArchive::load);

	while (m_arxiu->ReadString(tmp)) 
		m_script.Add(tmp);
	
	m_arxiu->Close();
	m_fitxer->Close();
}


bool CScrInterpret::existeixEtiqueta(CString cadena)
{
	int i;
	for (i=0;i<m_etiquetes.GetSize();i++)
		if (m_etiquetes[i].obtenirCadena()==cadena) return true;
	return false;
}

void CScrInterpret::definirVariablesExternes()
{
	int i;
	
	if (m_numObjecteActual>=m_objectes.GetSize()) return;

	m_nomObjecte=m_objectes[m_numObjecteActual].m_nom;
	m_area=m_objectes[m_numObjecteActual].m_area;
	m_compacitat=m_objectes[m_numObjecteActual].m_compacitat;
	m_dMax=m_objectes[m_numObjecteActual].m_dMax;
	m_dMin=m_objectes[m_numObjecteActual].m_dMin;
	m_orientacio=m_objectes[m_numObjecteActual].m_orientacio;
	m_perimetre=m_objectes[m_numObjecteActual].m_perimetre;
	m_xAgafada=m_objectes[m_numObjecteActual].m_xCentre;
	m_yAgafada=m_objectes[m_numObjecteActual].m_yCentre;	
	CAuxPuntRobot ptmp("PUNTAPROXIMACIO", m_xAgafada, m_yAgafada, ROB_Z_DEFECTE, ROB_PITCH_DEFECTE, (int)m_orientacio);

	for (i=0;i<m_punts.GetSize();i++) {
		if (m_punts[i].obtenirCadena()=="PUNTAPROXIMACIO") {
			m_punts[i]=ptmp;
			return ;
		}
	}
	m_punts.Add(ptmp);
}

bool CScrInterpret::existeixPunt(CString nom, CAuxPuntRobot * p)
{
	int i;
	for (i=0;i<m_punts.GetSize();i++) {
		if (m_punts[i].obtenirCadena()==nom) {
			*p=m_punts[i];
			return true;
		}
	}
	return false;
}

bool CScrInterpret::etiquetaALinia(CString etiqueta, int * linia)
{
	int i;
	for (i=0;i<m_etiquetes.GetSize();i++) {
		if (m_etiquetes[i].obtenirCadena()==etiqueta) {
			(*linia)=m_etiquetes[i].obtenirNumLinia();
			return true;
		}
	}
	return false;
}

void CScrInterpret::saltar(int numlinia)
{
	m_numLiniaActual=numlinia;
	m_posicioActual=0;
	m_liniaActual=m_script[m_numLiniaActual];
}

void CScrInterpret::assignarPunt(CString p1, CAuxPuntRobot p2)
{
	bool existeix=false;
	int i, pos1;

	// Es mira si el punt desti est definit
	for (i=0;i<m_punts.GetSize();i++)
		if (m_punts[i].obtenirCadena()==p1) {
			existeix=true;
			pos1=i;
		}
	// Si no existeix, el definim posant-hi la info del punt
	// origen, inclosa la cadena
	if (!existeix) 
		pos1=m_punts.Add(p2);
	else
	// Si existeix, hi copiam la info del punt origen, inclosa la
	// cadena
		m_punts[pos1]=p2;
	// Finalment, hi posam la cadena pertinent.
	m_punts[pos1].assignarCadena(p1);
	return;
}

void CScrInterpret::executar(CString nomArxiu)
{
	char missatge[200];

	// Si no hi ha cap objecte a la vista, l'script no
	// ha de fer res.
	// Si no es fes aquesta comprovaci, l'script donaria un
	// error de "punt no definit" pel punt puntAproximacio.
	if (m_objectes.GetSize()==0) {
		AfxMessageBox("Heu intentat executar l'script, per no s'ha detectat cap objecte.");
		return;
	}
	m_tinicial=(float)clock();
	iniciarEstat();
	llegirArxiu(nomArxiu);
	m_generantEtiquetes=true;
	saltar(0);
	parsePROGRAMA();
	if (m_errorParsejant) {
		sprintf(missatge, "Error en l'arxiu %s , a la linia %d carcter %d : %s", m_nomArxiu, m_liniaError, m_posicioError, m_textError);
		AfxMessageBox(missatge);
		return;
	}
	saltar(0);
	m_numObjecteActual=0;
	definirVariablesExternes();
	m_generantEtiquetes=false;
	parsePROGRAMA();
	if (m_errorParsejant) {
		sprintf(missatge, "Error en l'arxiu %s , a la linia %d carcter %d : %s", m_nomArxiu, m_liniaError, m_posicioError, m_textError);
		AfxMessageBox(missatge);
	}
}

void CScrInterpret::iniciarEstat()
{
	m_errorParsejant=false;
	m_numLiniaActual=0;
	m_posicioActual=0;
	m_liniaActual=_T("");
	m_textError=_T("");
	m_script.RemoveAll();
	m_etiquetes.RemoveAll();
	m_pila.RemoveAll();
	m_punts.RemoveAll();
	m_generantEtiquetes=true;
	m_numObjecteActual=0;
	m_puntActual=CAuxPuntRobot("RES", 0, 0, 0, 0, 0);
	m_valorActual=0;
	m_condicioActual=false;
	m_estat=1;
}


void CScrInterpret::executarContinu(CString nomArxiu)
{
	char missatge[200];
	int liniaSalt=0;

	// Si no hi ha cap objecte a la vista, l'script no
	// ha de fer res.
	// Si no es fes aquesta comprovaci, l'script donaria un
	// error de "punt no definit" pel punt puntAproximacio.
	if (m_objectes.GetSize()==0) {
		AfxMessageBox("Heu intentat executar l'script, per no s'ha detectat cap objecte.");
		return;
	}
	m_tinicial=(float)clock();
	if (m_primeraExecucioContinua) {
		iniciarEstat();
		llegirArxiu(nomArxiu);
		m_generantEtiquetes=true;
		saltar(0);
		parsePROGRAMA();
		if (m_errorParsejant) {
			sprintf(missatge, "Error en l'arxiu %s , a la linia %d carcter %d : %s", m_nomArxiu, m_liniaError, m_posicioError, m_textError);
			AfxMessageBox(missatge);
			return;
		}
		m_primeraExecucioContinua=false;
		saltar(0);
	}
	else { 
		etiquetaALinia("@INICI", &liniaSalt);
		saltar(liniaSalt);
	}
	m_numObjecteActual=0;
	definirVariablesExternes();
	m_generantEtiquetes=false;
	parsePROGRAMA();
	if (m_errorParsejant) {
		sprintf(missatge, "Error en l'arxiu %s , a la linia %d carcter %d : %s", m_nomArxiu, m_liniaError, m_posicioError, m_textError);
		AfxMessageBox(missatge);
	}
}

void CScrInterpret::iniciarExecucioContinua()
{
	m_primeraExecucioContinua=true;
}


void CScrInterpret::posarTempsTranscorregut(float temps)
{
	m_tempsTranscorregut=temps;
}

float CScrInterpret::obtenirTempsTranscorregut()
{
	return m_tempsTranscorregut;
}

void CScrInterpret::actuacioDesdeAnalisiMoviment()
{
	m_hihaMoviment=true;
}

void CScrInterpret::actuacioDesdeExecucioScript()
{
	m_hihaMoviment=false;
}

bool CScrInterpret::esCaracterNumeric(char c)
{
	return ((c>='0' && c<='9') || (c=='.') || (c=='-'));
}
