/////////////////////////////////////////////////////////////////
//
// Arxiu      : ScrInterpret.h
//
// Descripci : Aquesta classe proporciona mtodes per a
//				llegir arxius de script de l'entorn BRAS
//				(format bsc) i executar-los, interaccionant
//				amb un objecte de la classe CDspRobot.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#if !defined(AFX_INTERPRET_H__E6A5FD42_FA7F_11D3_B5EB_00C026C084F2__INCLUDED_)
#define AFX_INTERPRET_H__E6A5FD42_FA7F_11D3_B5EB_00C026C084F2__INCLUDED_

//
// Arxius de capalera
//
#include <time.h>
#include "DspRobot.h"
#include "AuxToken.h"
#include "AuxEtiqueta.h"
#include "AuxPuntRobot.h"
#include "AuxPropietatObjecte.h"
#include "DlgCfgCella.h"

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

//
// Missatges d'error
//
#define INT_ERROR_ESPERA_PUNT "S'esperava la definici directa d'un punt ( X , Y , Z , P , R ) o una variable punt"
#define INT_ERROR_ESPERA_VALORXVALID "S'esperava un valor numric vlid per a la coordenada X d'un punt"
#define INT_ERROR_ESPERA_VALORYVALID "S'esperava un valor numric vlid per a la coordenada Y d'un punt"
#define INT_ERROR_ESPERA_VALORZVALID "S'esperava un valor numric vlid per a la coordenada Z d'un punt"
#define INT_ERROR_ESPERA_VALORPVALID "S'esperava un valor numric vlid per a la rotaci de Pitch relacionada amb el punt"
#define INT_ERROR_ESPERA_VALORRVALID "S'esperava un valor numric vlid per a la coordenada Roll relacionada amb el punt"
#define INT_ERROR_ESPERA_COMA "S'esperava una coma. Recordeu que la coma no pot anar afegida a l'identificador o al nmero"
#define INT_ERROR_ESPERA_TANCAPARENTESI "S'esperava el simbol de tancament de parntesi"
#define INT_ERROR_ESPERA_OBREPARENTESI "S'esperava el simbol d'obertura de parntesi"
#define INT_ERROR_ESPERA_VALOR "S'esperava un valor numric vlid o un identificador vlid"
#define INT_ERROR_CONDICIO_NOVALIDA "La condici no s vlida."
#define INT_ERROR_ESPERA_COMPARADOR "S'esperava un simbol de comparaci"
#define INT_ERROR_ESPERA_COMANDAVALIDA "S'esperava una instrucci vlida."
#define INT_ERROR_ESPERA_IFGOTO "La sentncia condicional sols admet instruccions de salt."
#define INT_ERROR_GOTOETIQUETA "Una etiqueta vlida per a un salt comena necessriament pel smbol @."
#define INT_ERROR_ESPERA_PUNTICOMA "S'esperava un punt i coma."
#define INT_ERROR_ESPERA_IDENTIFICADOR "S'esperava un identificador vlid."
#define INT_ERROR_ESPERA_ASSIGNACIO "S'esperava el smbol d'assignaci '='."
#define INT_ERROR_ESPERA_VALORCONSTANT "S'esperava un valor numric constant."
#define INT_ERROR_ETIQUETADEFINIDA "Ja heu definit una etiqueta amb aquest nom."
#define INT_ERROR_PUNT_NODEFINIT "El punt que intentau assignar no ha estat definit encara."
#define INT_ERROR_OPERACIO_DESCONEGUDA "Operaci aritmtica desconeguda."
#define INT_ERROR_ETIQUETA_NODEFINIDA "Heu intentat fer un salt a una etiqueta no definida."
#define INT_ERROR_ESPERA_PUNTROBOT "S'esperava un punt definible en el robot, de la forma $punt, a on punt s un sencer. Per exemple, $2"
#define INT_ERROR_RETURN_WITHOUT_GOSUB "Heu provat de retornar d'una subrutina que no havieu cridat convenientment."

//
// Definici de tipus
//

// Els objectes de classe CLlistaComandes estan pensats
// per a contenir el llistat del programa.
typedef CArray<CString, CString> CLlistaComandes;
// Els objectes de classe CPilaCridades estan pensats per
// a contenir la pila de cridades que ha realitzat el
// programa bsc mitjanant la sentncia de salt a subrutina.
typedef CArray<int, int> CPilaCridades;

class CScrInterpret  
{
public:
	void actuacioDesdeExecucioScript();
	void actuacioDesdeAnalisiMoviment();
	float obtenirTempsTranscorregut();
	void posarTempsTranscorregut(float temps);
	void iniciarExecucioContinua();
	void executarContinu(CString nomArxiu);
	void executar(CString nomArxiu);
	CLlistaPropietatsObjectes m_objectes;
	void inicialitzar(void *sistema);
	CScrInterpret();
	virtual ~CScrInterpret();

private:
	bool esCaracterNumeric(char c);
	bool m_hihaMoviment;
	CDlgCfgCella * m_cella;
	float m_tfinal;
	float m_tinicial;
	float m_tempsTranscorregut;
	bool m_primeraExecucioContinua;
	CPilaCridades m_pila;
	int m_estat;
	void iniciarEstat();
	void assignarPunt(CString p1, CAuxPuntRobot p2);
	void saltar(int numlinia);
	bool etiquetaALinia(CString etiqueta, int *linia);
	bool m_condicioActual;
	double m_valorActual;
	bool existeixPunt(CString nom, CAuxPuntRobot *p);
	CAuxPuntRobot m_puntActual;
	CString m_nomObjecte;
	int m_perimetre;
	int m_yAgafada;
	int m_xAgafada;
	long m_area;
	int m_dMin;
	int m_dMax;
	double m_compacitat;
	double m_orientacio;

	void definirVariablesExternes();
	int m_numObjecteActual;
	CLlistaPuntsRobot m_punts;
	bool existeixEtiqueta(CString cadena);
	bool m_generantEtiquetes;
	CLlistaEtiquetes m_etiquetes;
	CString m_liniaActual;
	void llegirArxiu(CString nomArxiu);
	CLlistaComandes m_script;
	void parsePROGRAMA();
	void parseLLISTACOMANDES();
	void parseCONDICIO();
	void parseEXPRESSIO();
	double parseVALOR();
	void parseEXPRESSIO2();
	void posarErrorParsing(CString error);
	int m_posicioError;
	int m_liniaError;
	int m_numLiniaActual;
	CAuxToken m_tokenActual;
	int m_posicioActual;
	CString m_textError;
	bool m_errorParsejant;
	CLlistaTokens m_tokens;
	void obtenirToken(CAuxToken *elToken);
	void parsePUNT();
	CArchive * m_arxiu;
	CFile * m_fitxer;
	CString m_nomArxiu;
	CDspRobot * m_robot;
};

#endif // !defined(AFX_INTERPRET_H__E6A5FD42_FA7F_11D3_B5EB_00C026C084F2__INCLUDED_)
