/////////////////////////////////////////////////////////////////
//
// Arxiu      : Sistema.cpp
//
// Descripci : Es el cor de l'aplicaci. Cont tots els objectes 
//              que han d'esser persistents al llarg de l'execuci
//				del programa. Tamb s'encarrega d'inicialitzar-los
//				tots, cosa que fa dins del seu constructor, i de 
//				finalitzar-los, cosa que fa en el destructor.
//				A ms, s'encarrega de realitzar aquelles funcions 
//				d'interacci entre objectes que no tenen cabuda 
//				en cap de les parts implicades. Com ara, la 
//				conversi de sistema de coordenades pantalla a 
//				sistema de coordenades robot.
//
// Autor      : Antoni Burguera i Burguera
//
/////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BRAS.h"
#include "Sistema.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CSistema::CSistema()
{
	//
	// Noms dels arxius de configuraci
	//
	m_arxiuConfiguracioImatge="bras_img.cfg";
	m_arxiuConfiguracioGeneral="bras_gen.cfg";
	m_arxiuConfiguracioRobot="bras_rob.cfg";
	m_arxiuConfiguracioCella="bras_cel.cfg";
	m_arxiuConfiguracioCriteris="bras_cri.cfg";

	// 
	// Inicialitzaci dels objectes
	//
	m_imgconfig.inicialitzar(this);
	m_genconfig.inicialitzar(this);
	m_robconfig.inicialitzar(this);
	m_camara.inicialitzar(this);
	m_robot.inicialitzar(this);
	m_identificacio.inicialitzar(this);
	m_cellconfig.inicialitzar(this);
	m_histograma.inicialitzar(this);
	m_terminal.inicialitzar(this);
	m_interpret.inicialitzar(this);
	m_analisi.inicialitzar(this);
	m_creacriteris.inicialitzar(this);
}

CSistema::~CSistema()
{
	m_imgconfig.finalitzar();
	m_genconfig.finalitzar();
	m_robconfig.finalitzar();
	m_cellconfig.finalitzar();
	m_camara.finalitzar();
	m_creacriteris.finalitzar();
}

void CSistema::pantallaARobot(int *x, int *y, double *o)
{
	double x0, y0, x1, y1, a, b, a1, b1;
	double xpprima, ypprima, xrprima, yrprima;
	double angle, escala;

	x0=-(m_cellconfig.m_xp1);
	y0=m_cellconfig.m_yp1;
	x1=-(m_cellconfig.m_xp2);
	y1=m_cellconfig.m_yp2;

	a=m_cellconfig.m_xr1;
	b=m_cellconfig.m_yr1;
	a1=m_cellconfig.m_xr2;
	b1=m_cellconfig.m_yr2;

	angle=(double)(atan((y1-y0)/(x1-x0))-atan((b1-b)/(a1-a)));

	xpprima=-(*x);
	ypprima=(*y);

	escala=sqrt(((a1-a)*(a1-a))+((b1-b)*(b1-b)));
	escala=escala/sqrt(((x1-x0)*(x1-x0))+((y1-y0)*(y1-y0)));

	xrprima=(int)(escala*(xpprima*cos(angle) + ypprima*sin(angle) - x0 * cos(angle) - y0 * sin(angle)));
	yrprima=(int)(escala*(-xpprima*sin(angle) + ypprima*cos(angle) + x0*sin(angle) - y0*cos(angle)));

	*x=(int)(xrprima+a);
	*y=(int)(yrprima+b);
	*o=(((((3.14159-(*o)-angle)-atan2(*y, *x))*360/(2*3.14159))-270)*10);
}

void CSistema::calcularAgafada(CAuxObjecte * obj, int *xc, int *yc, double *orien, int xmn, int ymn)
{
	int dx, dy, xr, yr, xo, yo;
	double distancia, rotacio, alfa;
	CPoint p;
	
	rotacio=(obj->obtenirRotacioAgafada())*2*3.14159/360;
	distancia=sqrt((xmn-(*xc))*(xmn-(*xc))+(ymn-(*yc))*(ymn-(*yc)));
	alfa=rotacio-(*orien);

	dx=(int)(((obj->obtenirDespXAgafada())*distancia)/100);
	dy=(int)(((obj->obtenirDespYAgafada())*distancia)/100);

//	A=-(*xc)*cos(alfa)+(*yc)*sin(alfa)+dx;
//	B=-(*xc)*sin(alfa)-(*yc)*cos(alfa)+dy;

	p=traslladar((rotar((CPoint(-(int)distancia, 0)),*orien)), *xc, *yc);
	xmn=p.x;
	ymn=p.y;

	p=traslladar((rotar((traslladar((rotar((traslladar(CPoint(xmn, ymn), -(*xc), -(*yc))), alfa)),-dx, -dy)), (*orien))), (*xc), (*yc));
	xr=p.x;
	yr=p.y;
	
	p=traslladar((rotar((traslladar((rotar((traslladar(CPoint((*xc), (*yc)), -(*xc), -(*yc))), alfa)),-dx, -dy)), (*orien))), (*xc), (*yc));
	xo=p.x;
	yo=p.y;

	*xc=xo;
	*yc=yo;

	*orien=atan2((yr-yo), (xr-xo));
}

CPoint CSistema::rotar(CPoint punt, double angle)
{
	return (CPoint((int)(cos(angle)*punt.x - sin(angle)*punt.y),(int)(sin(angle)*punt.x + cos(angle)*punt.y)));
}

CPoint CSistema::traslladar(CPoint punt, int dx, int dy)
{
	return (CPoint((int)(punt.x+dx), (int)(punt.y+dy)));
}

void CSistema::realitzarProcesImatge(CPriImatge * imatge)
{
	UINT i;
	// Es realitza el threshold
	imatge->threshold(9999, m_imgconfig.m_emfatitzarHistograma, m_imgconfig.m_ampladaSuavitzat, m_imgconfig.m_minAmuntAvall, m_imgconfig.m_minAmuntAvall);
	
	switch (m_imgconfig.m_formaProcessat) {
	case 0: 
		// Si procs s "Noms threshold", ja est
		return;
		break;
	case 1:
		// Si procs s "Noms erosi"
		// es fa l'erosi i ja est
		for (i=0;i<m_imgconfig.m_numErosions;i++)
			imatge->erosionar(m_imgconfig.m_tamanyMascara);
		return;
		break;
	case 2:
		// Si procs s "Noms dilataci"
		// es fa la dilataci i ja est
		for (i=0;i<m_imgconfig.m_numDilatacions;i++)
			imatge->dilatar(m_imgconfig.m_tamanyMascara);
		return;
		break;
	case 3:
		// Si procs s Erosi + Dilataci
		for (i=0;i<m_imgconfig.m_numErosions;i++)
			imatge->erosionar(m_imgconfig.m_tamanyMascara);
		for (i=0;i<m_imgconfig.m_numDilatacions;i++)
			imatge->dilatar(m_imgconfig.m_tamanyMascara);
		return;
		break;
	case 4:
		// Si procs s Dilataci + Erosi
		for (i=0;i<m_imgconfig.m_numDilatacions;i++)
			imatge->dilatar(m_imgconfig.m_tamanyMascara);
		for (i=0;i<m_imgconfig.m_numErosions;i++)
			imatge->erosionar(m_imgconfig.m_tamanyMascara);
		return;
		break;
	}
	return;
}
